@@ -1,6 +1,8 @@
 #  File src/library/base/R/character.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,36 +16,32 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
-strsplit <- function(x, split, extended = TRUE, fixed = FALSE, perl = FALSE)
-    .Internal(strsplit(x, as.character(split), as.logical(extended),
-                       as.logical(fixed), as.logical(perl)))
-
 substr <- function(x, start, stop)
 {
     if(!is.character(x)) x <- as.character(x)
     .Internal(substr(x, as.integer(start), as.integer(stop)))
 }
 
-substring <- function(text,first,last=1000000)
+substring <- function(text, first, last=1000000L)
 {
-    if(!is.character(text)) x <- as.character(text)
+    if(!is.character(text)) text <- as.character(text)
     n <- max(lt <- length(text), length(first), length(last))
     if(lt && lt < n) text <- rep(text, length.out = n)
-    substr(text, first, last)
+    .Internal(substr(text, as.integer(first), as.integer(last)))
 }
 
 `substr<-` <- function(x, start, stop, value)
     .Internal(`substr<-`(x, as.integer(start), as.integer(stop), value))
 
-`substring<-` <- function(text, first, last=1000000, value)
-    `substr<-`(text, first, last, value)
+`substring<-` <- function(text, first, last=1000000L, value)
+    .Internal(`substr<-`(text, as.integer(first), as.integer(last), value))
 
 abbreviate <-
-    function(names.arg, minlength = 4, use.classes = TRUE, dot = FALSE,
-             method = c("left.kept", "both.sides"))
+    function(names.arg, minlength = 4L, use.classes = TRUE, dot = FALSE,
+             strict = FALSE, method = c("left.kept", "both.sides"))
 {
     ## we just ignore use.classes
-    if(minlength <= 0)
+    if(minlength <= 0L)
 	return(rep.int("", length(names.arg)))
     ## need to remove leading/trailing spaces before we check for dups
     ## This is inefficient but easier than modifying do_abbrev (=> FIXME !)
@@ -52,32 +50,37 @@ abbreviate <-
     old <- names.arg
     if(any(dups))
 	names.arg <- names.arg[!dups]
-    method <- match.arg(method)
-    if(method == "both.sides")
-        ## string reversion: FIXME reverse .Internal(abbreviate(.))
-	chRev <- function(x)
-	    sapply(lapply(strsplit(x, NULL), rev), paste, collapse="")
-    dup2 <- rep.int(TRUE, length(names.arg))
-    x <- these <- names.arg
-    repeat {
-	ans <- .Internal(abbreviate(these, minlength, use.classes))
-        ## NB: fulfills   max(nchar(ans)) <= minlength
-	x[dup2] <- ans
-	if(!any(dup2 <- duplicated(x))) break
-	if(method == "both.sides") { ## abbreviate the dupl. ones from the other side:
-	    x[dup2] <- chRev(.Internal(abbreviate(chRev(names.arg[dup2]),
-						  minlength, use.classes)))
+    x <- names.arg
+    if(strict) {
+	x[] <- .Internal(abbreviate(x, minlength, use.classes))
+    } else {
+	method <- match.arg(method)
+	if(method == "both.sides")
+	    ## string reversion: FIXME reverse .Internal(abbreviate(.))
+	    chRev <- function(x)
+		sapply(lapply(strsplit(x, NULL), rev), paste, collapse="")
+	dup2 <- rep.int(TRUE, length(names.arg))
+	these <- names.arg
+	repeat {
+	    ans <- .Internal(abbreviate(these, minlength, use.classes))
+	    ## NB: fulfills   max(nchar(ans)) <= minlength
+	    x[dup2] <- ans
 	    if(!any(dup2 <- duplicated(x))) break
+	    if(method == "both.sides") { ## abbreviate the dupl. ones from the other side:
+		x[dup2] <- chRev(.Internal(abbreviate(chRev(names.arg[dup2]),
+						      minlength, use.classes)))
+		if(!any(dup2 <- duplicated(x))) break
+	    }
+	    minlength <- minlength+1
+	    dup2 <- dup2 | match(x, x[dup2], 0L)
+	    these <- names.arg[dup2]
 	}
-	minlength <- minlength+1
-	dup2 <- dup2 | match(x, x[dup2], 0)
-	these <- names.arg[dup2]
     }
     if(any(dups))
 	x <- x[match(old,names.arg)]
-    if(dot) { # add "." where we did abbreviate:
-        chgd <- x != old
-	x[chgd] <- paste(x[chgd],".",sep = "")
+    if(dot) {			    # add "." where we did abbreviate:
+	chgd <- x != old
+	x[chgd] <- paste0(x[chgd],".")
     }
     names(x) <- old
     x
@@ -116,15 +119,15 @@ sQuote <- function(x) {
         if(identical(q, TRUE)) {
             li <- l10n_info()
             if(li$"UTF-8") q <- "UTF-8"
-            if(!is.null(li$codepage) && li$codepage > 0) {
+            if(!is.null(li$codepage) && li$codepage > 0L) {
                 ## we can't just use iconv, as that seems to think
                 ## it is in latin1 in CP1252
-                if(li$codepage >= 1250 && li$codepage <= 1258
-                   || li$codepage == 874) {
+                if(li$codepage >= 1250L && li$codepage <= 1258L
+                   || li$codepage == 874L) {
                     before <- "\x91"; after <- "\x92"
                 } else {
                     z <- iconv(c("\xe2\x80\x98", "\xe2\x80\x99"), "UTF-8", "")
-                    before <- z[1]; after <- z[2]
+                    before <- z[1L]; after <- z[2L]
                 }
             }
         }
@@ -134,14 +137,14 @@ sQuote <- function(x) {
         if(identical(q, "UTF-8")) {
             before <- "\xe2\x80\x98"; after <- "\xe2\x80\x99"
         }
-        if(is.character(q) && length(q) >= 4) {
-            before <- q[1]; after <- q[2]
+        if(is.character(q) && length(q) >= 4L) {
+            before <- q[1L]; after <- q[2L]
         }
         ## we do not want these strings marked as in the encoding
         ## R was built under
         Encoding(before) <- Encoding(after) <- "unknown"
     }
-    paste(before, x, after, sep = "")
+    paste0(before, x, after)
 }
 
 dQuote <- function(x) {
@@ -151,13 +154,13 @@ dQuote <- function(x) {
         if(identical(q, TRUE)) {
             li <- l10n_info()
             if(li$"UTF-8") q <- "UTF-8"
-            if(!is.null(li$codepage) && li$codepage > 0) {
-                if(li$codepage >= 1250 && li$codepage <= 1258
-                    || li$codepage == 874) {
+            if(!is.null(li$codepage) && li$codepage > 0L) {
+                if(li$codepage >= 1250L && li$codepage <= 1258L
+                    || li$codepage == 874L) {
                     before <- "\x93"; after <- "\x94"
                 } else {
                     z <- iconv(c("\xe2\x80\x9c", "\xe2\x80\x9d"), "UTF-8", "")
-                    before <- z[1]; after <- z[2]
+                    before <- z[1L]; after <- z[2L]
                 }
             }
         }
@@ -167,10 +170,16 @@ dQuote <- function(x) {
         if(identical(q, "UTF-8")) {
             before <- "\xe2\x80\x9c"; after <- "\xe2\x80\x9d"
         }
-        if(is.character(q) && length(q) >= 4) {
-            before <- q[3]; after <- q[4]
+        if(is.character(q) && length(q) >= 4L) {
+            before <- q[3L]; after <- q[4L]
         }
         Encoding(before) <- Encoding(after) <- "unknown"
     }
-    paste(before, x, after, sep = "")
+    paste0(before, x, after)
+}
+
+strtoi <-
+function(x, base = 0L)
+{
+    .Internal(strtoi(as.character(x), as.integer(base)))
 }
