@@ -1,3 +1,21 @@
+#  File src/library/base/R/frametools.R
+#  Part of the R package, http://www.R-project.org
+#
+#  Copyright (C) 1995-2012 The R Core Team
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 subset.data.frame <- function (x, subset, select, drop = FALSE, ...)
 {
     if(missing(subset))
@@ -11,7 +29,7 @@ subset.data.frame <- function (x, subset, select, drop = FALSE, ...)
     if(missing(select))
 	vars <- TRUE
     else {
-	nl <- as.list(1:ncol(x))
+	nl <- as.list(seq_along(x))
 	names(nl) <- names(x)
 	vars <- eval(substitute(select), nl, parent.frame())
     }
@@ -30,7 +48,7 @@ subset.matrix <- function(x, subset, select, drop = FALSE, ...)
     if(missing(select))
 	vars <- TRUE
     else {
-	nl <- as.list(1:ncol(x))
+	nl <- as.list(1L:ncol(x))
 	names(nl) <- colnames(x)
 	vars <- eval(substitute(select), nl, parent.frame())
     }
@@ -39,7 +57,7 @@ subset.matrix <- function(x, subset, select, drop = FALSE, ...)
     x[subset & !is.na(subset), vars, drop = drop]
 }
 
-### Notice use of "illegal" variable name for the first argument
+### Notice use of non-syntactic variable name for the first argument
 ### This used to be "x", but then you couldn't create a variable
 ### called "x"...
 
@@ -53,8 +71,8 @@ transform.data.frame <- function (`_data`, ...)
 	`_data`[inx[matched]] <- e[matched]
 	`_data` <- data.frame(`_data`)
     }
-    if (!all(matched))
-	data.frame(`_data`, e[!matched])
+    if (!all(matched))  # add as separate arguments to get replication
+	do.call("data.frame", c(list(`_data`), e[!matched]))
     else `_data`
 }
 
