@@ -1,6 +1,8 @@
 #  File src/library/base/R/message.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -62,7 +64,7 @@ function(..., domain = NULL, appendLF = TRUE)
             args <- .Internal(gettext(domain, unlist(args)))
         paste(args, collapse = "")
     } else ""
-    if(appendLF) paste(msg, "\n", sep = "") else msg
+    if(appendLF) paste0(msg, "\n") else msg
 }
 
 .packageStartupMessage <- function (message, call = NULL)
