@@ -1,6 +1,8 @@
 #  File src/library/base/R/sample.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +18,7 @@
 
 sample <- function(x, size, replace=FALSE, prob=NULL)
 {
-    if(length(x) == 1 && is.numeric(x) && x >= 1) {
+    if(length(x) == 1L && is.numeric(x) && x >= 1) {
 	if(missing(size)) size <- x
 	.Internal(sample(x, size, replace, prob))
     }
@@ -25,3 +27,6 @@ sample <- function(x, size, replace=FALSE, prob=NULL)
 	x[.Internal(sample(length(x), size, replace, prob))]
     }
 }
+
+sample.int  <- function(n, size=n, replace=FALSE, prob=NULL)
+    .Internal(sample(n, size, replace, prob))
