@@ -1,6 +1,8 @@
 #  File src/library/base/R/summary.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -60,9 +62,33 @@ summary.default <-
     value
 }
 
+format.summaryDefault <- function(x, ...)
+{
+    xx <- if(is.numeric(x) || is.complex(x)) zapsmall(x) else x
+    class(xx) <- class(x)[-1]
+    m <- match("NA's", names(x), 0)
+    if(inherits(x, "Date") || inherits(x, "POSIXct")) {
+        if(length(a <- attr(x, "NAs")))
+            c(format(xx, ...), "NA's" = as.character(a))
+        else format(xx)
+    } else if(m && !is.character(x))
+        xx <- c(format(xx[-m], ...), "NA's" = as.character(xx[m]))
+    else format(xx, ...)
+}
+
 print.summaryDefault <- function(x, ...)
 {
     xx <- if(is.numeric(x) || is.complex(x)) zapsmall(x) else x
+    class(xx) <- class(x)[-1] # for format
+    m <- match("NA's", names(xx), 0)
+    if(inherits(x, "Date") || inherits(x, "POSIXct")) {
+        xx <- if(length(a <- attr(x, "NAs")))
+            c(format(xx), "NA's" = as.character(a))
+        else format(xx)
+        print(xx, ...)
+        return(invisible(x))
+    } else if(m && !is.character(x))
+        xx <- c(format(xx[-m]), "NA's" = as.character(xx[m]))
     print.table(xx, ...)
     invisible(x)
 }
@@ -128,9 +154,9 @@ summary.data.frame <-
             nm[i] <- paste(cn, collapse="  ")
             z[[i]] <- sms
         } else {
+            sms <- format(sms, digits = digits) # may add NAs row
             lbs <- format(names(sms))
-            sms <- paste(lbs, ":", format(sms, digits = digits), "  ",
-                         sep = "")
+            sms <- paste0(lbs, ":", sms, "  ")
             lw[i] <- ncw(lbs[1L])
             length(sms) <- nr
             z[[i]] <- sms
@@ -143,7 +169,7 @@ summary.data.frame <-
 		paste(which(is.na(lw)), collapse = ", "))
     blanks <- paste(character(max(lw, na.rm=TRUE) + 2L), collapse = " ")
     pad <- floor(lw - ncw(nm)/2)
-    nm <- paste(substring(blanks, 1, pad), nm, sep = "")
+    nm <- paste0(substring(blanks, 1, pad), nm)
     dimnames(z) <- list(rep.int("", nr), nm)
     attr(z, "class") <- c("table") #, "matrix")
     z
