@@ -1,16 +1,36 @@
-table <- function (..., exclude = c(NA, NaN),
+#  File src/library/base/R/table.R
+#  Part of the R package, http://www.R-project.org
+#
+#  Copyright (C) 1995-2012 The R Core Team
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
+table <- function (..., exclude = if (useNA=="no") c(NA, NaN),
+                   useNA = c("no", "ifany", "always"),
 		   dnn = list.names(...), deparse.level = 1)
 {
     list.names <- function(...) {
-	l <- as.list(substitute(list(...)))[-1]
+	l <- as.list(substitute(list(...)))[-1L]
 	nm <- names(l)
 	fixup <- if (is.null(nm)) seq_along(l) else nm == ""
-	dep <- sapply(l[fixup], function(x)
-	    switch (deparse.level + 1,
-		    "", ## 0
-		    if (is.symbol(x)) as.character(x) else "", ## 1
-		    deparse(x)[1]) ## 2
-		      )
+	dep <- vapply(l[fixup], function(x)
+		      switch(deparse.level + 1,
+			     "", ## 0
+			     if (is.symbol(x)) as.character(x) else "", ## 1
+			     deparse(x, nlines=1)[1L] ## 2
+			     ),
+		      "")
 	if (is.null(nm))
 	    dep
 	else {
@@ -18,22 +38,25 @@ table <- function (..., exclude = c(NA, NaN),
 	    nm
 	}
     }
+    if (!missing(exclude) && is.null(exclude))
+        useNA <- "always"
 
+    useNA <- match.arg(useNA)
     args <- list(...)
-    if (length(args) == 0)
+    if (!length(args))
 	stop("nothing to tabulate")
-    if (length(args) == 1 && is.list(args[[1]])) {
-	args <- args[[1]]
+    if (length(args) == 1L && is.list(args[[1L]])) {
+	args <- args[[1L]]
 	if (length(dnn) != length(args))
 	    dnn <- if (!is.null(argn <- names(args)))
 		 argn
 	    else
-		 paste(dnn[1], 1:length(args), sep = '.')
+		 paste(dnn[1L], seq_along(args), sep = ".")
     }
     # 0L, 1L, etc: keep 'bin' and 'pd' integer - as long as tabulate() requires it
     bin <- 0L
     lens <- NULL
-    dims <- integer(0)
+    dims <- integer()
     pd <- 1L
     dn <- NULL
     for (a in args) {
@@ -42,17 +65,41 @@ table <- function (..., exclude = c(NA, NaN),
 	    stop("all arguments must have the same length")
         cat <-
             if (is.factor(a)) {
-                if (!missing(exclude)) {
-                    ll <- levels(a)
-                    factor(a, levels = ll[!(ll %in% exclude)],
-                           exclude = if(is.null(exclude)) NULL else NA)
-                } else a
-            } else factor(a, exclude = exclude)
+                if (any(is.na(levels(a)))) # Don't touch this!
+                    a
+                else {
+                    ## The logic here is tricky because it tries to do
+                    ## something sensible if both 'exclude' and
+                    ## 'useNA' is set. A non-null setting of 'exclude'
+                    ## sets the excluded levels to missing, which is
+                    ## different from the <NA> factor level. Excluded
+                    ## levels are NOT tabulated, even if 'useNA' is
+                    ## set.
+                    if (is.null(exclude) && useNA != "no")
+                        addNA(a, ifany = (useNA == "ifany"))
+                    else {
+                        if (useNA != "no")
+                            a <- addNA(a, ifany = (useNA == "ifany"))
+                        ll <- levels(a)
+                        a <- factor(a, levels = ll[!(ll %in% exclude)],
+                               exclude = if (useNA == "no") NA)
+                    }
+                }
+            }
+            else {
+                a <- factor(a, exclude = exclude)
+                if (useNA != "no")
+                    addNA(a, ifany = (useNA == "ifany"))
+                else
+                    a
+            }
 
 	nl <- length(ll <- levels(cat))
 	dims <- c(dims, nl)
+        if (prod(dims) > .Machine$integer.max)
+            stop("attempt to make a table with >= 2^31 elements")
 	dn <- c(dn, list(ll))
-	## requiring   all(unique(as.integer(cat)) == 1:nlevels(cat))  :
+	## requiring   all(unique(as.integer(cat)) == 1L:nlevels(cat))  :
 	bin <- bin + pd * (as.integer(cat) - 1L)
 	pd <- pd * nl
     }
@@ -108,12 +155,12 @@ summary.table <- function(object, ...)
     if(n.vars > 1) {
 	m <- vector("list", length = n.vars)
 	relFreqs <- object / n.cases
-	for(k in 1:n.vars)
+	for(k in 1L:n.vars)
 	    m[[k]] <- apply(relFreqs, k, sum)
-	expected <- apply(do.call("expand.grid", m), 1, prod) * n.cases
+	expected <- apply(do.call("expand.grid", m), 1L, prod) * n.cases
 	statistic <- sum((c(object) - expected)^2 / expected)
-	parameter <-
-	    prod(sapply(m, length)) - 1 - sum(sapply(m, length) - 1)
+	lm <- vapply(m, length, 1L)
+	parameter <- prod(lm) - 1L - sum(lm - 1L)
 	y <- c(y, list(statistic = statistic,
 		       parameter = parameter,
 		       approx.ok = all(expected >= 5),
@@ -148,13 +195,16 @@ function(x, digits = max(1, getOption("digits") - 3), ...)
 }
 
 as.data.frame.table <-
-    function(x, row.names = NULL, ..., responseName = "Freq")
+    function(x, row.names = NULL, ..., responseName = "Freq",
+             stringsAsFactors = TRUE)
 {
     x <- as.table(x)
-    ex <- quote(data.frame(do.call("expand.grid", dimnames(x)),
+    ex <- quote(data.frame(do.call("expand.grid",
+                                   c(dimnames(x),
+                                     stringsAsFactors = stringsAsFactors)),
                            Freq = c(x),
                            row.names = row.names))
-    names(ex)[3] <- responseName
+    names(ex)[3L] <- responseName
     eval(ex)
 }
 
@@ -166,15 +216,15 @@ as.table.default <- function(x, ...)
 	return(x)
     else if(is.array(x) || is.numeric(x)) {
 	x <- as.array(x)
-	if(any(dim(x) == 0))
+	if(any(dim(x) == 0L))
 	    stop("cannot coerce into a table")
 	## Try providing dimnames where missing.
 	dnx <- dimnames(x)
 	if(is.null(dnx))
 	    dnx <- vector("list", length(dim(x)))
-	for(i in which(sapply(dnx, is.null)))
+	for(i in which(vapply(dnx, is.null, NA)))
 	    dnx[[i]] <-
-                make.unique(LETTERS[seq.int(from=0, length = dim(x)[i]) %% 26 + 1],
+                make.unique(LETTERS[seq.int(from=0, length.out = dim(x)[i]) %% 26 + 1],
                             sep = "")
 	dimnames(x) <- dnx
 	class(x) <- c("table", oldClass(x))
@@ -187,7 +237,7 @@ as.table.default <- function(x, ...)
 prop.table <- function(x, margin = NULL)
 {
     if(length(margin))
-	sweep(x, margin, margin.table(x, margin), "/")
+	sweep(x, margin, margin.table(x, margin), "/", check.margin=FALSE)
     else
 	x / sum(x)
 }
