@@ -1,6 +1,8 @@
 #  File src/library/grDevices/R/unix/dev2bitmap.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -34,13 +36,17 @@ dev2bitmap <- function(file, type="png16m", height = 7, width = 7, res = 72,
     gshelp <- system(paste(gsexe, "-help"), intern=TRUE)
     st <- grep("^Available", gshelp)
     en <- grep("^Search", gshelp)
-    gsdevs <- gshelp[(st+1):(en-1)]
-    devs <- c(strsplit(gsdevs, " "), recursive=TRUE)
-    if(match(type, devs, 0) == 0)
-        stop(gettextf("device '%s' is not available\n", type),
-             gettextf("Available devices are %s",
-                      paste(gsdevs, collapse="\n")),
-             domain = NA)
+    if(!length(st) || !length(en))
+        warning("unrecognized format of gs -help")
+    else {
+        gsdevs <- gshelp[(st+1):(en-1)]
+        devs <- c(strsplit(gsdevs, " "), recursive=TRUE)
+        if(match(type, devs, 0) == 0)
+            stop(gettextf("device '%s' is not available\n", type),
+                 gettextf("Available devices are %s",
+                          paste(gsdevs, collapse="\n")),
+                 domain = NA)
+    }
     if(missing(pointsize)) pointsize <- 1.5*min(width, height)
     tmp <- tempfile("Rbit")
     on.exit(unlink(tmp))
@@ -90,13 +96,17 @@ bitmap <- function(file, type="png16m", height = 7, width = 7, res = 72,
     gshelp <- system(paste(gsexe, "-help"), intern=TRUE)
     st <- grep("^Available", gshelp)
     en <- grep("^Search", gshelp)
-    gsdevs <- gshelp[(st+1):(en-1)]
-    devs <- c(strsplit(gsdevs, " "), recursive=TRUE)
-    if(match(type, devs, 0) == 0)
-        stop(gettextf("device '%s' is not available\n", type),
-             gettextf("Available devices are %s",
-                      paste(gsdevs, collapse="\n")),
-             domain = NA)
+    if(!length(st) || !length(en))
+        warning("unrecognized format of gs -help")
+    else {
+        gsdevs <- gshelp[(st+1):(en-1)]
+        devs <- c(strsplit(gsdevs, " "), recursive=TRUE)
+        if(match(type, devs, 0) == 0)
+            stop(gettextf("device '%s' is not available\n", type),
+                 gettextf("Available devices are %s",
+                          paste(gsdevs, collapse="\n")),
+                 domain = NA)
+    }
     if(missing(pointsize)) pointsize <- 1.5*min(width, height)
     extra <- ""
     if (!is.na(taa)) extra <- paste(" -dTextAlphaBits=", taa, sep="")
