@@ -1,6 +1,8 @@
 #  File src/library/graphics/R/units.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -16,15 +18,15 @@
 
 xinch <- function(x=1, warn.log=TRUE) {
     if(warn.log && par("xlog")) warning("x log scale:  xinch() is non-sense")
-    x * diff(par("usr")[1:2])/par("pin")[1]
+    x * diff(par("usr")[1L:2])/par("pin")[1L]
 }
 yinch <- function(y=1, warn.log=TRUE) {
     if(warn.log && par("ylog")) warning("y log scale:  yinch() is non-sense")
-    y * diff(par("usr")[3:4])/par("pin")[2]
+    y * diff(par("usr")[3:4])/par("pin")[2L]
 }
 
 xyinch <- function(xy=1, warn.log=TRUE) {
     if(warn.log && (par("xlog") || par("ylog")))
 	warning("log scale:  xyinch() is non-sense")
-    u <- par("usr"); xy * c(u[2]-u[1], u[4]-u[3]) / par("pin")
+    u <- par("usr"); xy * c(u[2L]-u[1L], u[4L]-u[3L]) / par("pin")
 }
