@@ -1,6 +1,8 @@
 #  File src/library/grid/R/unit.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -117,7 +119,7 @@ convertNative <- function(unit, dimension="x", type="location") {
 # in the current context
 calcStringMetric <- function(text) {
     # .Call rather than .Call.graphics because it is a one-off calculation
-    metric <- grid.Call("L_stringMetric", text)
+    metric <- grid.Call(L_stringMetric, text)
     names(metric) <- c("ascent", "descent", "width")
     metric
 }
