@@ -1,6 +1,8 @@
 #  File src/library/methods/R/BasicClasses.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,6 +16,12 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
+## a few class name definitions needed elsewhere
+.anyClassName <- structure("ANY", package = "methods")
+.signatureClassName <- structure("signature", package = "methods")
+
+
+
 .InitBasicClasses <- function(envir)
 {
     ## setClass won't allow redefining basic classes,
@@ -50,6 +58,10 @@
 
     .setBaseClass("externalptr", prototype = .newExternalptr(), where = envir); clList <- c(clList, "externalptr")
 
+    .setBaseClass("builtin", prototype = `<-`, where = envir); clList <- c(clList, "builtin")
+
+    .setBaseClass("special", prototype = `if`, where = envir); clList <- c(clList, "special")
+
     ## S4, S3 are basic classes that are used to define methods related to being S4, S3 object
     for(cl in c("S4", "S3")) {
         tmp <- newClassRepresentation(className=cl, prototype = defaultPrototype(), virtual=TRUE, package = "methods")
@@ -248,7 +260,9 @@
                 Classi <- class(obj)
                 defi <- getClassDef(Classi)
                 if(is.null(defi))
-                  stop(gettextf("unnamed argument to initialize() for S3 class must have a class definition; \"%s\" does not", Classi), domain = NA)
+                  stop(gettextf("unnamed argument to initialize() for S3 class must have a class definition; %s does not",
+                                dQuote(Classi)),
+                       domain = NA)
                 if(is(obj, S3ClassP)) {
                     ## eligible to be the S3 part; merge other slots from prototype;
                     ## obj then becomes the object, with its original class as the S3Class
@@ -260,7 +274,7 @@
                     ## the S3Class stays from the prototype
                     .Object <- .mergeAttrs(obj, .Object)
                 }
-                else stop(gettextf("unnamed argument must extend either the S3 class or the class of the data part; not true of class \"%s\"", Classi), domain = NA)
+                else stop(gettextf("unnamed argument must extend either the S3 class or the class of the data part; not true of class %s", dQuote(Classi)), domain = NA)
 
             }
         }
@@ -312,7 +326,10 @@
     setMethod("coerce", c("ANY", "S3"), function (from, to, strict = TRUE)
               {
                   switch(typeof(from),
-                         S4 = stop(gettextf("Class \"%s\" does not have an S3 data part, and so is of type \"S4\"; no S3 equivalent",class(from)), domain = NA),
+                         S4 =
+                         stop(gettextf("Class %s does not have an S3 data part, and so is of type \"S4\"; no S3 equivalent",
+                                       dQuote(class(from))),
+                              domain = NA),
                          .notS4(from) )
               },
               where = envir)
@@ -325,14 +342,16 @@
                       cl <- .class1(from)
                       classDef <- getClass(cl)
                       if(identical(classDef@virtual, TRUE))
-                        stop(gettextf("Class \"%s\" is VIRTUAL; not meaningful to create an S4 object from this class", cl), domain = NA)
+                        stop(gettextf("Class %s is VIRTUAL; not meaningful to create an S4 object from this class",
+                                      dQuote(cl)),
+                             domain = NA)
                       pr <- classDef@prototype
                       value <- new(cl)
                       slots <- classDef@slots
                       if(match(".Data", names(slots), 0L) > 0L) {
                           data <- unclass(from)
                           if(!is(data, slots[[".Data"]]))
-                            stop(gettextf("Object must be a valid data part for class \"%s\"; not true of type \"%s\"", cl, class(data)),
+                            stop(gettextf("Object must be a valid data part for class %s; not true of type %s", dQuote(cl), dQuote(class(data))),
                                  domain = NA)
                           value@.Data <- unclass(from)
                       }
@@ -537,9 +556,9 @@
       assign(".S3MethodsClasses", S3table, envir = where)
     }
     else S3table <- get(".S3MethodsClasses", envir = where)
-    specialTypes <- c("environment", "externalptr", "name", "NULL")
-    specialClasses <- paste(".", specialTypes, sep="")
-    for(i in seq_along(specialTypes)) {
+    specialClasses <- .indirectAbnormalClasses
+    specialTypes <- .AbnormalTypes # only part matching classes used
+    for(i in seq_along(specialClasses)) {
         cl <- specialTypes[[i]]
       ncl <- specialClasses[[i]]
       setClass(ncl, representation(.xData = cl), where = where)
@@ -570,21 +589,43 @@
                   objs <- names(args)
                   hasEnvArg <- length(args) && !all(nzchar(objs))
                   if(hasEnvArg) {
-                      i <- seq_along(args)[!nzchar(objs)]
+                      ii <- seq_along(args)[!nzchar(objs)]
+                      i <- integer()
+                      for(iii in ii) {
+                          if(is(args[[iii]], "environment"))
+                              i <- c(i, iii)
+                      }
                       if(length(i)>1)
                           stop("Can't have more than one unnamed argument as environment")
-                      selfEnv <- args[[i]]
-                      args <- args[-i]
-                      objs <- objs[-i]
-                      if(!is(selfEnv, "environment"))
-                          stop("Unnamed argument to new() must be an environment for the new object")
-                      selfEnv <- as.environment(selfEnv)
+                      if(length(i) == 1) {
+                          selfEnv <- args[[i]]
+                          args <- args[-i]
+                          objs <- objs[-i]
+                          if(!is(selfEnv, "environment"))
+                              stop("Unnamed argument to new() must be an environment for the new object")
+                          selfEnv <- as.environment(selfEnv)
+                      }
+                      ## else, no environment superclasses
+                      else
+                          selfEnv <- new.env()
                   }
                   else
                       selfEnv <- new.env()
-                  for(what in objs)
-                      assign(what, elNamed(args, what), envir = selfEnv)
+                  if(length(objs)) {
+                      ## don't assign locally named slots of subclasses
+                      ClassDef <- getClass(class(.Object))
+                      slots <- slotNames(ClassDef)
+                      localObjs <- is.na(match(objs, slots))
+                      if(any(localObjs)) {
+                          for(what in objs[localObjs])
+                              assign(what, elNamed(args, what), envir = selfEnv)
+                          objs <- objs[!localObjs]
+                          args <- args[!localObjs]
+                      }
+                  }
                   .Object@.xData <- selfEnv
+                  if(length(objs)) # call next method with remaining args
+                      .Object <- do.call(callNextMethod, c(.Object, args))
                   .Object
               }, where = where)
 }
