@@ -1,6 +1,8 @@
 #  File src/library/methods/R/zzz.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,6 +16,10 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
+utils::globalVariables(c(".possibleExtends", ".makeGeneric",
+                         ".newClassRepresentation", ".classGeneratorFunction",
+                         ".mergeClassDefSlots", "..mergeClassDefSlots"))
+
 ..First.lib  <-
   ## Initialize the methods package.  Called from .onLoad, first
   ## during INSTALL, when saved will be FALSE, at which time
@@ -21,7 +27,7 @@
   function(libname, pkgname, where)
 {
     if(missing(where)) {
-        where <- match(paste("package:", pkgname, sep=""), search())
+        where <- match(paste0("package:", pkgname), search())
         if(is.na(where)) {
             warning(gettextf("not a package name: %s", sQuote(pkgname)),
                     domain = NA)
@@ -43,7 +49,7 @@
         .noMlistsFlag <<- (is.character(mopt) && all(mopt != "YES"))
         if(!.noMlistsFlag)
             cat("Initializing with support for old-style methods list objects\n")
-        cat("initializing class and method definitions ...")
+        cat("initializing class and method definitions ...\n")
         on.exit(assign(".saveImage", NA, envir = where))
         ## set up default prototype (uses .Call so has be at load time)
         assign(".defaultPrototype",
@@ -60,12 +66,13 @@
         assign(".classEnv", ..classEnv, envir = where)
         assign("makeGeneric", .makeGeneric, envir = where)
         assign("newClassRepresentation", .newClassRepresentation, envir = where)
+        assign("classGeneratorFunction", .classGeneratorFunction, envir = where)
         assign(".mergeClassDefSlots", ..mergeClassDefSlots, envir = where)
         assign(".addToMetaTable", ..addToMetaTable, envir = where)
         assign(".extendsForS3", ..extendsForS3, envir = where)
         .makeBasicFuns(where)
         rm(.makeGeneric, .newClassRepresentation, .possibleExtends,
-           ..mergeClassDefSlots, envir = where)
+           ..mergeClassDefSlots, .classGeneratorFunction, envir = where)
         .InitMethodDefinitions(where)
         .InitShowMethods(where)
         assign(".isPrototype", ..isPrototype, envir = where)
@@ -154,4 +161,4 @@
 ## cat("Saving namespace image ...\n")
 
 ## want ASCII quotes, not fancy nor translated ones
-.dQ <- function (x) paste('"', x, '"', sep = '')
+.dQ <- function (x) paste0('"', x, '"')
