@@ -1,6 +1,8 @@
 #  File src/library/stats/R/StructTS.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 2002-12 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -19,49 +21,52 @@ StructTS <- function(x, type = c("level", "trend", "BSM"),
 {
     KalmanLike2 <- function (y, mod, nit = 0)
     {
-        x <- .Call(R_KalmanLike, y, mod$Z, mod$a, mod$P, mod$T, mod$V,
+        x <- .Call(C_KalmanLike, y, mod$Z, mod$a, mod$P, mod$T, mod$V,
                    mod$h, mod$Pn, as.integer(nit), FALSE, fast=TRUE)
         0.5 * sum(x)/length(y)
     }
     makeLevel <- function(x)
     {
-        T <- matrix(1, 1, 1)
-        Z <- 1
-        xm <- if(is.na(x[1])) mean(x, na.rm = TRUE) else x[1]
+        T <- matrix(1., 1L, 1L)
+        Z <- 1.
+        xm <- if(is.na(x[1L])) mean(x, na.rm = TRUE) else x[1L]
         if(is.na(xm)) stop("the series is entirely NA")
         a <- xm
-        P <- Pn <- matrix(0, 1, 1)
-        h <- 1
+        P <- Pn <- matrix(0., 1L, 1L)
+        h <- 1.0
         V <- diag(1)
         return(list(Z=Z, a=a, P=P, T=T, V=V, h=h, Pn=Pn))
     }
     makeTrend <- function(x)
     {
-        T <- matrix(c(1,0,1,1), 2, 2)
-        Z <- c(1, 0)
-        xm <- if(is.na(x[1])) mean(x, na.rm = TRUE) else x[1]
+        T <- matrix(c(1.,0.,1.,1.), 2L, 2L)
+        Z <- c(1., 0.)
+        xm <- if(is.na(x[1L])) mean(x, na.rm = TRUE) else x[1L]
         if(is.na(xm)) stop("the series is entirely NA")
         a <- c(xm, 0)
-        P <- Pn <- matrix(0, 2, 2)
-        h <- 1
+        P <- Pn <- matrix(0., 2L, 2L)
+        h <- 1.0
         V <- diag(2)
         return(list(Z=Z, a=a, P=P, T=T, V=V, h=h, Pn=Pn))
     }
     makeBSM <- function(x, nf)
     {
-        if(nf <= 1) stop("frequency must be a positive integer for BSM")
-        T <- matrix(0, nf + 1, nf + 1)
-        T[1:2, 1:2] <- c(1, 0, 1, 1)
-        T[3, ] <- c(0, 0, rep(-1, nf - 1))
-        ind <- 3:nf
-        T[cbind(ind+1, ind)] <- 1
-        Z <- c(1, 0, 1, rep(0, nf - 2))
-        xm <- if(is.na(x[1])) mean(x, na.rm = TRUE) else x[1]
+        ## See Harvey (1993, p.143)
+        if(nf <= 1L) stop("frequency must be a positive integer >= 2 for BSM")
+        T <- matrix(0., nf + 1L, nf + 1L)
+        T[1L:2L, 1L:2L] <- c(1, 0, 1, 1)
+        T[3L, ] <- c(0, 0, rep(-1, nf - 1L))
+        if(nf >= 3L) {
+            ind <- 3:nf
+            T[cbind(ind+1L, ind)] <- 1
+        }
+        Z <- c(1., 0., 1., rep(0., nf - 2L))
+        xm <- if(is.na(x[1L])) mean(x, na.rm = TRUE) else x[1L]
         if(is.na(xm)) stop("the series is entirely NA")
         a <- c(xm, rep(0, nf))
-        P <- Pn <- matrix(0, nf+1, nf+1)
-        h <- 1
-        V <- diag(c(1, 1, 1, rep(0, nf-2)))
+        P <- Pn <- matrix(0., nf+1L, nf+1L)
+        h <- 1.
+        V <- diag(c(1., 1., 1., rep(0., nf-2L)))
         return(list(Z=Z, a=a, P=P, T=T, V=V, h=h, Pn=Pn))
     }
     getLike <- function(par)
@@ -69,23 +74,21 @@ StructTS <- function(x, type = c("level", "trend", "BSM"),
         p <- cf
         p[mask] <- par
         if(all(p == 0)) return(1000)
-        Z$V[cbind(1:np, 1:np)] <- p[-(np+1)]*vx
+        Z$V[cbind(1L:np, 1L:np)] <- p[-(np+1L)]*vx
         Z$h <- p[np+1]*vx
         Z$P[] <- 1e6*vx
         Z$a <- a0
-        res <- KalmanLike2(y, Z, -1)
-#        print(c(res, p))
-        res
+        KalmanLike2(y, Z, -1.0)
     }
 
     series <- deparse(substitute(x))
-    if(NCOL(x) > 1)
+    if(NCOL(x) > 1L)
         stop("only implemented for univariate time series")
     x <- as.ts(x)
     if(!is.numeric(x))
         stop("'x' must be numeric")
     storage.mode(x) <- "double"
-    if(is.na(x[1]))
+    if(is.na(x[1L]))
         stop("the first value of the time series must not be missing")
     type <- if(missing(type)) if(frequency(x) > 1) "BSM" else "trend"
     else match.arg(type)
@@ -99,8 +102,8 @@ StructTS <- function(x, type = c("level", "trend", "BSM"),
                 )
     a0 <- Z$a
     vx <- var(x, na.rm=TRUE)/100
-    np <- switch(type, "level" = 1, "trend" = 2, "BSM" = 3)
-    if (is.null(fixed)) fixed <- rep(NA_real_, np+1)
+    np <- switch(type, "level" = 1L, "trend" = 2L, "BSM" = 3L)
+    if (is.null(fixed)) fixed <- rep(NA_real_, np+1L)
     mask <- is.na(fixed)
     if(!any(mask)) stop("all parameters were fixed")
     cf <- fixed/vx
@@ -108,14 +111,14 @@ StructTS <- function(x, type = c("level", "trend", "BSM"),
 
     y <- x
     res <- optim(init[mask], getLike, method = "L-BFGS-B",
-                 lower = rep(0, np+1), upper = rep(Inf, np+1),
+                 lower = rep(0, np+1L), upper = rep(Inf, np+1L),
                  control = optim.control)
         if(res$convergence > 0)
             warning("possible convergence problem: optim gave code=",
                     res$convergence, " ", res$message)
     coef <- cf
     coef[mask] <- res$par
-    Z$V[cbind(1:np, 1:np)] <- coef[1:np]*vx
+    Z$V[cbind(1L:np, 1L:np)] <- coef[1L:np]*vx
     Z$h <- coef[np+1]*vx
     Z$P[] <- 1e6*vx
     Z$a <- a0
@@ -130,11 +133,11 @@ StructTS <- function(x, type = c("level", "trend", "BSM"),
                  "BSM" = c("level", "slope", "sea")
                  )
     states <- z$states
-    if(type == "BSM") states <- states[, 1:3]
+    if(type == "BSM") states <- states[, 1L:3L]
     dimnames(states) <- list(time(x), cn)
-    states <- ts(states, start = xtsp[1], frequency = nf)
+    states <- ts(states, start = xtsp[1L], frequency = nf)
 
-    coef <- coef*vx
+    coef <- pmax(coef*vx, 0) # computed values just below 0 are possible
     names(coef) <- switch(type,
                           "level" = c("level", "epsilon"),
                           "trend" = c("level", "slope", "epsilon"),
@@ -151,9 +154,9 @@ StructTS <- function(x, type = c("level", "trend", "BSM"),
 
 print.StructTS <- function(x, digits = max(3, getOption("digits") - 3), ...)
 {
-    cat("\nCall:", deparse(x$call, width.cutoff = 75), "", sep = "\n")
+    cat("\nCall:", deparse(x$call, width.cutoff = 75L), "", sep = "\n")
     cat("Variances:\n")
-    print.default(x$coef, print.gap = 2, digits=digits)
+    print.default(x$coef, print.gap = 2L, digits=digits)
     invisible(x)
 }
 
@@ -161,10 +164,10 @@ predict.StructTS <- function(object, n.ahead = 1, se.fit = TRUE, ...)
 {
     xtsp <- object$xtsp
     z <- KalmanForecast(n.ahead, object$model)
-    pred <- ts(z[[1]], start = xtsp[2] + 1/xtsp[3], frequency = xtsp[3])
+    pred <- ts(z[[1L]], start = xtsp[2L] + 1/xtsp[3L], frequency = xtsp[3L])
     if (se.fit) {
-        se <- ts(sqrt(z[[2]]), start = xtsp[2] + 1/xtsp[3],
-                 frequency = xtsp[3])
+        se <- ts(sqrt(z[[2L]]), start = xtsp[2L] + 1/xtsp[3L],
+                 frequency = xtsp[3L])
         return(list(pred=pred, se=se))
     }
     else return(pred)
@@ -178,15 +181,15 @@ tsdiag.StructTS <- function(object, gof.lag = 10, ...)
     rs <- object$residuals
     stdres <- rs
     plot(stdres, type = "h", main = "Standardized Residuals", ylab = "")
-    abline(h = 0)
+    abline(h = 0.)
     acf(object$residuals, plot = TRUE, main = "ACF of Residuals",
         na.action = na.pass)
     nlag <- gof.lag
     pval <- numeric(nlag)
-    for(i in 1:nlag) pval[i] <- Box.test(rs, i, type = "Ljung-Box")$p.value
-    plot(1:nlag, pval, xlab = "lag", ylab = "p value", ylim = c(0,1),
+    for(i in 1L:nlag) pval[i] <- Box.test(rs, i, type = "Ljung-Box")$p.value
+    plot(1L:nlag, pval, xlab = "lag", ylab = "p value", ylim = c(0,1),
          main = "p values for Ljung-Box statistic")
-    abline(h = 0.05, lty = 2, col = "blue")
+    abline(h = 0.05, lty = 2L, col = "blue")
 }
 
 
@@ -196,7 +199,7 @@ tsSmooth.StructTS <- function(object, ...)
 {
     res <- KalmanSmooth(object$data, object$model0, -1)$smooth
     dn <- dim(fitted(object))
-    res <- res[, 1:dn[2], drop = FALSE]
+    res <- res[, 1L:dn[2L], drop = FALSE]
     dimnames(res) <- dimnames(fitted(object))
-    ts(res, start = object$xtsp[1], frequency = object$xtsp[3])
+    ts(res, start = object$xtsp[1L], frequency = object$xtsp[3L])
 }
