@@ -1,6 +1,8 @@
 #  File src/library/stats/R/acf.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1999-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -32,20 +34,21 @@ acf <-
     x <- as.matrix(x)
     if(!is.numeric(x))
         stop("'x' must be numeric")
-    sampleT <- nrow(x)
-    nser <- ncol(x)
+    sampleT <- as.integer(nrow(x))
+    nser <- as.integer(ncol(x))
+    if(is.na(sampleT) || is.na(nser)) stop("sampleT and nser must be ints",
+                                           domain = NA)
     if (is.null(lag.max))
         lag.max <- floor(10 * (log10(sampleT) - log10(nser)))
-    lag.max <- min(lag.max, sampleT - 1)
-    if (lag.max < 0) stop("'lag.max' must be at least 0")
+    lag.max <- as.integer(min(lag.max, sampleT - 1L))
+    if (is.na(lag.max) || lag.max < 0) stop("'lag.max' must be at least 0")
     if(demean)
 	x <- sweep(x, 2, colMeans(x, na.rm = TRUE), check.margin=FALSE)
     lag <- matrix(1, nser, nser)
     lag[lower.tri(lag)] <- -1
-    acf <- array(.C(R_acf,
-                    as.double(x), as.integer(sampleT), as.integer(nser),
-                    as.integer(lag.max), as.integer(type=="correlation"),
-                    acf=double((lag.max+1L) * nser * nser), NAOK = TRUE
+    acf <- array(.C(C_acf, as.double(x), sampleT, nser, lag.max,
+                    as.integer(type == "correlation"),
+                    acf = double((lag.max+1L) * nser * nser), NAOK = TRUE
                     )$acf, c(lag.max + 1L, nser, nser))
     lag <- outer(0:lag.max, lag/x.freq)
     acf.out <- structure(.Data = list(acf = acf, type = type,
@@ -86,7 +89,7 @@ pacf.default <- function(x, lag.max = NULL, plot = TRUE,
         x <- scale(x, TRUE, FALSE)
         acf <- drop(acf(x, lag.max = lag.max, plot = FALSE,
                         na.action = na.action)$acf)
-        pacf <- array(.C(R_uni_pacf,
+        pacf <- array(.C(C_uni_pacf,
                          as.double(acf),
                          pacf = double(lag.max),
                          as.integer(lag.max))$pacf,
@@ -119,27 +122,27 @@ plot.acf <-
               ...)
 {
     ci.type <- match.arg(ci.type)
-    if((nser <- ncol(x$lag)) < 1) stop("x$lag must have at least 1 column")
+    if((nser <- ncol(x$lag)) < 1L) stop("x$lag must have at least 1 column")
     if (is.null(ylab))
         ylab <- switch(x$type,
                        correlation = "ACF",
                        covariance = "ACF (cov)",
                        partial = "Partial ACF")
     if (is.null(snames <- x$snames))
-        snames <- paste("Series ", if (nser == 1) x$series else 1L:nser)
+        snames <- paste("Series ", if (nser == 1L) x$series else 1L:nser)
 
     with.ci <- ci > 0 && x$type != "covariance"
     with.ci.ma <- with.ci && ci.type == "ma" && x$type == "correlation"
-    if(with.ci.ma && x$lag[1,1,1] != 0) {
+    if(with.ci.ma && x$lag[1L, 1L, 1L] != 0L) {
         warning("can use ci.type=\"ma\" only if first lag is 0")
         with.ci.ma <- FALSE
     }
     clim0 <- if (with.ci) qnorm((1 + ci)/2)/sqrt(x$n.used) else c(0, 0)
 
-    Npgs <- 1 ## we will do [ Npgs x Npgs ] pages !
+    Npgs <- 1L ## we will do [ Npgs x Npgs ] pages !
     nr <- nser
-    if(nser > 1) { ## at most m x m (m := max.mfrow)  panels per page
-        sn.abbr <- if(nser > 2) abbreviate(snames) else snames
+    if(nser > 1L) { ## at most m x m (m := max.mfrow)  panels per page
+        sn.abbr <- if(nser > 2L) abbreviate(snames) else snames
 
         if(nser > max.mfrow) {
             ##  We need more than one page: The plots are laid out
@@ -149,7 +152,7 @@ plot.acf <-
             Npgs <- ceiling(nser / max.mfrow)
             nr <- ceiling(nser / Npgs)  # <= max.mfrow
         }
-        opar <- par(mfrow = rep(nr, 2), mar = mar, oma = oma, mgp = mgp,
+        opar <- par(mfrow = rep(nr, 2L), mar = mar, oma = oma, mgp = mgp,
                     ask = ask, xpd = xpd, cex.main = cex.main)
         on.exit(par(opar))
         if(verbose) { # FIXME: message() can be suppressed but not str()
@@ -171,6 +174,7 @@ plot.acf <-
     }
 
     for (I in 1L:Npgs) for (J in 1L:Npgs) {
+        dev.hold()
         ## Page [ I , J ] : Now do   nr x nr  'panels' on this page
         iind <- (I-1)*nr + 1L:nr
         jind <- (J-1)*nr + 1L:nr
@@ -206,6 +210,7 @@ plot.acf <-
             mtext(paste("[",I,",",J,"]"), side=1, line = -0.2, adj=1,
                   col = "dark gray", cex = 1, outer = TRUE)
         }
+        dev.flush()
     }
     invisible()
 }
@@ -217,9 +222,10 @@ ccf <- function(x, y, lag.max = NULL,
     type <- match.arg(type)
     if(is.matrix(x) || is.matrix(y))
         stop("univariate time series only")
-    X <- na.action(ts.intersect(as.ts(x), as.ts(y)))
+    X <- ts.intersect(as.ts(x), as.ts(y))
     colnames(X) <- c(deparse(substitute(x))[1L], deparse(substitute(y))[1L])
-    acf.out <- acf(X, lag.max = lag.max, plot = FALSE, type = type)
+    acf.out <- acf(X, lag.max = lag.max, plot = FALSE, type = type,
+                   na.action = na.action)
     lag <- c(rev(acf.out$lag[-1,2,1]), acf.out$lag[,1,2])
     y   <- c(rev(acf.out$acf[-1,2,1]), acf.out$acf[,1,2])
     acf.out$acf <- array(y, dim=c(length(y),1L,1L))
@@ -231,7 +237,7 @@ ccf <- function(x, y, lag.max = NULL,
     } else return(acf.out)
 }
 
-"[.acf" <- function(x, i, j)
+`[.acf` <- function(x, i, j)
 {
     if(missing(j)) j <- seq_len(ncol(x$lag))
     ii <- if(missing(i)) seq_len(nrow(x$lag))
@@ -256,7 +262,7 @@ print.acf <- function(x, digits=3, ...)
     } else {
         acfs <- format(x$acf, ...)
         lags <- format(x$lag, digits=3)
-        acfs <- array(paste(acfs, " (", lags, ")", sep=""), dim=dim(x$acf))
+        acfs <- array(paste0(acfs, " (", lags, ")"), dim=dim(x$acf))
         dimnames(acfs)  <- list(rep("", nrow(x$lag)), x$snames, x$snames)
         print(acfs, quote=FALSE, ...)
     }
