@@ -1,6 +1,9 @@
 #  File src/library/stats/R/add.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1994-8 W. N. Venables and B. D. Ripley
+#  Copyright (C) 1998-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,6 +17,7 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
+
 ## version to return NA for df = 0, as R did before 2.7.0
 safe_pchisq <- function(q, df, ...)
 {
@@ -74,7 +78,7 @@ add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 	nas <- !is.na(dev)
 	P <- dev
 	P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail=FALSE)
-	aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
+	aod[, c("LRT", "Pr(>Chi)")] <- list(dev, P)
     }
     head <- c("Single term additions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
@@ -199,10 +203,10 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
         df <- aod$Df
         nas <- !is.na(df)
         dev[nas] <- safe_pchisq(dev[nas], df[nas], lower.tail=FALSE)
-        aod[, "Pr(Chi)"] <- dev
+        aod[, "Pr(>Chi)"] <- dev
     } else if(test == "F") {
 	rdf <- object$df.residual
-	aod[, c("F value", "Pr(F)")] <- Fstat(aod, aod$RSS[1L], rdf)
+	aod[, c("F value", "Pr(>F)")] <- Fstat(aod, aod$RSS[1L], rdf)
     }
     head <- c("Single term additions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
@@ -211,7 +215,8 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     aod
 }
 
-add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
+add1.glm <- function(object, scope, scale = 0, test=c("none", "Rao", "LRT",
+                                                 "Chisq", "F"),
 		     x = NULL, k = 2, ...)
 {
     Fstat <- function(table, rdf) {
@@ -225,6 +230,9 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 	P[nnas] <- safe_pf(Fs[nnas], df[nnas], rdf - df[nnas], lower.tail=FALSE)
 	list(Fs=Fs, P=P)
     }
+    test <- match.arg(test)
+    if (test=="Chisq") test <- "LRT"
+
     if(!is.character(scope))
 	scope <- add.scope(object, update.formula(object, scope))
     if(!length(scope))
@@ -232,8 +240,8 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     oTerms <- attr(object$terms, "term.labels")
     int <- attr(object$terms, "intercept")
     ns <- length(scope)
-    dfs <- dev <- numeric(ns+1)
-    names(dfs) <- names(dev) <- c("<none>", scope)
+    dfs <- dev <- score <- numeric(ns+1)
+    names(dfs) <- names(dev) <- names(score) <- c("<none>", scope)
     add.rhs <- paste(scope, collapse = "+")
     add.rhs <- eval(parse(text = paste("~ . +", add.rhs)))
     new.form <- update.formula(object, add.rhs)
@@ -279,6 +287,8 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
                   family=object$family, control=object$control)
     dfs[1L] <- z$rank
     dev[1L] <- z$deviance
+    r <- z$residuals
+    w <- z$weights
     ## workaround for PR#7842. terms.formula may have flipped interactions
     sTerms <- sapply(strsplit(Terms, ":", fixed=TRUE),
                      function(x) paste(sort(x), collapse=":"))
@@ -290,6 +300,11 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 		      family=object$family, control=object$control)
 	dfs[tt] <- z$rank
 	dev[tt] <- z$deviance
+        if (test=="Rao") {
+          ## WLS for score test (comes out as model SS)
+          zz <- glm.fit(X, r, w, offset=offset)
+          score[tt] <- zz$null.deviance - zz$deviance
+        }
     }
     if (scale == 0)
 	dispersion <- summary(object, dispersion = NULL)$dispersion
@@ -307,20 +322,29 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 		      row.names = names(dfs), check.names = FALSE)
     if(all(is.na(aic))) aod <- aod[, -3]
     test <- match.arg(test)
-    if(test == "Chisq") {
+    if(test == "LRT") {
         dev <- pmax(0, loglik[1L] - loglik)
         dev[1L] <- NA
         LRT <- if(dispersion == 1) "LRT" else "scaled dev."
         aod[, LRT] <- dev
         nas <- !is.na(dev)
         dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
-        aod[, "Pr(Chi)"] <- dev
+        aod[, "Pr(>Chi)"] <- dev
+    } else if(test == "Rao") {
+        dev <- pmax(0, score) # roundoff guard
+        dev[1L] <- NA
+        nas <- !is.na(dev)
+        SC <- if(dispersion == 1) "Rao score" else "scaled Rao sc."
+        dev <- dev/dispersion
+        aod[, SC] <- dev
+        dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
+        aod[, "Pr(>Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
             warning(gettextf("F test assumes quasi%s family", fam),
                     domain = NA)
 	rdf <- object$df.residual
-	aod[, c("F value", "Pr(F)")] <- Fstat(aod, rdf)
+	aod[, c("F value", "Pr(>F)")] <- Fstat(aod, rdf)
     }
     head <- c("Single term additions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
@@ -375,7 +399,7 @@ drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
         nas <- !is.na(dev)
         P <- dev
         P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
-        aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
+        aod[, c("LRT", "Pr(>Chi)")] <- list(dev, P)
     }
     head <- c("Single term deletions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
@@ -441,7 +465,7 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
         df <- aod$Df
         nas <- !is.na(df)
         dev[nas] <- safe_pchisq(dev[nas], df[nas], lower.tail=FALSE)
-        aod[, "Pr(Chi)"] <- dev
+        aod[, "Pr(>Chi)"] <- dev
     } else if(test == "F") {
 	dev <- aod$"Sum of Sq"
 	dfs <- aod$Df
@@ -452,7 +476,7 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
 	P <- Fs
 	nas <- !is.na(Fs)
 	P[nas] <- safe_pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
-	aod[, c("F value", "Pr(F)")] <- list(Fs, P)
+	aod[, c("F value", "Pr(>F)")] <- list(Fs, P)
     }
     head <- c("Single term deletions", "\nModel:", deparse(formula(object)),
 	      if(scale > 0) paste("\nscale: ", format(scale), "\n"))
@@ -464,9 +488,11 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
 drop1.mlm <- function(object, scope, ...)
     stop("no 'drop1' method for \"mlm\" models")
 
-drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
+drop1.glm <- function(object, scope, scale = 0, test=c("none", "Rao", "LRT", "Chisq", "F"),
 		      k = 2, ...)
 {
+    test <- match.arg(test)
+    if (test=="Chisq") test <- "LRT"
     x <- model.matrix(object)
 #    iswt <- !is.null(wt <- object$weights)
     n <- nrow(x)
@@ -485,6 +511,7 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     chisq <- object$deviance
     dfs <- numeric(ns)
     dev <- numeric(ns)
+    score <- numeric(ns)
     y <- object$y
     if(is.null(y)) {
         y <- model.response(model.frame(object))
@@ -500,10 +527,22 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 		      family=object$family, control=object$control)
 	dfs[i] <- z$rank
 	dev[i] <- z$deviance
+
+        if (test=="Rao"){
+            r <- z$residuals
+            w <- z$weights
+            ## Approximative refit of full model to residuals using WLS
+            ## Score statistic comes out as (weighted) model SS
+            zz <- glm.fit(x, r, w, offset=object$offset)
+            score[i] <- zz$null.deviance - zz$deviance
+        }
     }
     scope <- c("<none>", scope)
     dfs <- c(object$rank, dfs)
     dev <- c(chisq, dev)
+    if (test=="Rao") {
+      score <- c(NA, score)
+    }
     dispersion <- if (is.null(scale) || scale == 0)
 	summary(object, dispersion = NULL)$dispersion
     else scale
@@ -519,15 +558,22 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     aod <- data.frame(Df = dfs, Deviance = dev, AIC = aic,
 		      row.names = scope, check.names = FALSE)
     if(all(is.na(aic))) aod <- aod[, -3]
-    test <- match.arg(test)
-    if(test == "Chisq") {
+    if(test == "LRT") {
         dev <- pmax(0, loglik - loglik[1L])
         dev[1L] <- NA
         nas <- !is.na(dev)
         LRT <- if(dispersion == 1) "LRT" else "scaled dev."
         aod[, LRT] <- dev
         dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
-        aod[, "Pr(Chi)"] <- dev
+        aod[, "Pr(>Chi)"] <- dev
+    } else if(test == "Rao") {
+        dev <- pmax(0, score) # roundoff guard
+        nas <- !is.na(dev)
+        SC <- if(dispersion == 1) "Rao score" else "scaled Rao sc."
+        dev <- dev/dispersion
+        aod[, SC] <- dev
+        dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
+        aod[, "Pr(>Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
             warning(gettextf("F test assumes 'quasi%s' family", fam),
@@ -542,7 +588,7 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 	P <- Fs
 	nas <- !is.na(Fs)
 	P[nas] <- safe_pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
-	aod[, c("F value", "Pr(F)")] <- list(Fs, P)
+	aod[, c("F value", "Pr(>F)")] <- list(Fs, P)
     }
     head <- c("Single term deletions", "\nModel:", deparse(formula(object)),
 	      if(!is.null(scale) && scale > 0)
@@ -630,28 +676,12 @@ factor.scope <- function(factor, scope)
     list(drop = nmdrop, add = nmadd)
 }
 
+
+## a slightly simplified version of stepAIC().
 step <- function(object, scope, scale = 0,
 		 direction = c("both", "backward", "forward"),
 		 trace = 1, keep = NULL, steps = 1000, k = 2, ...)
 {
-#     fixFormulaObject <- function(object) {
-# 	tt <- terms(object)
-# 	tmp <- attr(tt, "term.labels")
-# 	if (!attr(tt, "intercept"))
-# 	    tmp <- c(tmp, "0")
-# 	if (!length(tmp))
-# 	    tmp <- "1"
-#         tmp <- paste("~", paste(tmp, collapse = " + "))
-#         form <- formula(object) # some formulae have no lhs
-#         tmp <- if(length(form) > 2) paste(deparse(form[[2L]]), tmp)
-#         ## must be as.character as deparse gives spurious ()
-# 	if (length(offset <- attr(tt, "offset")))
-# 	    tmp <- paste(tmp, as.character(attr(tt, "variables")[offset + 1]),
-# 			 sep = " + ")
-# 	form <- formula(tmp)
-#         environment(form) <- environment(tt)
-#         form
-#     }
     mydeviance <- function(x, ...)
     {
         dev <- deviance(x)
@@ -661,7 +691,7 @@ step <- function(object, scope, scale = 0,
     cut.string <- function(string)
     {
 	if(length(string) > 1L)
-	    string[-1L] <- paste("\n", string[-1L], sep = "")
+	    string[-1L] <- paste0("\n", string[-1L])
 	string
     }
     re.arrange <- function(keep)
@@ -733,6 +763,8 @@ step <- function(object, scope, scale = 0,
     bAIC <- bAIC[2L]
     if(is.na(bAIC))
         stop("AIC is not defined for this model, so 'step' cannot proceed")
+    if(bAIC == -Inf)
+        stop("AIC is -infinity for this model, so 'step' cannot proceed")
     nm <- 1
     ## Terms <- fit$terms
     if(trace) {
