@@ -1,6 +1,8 @@
 #  File src/library/stats/R/ar.burg.mts.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1999-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +16,7 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
-"ar.burg.mts" <-
+ar.burg.mts <-
 function (x, aic = TRUE, order.max = NULL, na.action = na.fail,
     demean = TRUE, series = NULL, var.method = 1, ...)
 {
@@ -40,7 +42,7 @@ function (x, aic = TRUE, order.max = NULL, na.action = na.fail,
         floor(10 * log10(n.used))
     else floor(order.max)
     xaic <- numeric(order.max + 1L)
-    z <- .C(R_multi_burg,
+    z <- .C(C_multi_burg,
             as.integer(n.used),
             resid = as.double(x),
             as.integer(order.max),
