@@ -1,6 +1,8 @@
 #  File src/library/stats/R/family.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -35,7 +37,7 @@ power <- function(lambda = 1)
     mu.eta <- function(eta)
         pmax((1/lambda) * eta^(1/lambda - 1), .Machine$double.eps)
     valideta <- function(eta) all(eta>0)
-    link <- paste("mu^", round(lambda, 3), sep="")
+    link <- paste0("mu^", round(lambda, 3))
     structure(list(linkfun = linkfun, linkinv = linkinv,
                    mu.eta = mu.eta, valideta = valideta, name = link),
               class="link-glm")
@@ -48,12 +50,9 @@ make.link <- function (link)
 {
     switch(link,
            "logit" = {
-               linkfun <- function(mu)
-                   .Call("logit_link", mu, PACKAGE="stats")
-               linkinv <- function(eta)
-                   .Call("logit_linkinv", eta, PACKAGE="stats")
-               mu.eta <- function(eta)
-                   .Call("logit_mu_eta", eta, PACKAGE="stats")
+               linkfun <- function(mu) .Call(C_logit_link, mu)
+               linkinv <- function(eta) .Call(C_logit_linkinv, eta)
+               mu.eta <- function(eta) .Call(C_logit_mu_eta, eta)
                valideta <- function(eta) TRUE
            },
            "probit" = {
@@ -132,20 +131,7 @@ make.link <- function (link)
 poisson <- function (link = "log")
 {
     linktemp <- substitute(link)
-    ## idea is that we can specify a character string or a name for
-    ## the standard links, and if the name is not one of those, look
-    ## for an object of the appropiate class.
-    if (!is.character(linktemp)) {
-	linktemp <- deparse(linktemp)
-        ## the idea here seems to be that we can have a character variable
-        ## 'link' naming the link (undocumented).  Deprecate in 2.4.0.
-	if (linktemp == "link") {
-            warning("use of poisson(link=link) is deprecated\n", domain = NA)
-	    linktemp <- eval(link)
-            if(!is.character(linktemp) || length(linktemp) != 1L)
-                stop("'link' is invalid", domain=NA)
-        }
-    }
+    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
     okLinks <- c("log", "identity", "sqrt")
     if (linktemp %in% okLinks)
 	stats <- make.link(linktemp)
@@ -201,19 +187,7 @@ poisson <- function (link = "log")
 quasipoisson <- function (link = "log")
 {
     linktemp <- substitute(link)
-    if (!is.character(linktemp)) {
-	linktemp <- deparse(linktemp)
-        ## the idea here seems to be that we can have a character variable
-        ## 'link' naming the link.
-	if (linktemp == "link") {
-            warning("use of quasipoisson(link=link) is deprecated\n",
-                    domain = NA)
-	    linktemp <- eval(link)
-            if(!is.character(linktemp) || length(linktemp) != 1L)
-                stop("'link' is invalid", domain=NA)
-        }
-    }
-
+    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
     okLinks <- c("log", "identity", "sqrt")
     if (linktemp %in% okLinks)
         stats <- make.link(linktemp)
@@ -259,17 +233,7 @@ quasipoisson <- function (link = "log")
 gaussian <- function (link = "identity")
 {
     linktemp <- substitute(link)
-    if (!is.character(linktemp)) {
-	linktemp <- deparse(linktemp)
-        ## the idea here seems to be that we can have a character variable
-        ## 'link' naming the link.
-	if (linktemp == "link") {
-            warning("use of gaussian(link=link) is deprecated\n", domain = NA)
-	    linktemp <- eval(link)
-            if(!is.character(linktemp) || length(linktemp) != 1L)
-                stop("'link' is invalid", domain=NA)
-        }
-    }
+    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
     okLinks <- c("inverse", "log", "identity")
     if (linktemp %in% okLinks)
 	stats <- make.link(linktemp)
@@ -316,17 +280,7 @@ gaussian <- function (link = "identity")
 binomial <- function (link = "logit")
 {
     linktemp <- substitute(link)
-    if (!is.character(linktemp)) {
-	linktemp <- deparse(linktemp)
-        ## the idea here seems to be that we can have a character variable
-        ## 'link' naming the link.
-	if (linktemp == "link") {
-            warning("use of binomial(link=link) is deprecated\n", domain = NA)
-	    linktemp <- eval(link)
-            if(!is.character(linktemp) || length(linktemp) != 1L)
-                stop("'link' is invalid", domain=NA)
-        }
-    }
+    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
     okLinks <- c("logit", "probit", "cloglog", "cauchit", "log")
     if (linktemp %in% okLinks)
         stats <- make.link(linktemp)
@@ -346,8 +300,7 @@ binomial <- function (link = "logit")
     }
     variance <- function(mu) mu * (1 - mu)
     validmu <- function(mu) all(mu>0) && all(mu<1)
-    dev.resids <- function(y, mu, wt)
-        .Call("binomial_dev_resids", y, mu, wt, PACKAGE="stats")
+    dev.resids <- function(y, mu, wt) .Call(C_binomial_dev_resids, y, mu, wt)
     aic <- function(y, n, mu, wt, dev) {
         m <- if(any(n > 1)) n else wt
 	-2*sum(ifelse(m > 0, (wt/m), 0)*
@@ -426,18 +379,7 @@ binomial <- function (link = "logit")
 quasibinomial <- function (link = "logit")
 {
     linktemp <- substitute(link)
-    if (!is.character(linktemp)) {
-	linktemp <- deparse(linktemp)
-        ## the idea here seems to be that we can have a character variable
-        ## 'link' naming the link.
-	if (linktemp == "link") {
-            warning("use of quasibinomial(link=link) is deprecated\n", domain = NA)
-	    linktemp <- eval(link)
-            if(!is.character(linktemp) || length(linktemp) != 1L)
-                stop("'link' is invalid", domain=NA)
-        }
-    }
-
+    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
     okLinks <- c("logit", "probit", "cloglog", "cauchit", "log")
     if (linktemp %in% okLinks)
         stats <- make.link(linktemp)
@@ -495,17 +437,7 @@ quasibinomial <- function (link = "logit")
 Gamma <- function (link = "inverse")
 {
     linktemp <- substitute(link)
-    if (!is.character(linktemp)) {
-	linktemp <- deparse(linktemp)
-        ## the idea here seems to be that we can have a character variable
-        ## 'link' naming the link.
-	if (linktemp == "link") {
-            warning("use of Gamma(link=link) is deprecated\n", domain = NA)
-	    linktemp <- eval(link)
-            if(!is.character(linktemp) || length(linktemp) != 1L)
-                stop("'link' is invalid", domain=NA)
-        }
-    }
+    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
     okLinks <- c("inverse", "log", "identity")
     if (linktemp %in% okLinks)
 	stats <- make.link(linktemp)
@@ -561,18 +493,7 @@ Gamma <- function (link = "inverse")
 inverse.gaussian <- function(link = "1/mu^2")
 {
     linktemp <- substitute(link)
-    if (!is.character(linktemp)) {
-	linktemp <- deparse(linktemp)
-        ## the idea here seems to be that we can have a character variable
-        ## 'link' naming the link.
-	if (linktemp == "link") {
-            warning("use of inverse.gaussian(link=link) is deprecated\n",
-                    domain = NA)
-	    linktemp <- eval(link)
-            if(!is.character(linktemp) || length(linktemp) != 1L)
-                stop("'link' is invalid", domain=NA)
-        }
-    }
+    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
     okLinks <- c("inverse", "log", "identity", "1/mu^2")
     if (linktemp %in% okLinks)
 	stats <- make.link(linktemp)
