@@ -1,17 +1,35 @@
+#  File src/library/stats/R/ftable.R
+#  Part of the R package, http://www.R-project.org
+#
+#  Copyright (C) 1995-2012 The R Core Team
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 ftable <- function(x, ...) UseMethod("ftable")
 
 ftable.default <- function(..., exclude = c(NA, NaN),
                            row.vars = NULL, col.vars = NULL) {
     args <- list(...)
-    if (length(args) == 0)
+    if (length(args) == 0L)
         stop("nothing to tabulate")
-    x <- args[[1]]
+    x <- args[[1L]]
     if(is.list(x))
         x <- table(x, exclude = exclude)
     else if(inherits(x, "ftable")) {
         x <- as.table(x)
     }
-    else if(!(is.array(x) && (length(dim(x)) > 1))) {
+    else if(!(is.array(x) && (length(dim(x)) > 1L))) {
         x <- table(..., exclude = exclude)
     }
     dn <- dimnames(x)
@@ -67,7 +85,7 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
 {
     if(missing(formula) || !inherits(formula, "formula"))
         stop("'formula' missing or incorrect")
-    if(length(formula) != 3)
+    if(length(formula) != 3L)
         stop("'formula' must have both left and right hand sides")
     ## need to cope with '.' in formula
     tt <- if(is.data.frame(data)) terms(formula, data=data)
@@ -75,8 +93,8 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
     if(any(attr(tt, "order") > 1))
         stop("interactions are not allowed")
     ## here we do NOT want '.' expanded
-    rvars <- attr(terms(formula[-2], allowDotAsName=TRUE), "term.labels")
-    cvars <- attr(terms(formula[-3], allowDotAsName=TRUE), "term.labels")
+    rvars <- attr(terms(formula[-2L], allowDotAsName=TRUE), "term.labels")
+    cvars <- attr(terms(formula[-3L], allowDotAsName=TRUE), "term.labels")
     rhs.has.dot <- any(rvars == ".")
     lhs.has.dot <- any(cvars == ".")
     if(lhs.has.dot && rhs.has.dot)
@@ -85,7 +103,7 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
     edata <- eval(m$data, parent.frame())
     if(inherits(edata, "ftable")
        || inherits(edata, "table")
-       || length(dim(edata)) > 2) {
+       || length(dim(edata)) > 2L) {
         if(inherits(edata, "ftable")) {
             data <- as.table(data)
         }
@@ -127,7 +145,7 @@ ftable.formula <- function(formula, data = NULL, subset, na.action, ...)
                                    paste(c(rvars, cvars),
                                          collapse = "+")),
                                 env = environment(formula))
-        m[[1]] <- as.name("model.frame")
+        m[[1L]] <- as.name("model.frame")
         mf <- eval(m, parent.frame())
         ftable(mf, row.vars = rvars, col.vars = cvars, ...)
     }
@@ -145,8 +163,7 @@ as.table.ftable <- function(x, ...)
                dimnames = c(xrv, xcv))
     nrv <- length(xrv)
     ncv <- length(xcv)
-    x <- aperm(x, c(seq(from = nrv, to = 1),
-                    seq(from = nrv + ncv, to = nrv + 1)))
+    x <- aperm(x, c(rev(seq_len(nrv)), rev(seq_len(ncv) + nrv)))
     class(x) <- "table"
     x
 }
@@ -156,14 +173,14 @@ format.ftable <- function(x, quote = TRUE, digits = getOption("digits"), ...)
     if(!inherits(x, "ftable"))
         stop("'x' must be an \"ftable\" object")
     charQuote <- function(s)
-        if(quote) paste("\"", s, "\"", sep = "") else s
+        if(quote) paste0("\"", s, "\"") else s
     makeLabels <- function(lst) {
         lens <- sapply(lst, length)
         cplensU <- c(1, cumprod(lens))
         cplensD <- rev(c(1, cumprod(rev(lens))))
         y <- NULL
         for (i in rev(seq_along(lst))) {
-            ind <- 1 + seq(from = 0, to = lens[i] - 1) * cplensD[i + 1]
+            ind <- 1 + seq.int(from = 0, to = lens[i] - 1) * cplensD[i + 1]
             tmp <- character(length = cplensD[i])
             tmp[ind] <- charQuote(lst[[i]])
             y <- cbind(rep(tmp, times = cplensU[i]), y)
@@ -179,7 +196,7 @@ format.ftable <- function(x, quote = TRUE, digits = getOption("digits"), ...)
 
     xrv <- attr(x, "row.vars")
     xcv <- attr(x, "col.vars")
-    LABS <- cbind(rbind(matrix("", nr = length(xcv), nc = length(xrv)),
+    LABS <- cbind(rbind(matrix("", nrow = length(xcv), ncol = length(xrv)),
                         charQuote(makeNames(xrv)),
                         makeLabels(xrv)),
                   c(charQuote(makeNames(xcv)),
@@ -187,8 +204,8 @@ format.ftable <- function(x, quote = TRUE, digits = getOption("digits"), ...)
     DATA <- rbind(if(length(xcv)) t(makeLabels(xcv)),
                   rep("", times = ncol(x)),
                   format(unclass(x), digits = digits))
-    cbind(apply(LABS, 2, format, justify = "left"),
-	  apply(DATA, 2, format, justify = "right"))
+    cbind(apply(LABS, 2L, format, justify = "left"),
+	  apply(DATA, 2L, format, justify = "right"))
 }
 
 write.ftable <- function(x, file = "", quote = TRUE, append = FALSE,
@@ -216,12 +233,12 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
         ## We really need something seekable, see below.  If it is not,
         ## the best we can do is write everything to a tempfile.
         tmpf <- tempfile()
-        cat(file, file = tmpf)
-        file <- tmpf
-        on.exit(unlink(file))
+        cat(readLines(file), file = tmpf, sep="\n")
+        file <- file(tmpf, "r")
+        on.exit({close(file);unlink(tmpf)}, add=TRUE)
     }
 
-    z <- count.fields(file, sep, quote, skip)
+    z <- utils::count.fields(file, sep, quote, skip)
     n.row.vars <- z[max(which(z == max(z)))] - z[length(z)] + 1
 
     seek(file, where = 0)
@@ -240,16 +257,16 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
     ## so there is exactly one line which does not start with a space
     ## and has n.row.vars fields (and it cannot be the first one).
     j <- i[grep("^[^[:space:]]", lines[i])]
-    if((length(j) == 1) && (j > 1)) {
+    if((length(j) == 1L) && (j > 1)) {
         ## An ftable: we can figure things out ourselves.
         n.col.vars <- j - 1
         col.vars <- vector("list", length = n.col.vars)
         n <- c(1, z[1 : n.col.vars] - 1)
-        for(k in seq(from = 1, to = n.col.vars)) {
+        for(k in seq.int(from = 1, to = n.col.vars)) {
             s <- scan(file, what = "", sep = sep, quote = quote,
                       nlines = 1, quiet = TRUE)
-            col.vars[[k]] <- s[-1]
-            names(col.vars)[k] <- s[1]
+            col.vars[[k]] <- s[-1L]
+            names(col.vars)[k] <- s[1L]
         }
         row.vars <- vector("list", length = n.row.vars)
         names(row.vars) <- scan(file, what = "", sep = sep, quote =
@@ -258,7 +275,7 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
     }
     else {
         ## This is not really an ftable.
-        if((z[1] == 1) && z[2] == max(z)) {
+        if((z[1L] == 1) && z[2L] == max(z)) {
             ## Case A.  File looks like
             ##
             ##                                cvar.nam
@@ -269,12 +286,12 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
             col.vars <- vector("list", length = n.col.vars)
             s <- scan(file, what = "", sep = sep, quote = quote,
                       nlines = 2, quiet = TRUE)
-            names(col.vars) <- s[1]
-            s <- s[-1]
+            names(col.vars) <- s[1L]
+            s <- s[-1L]
             row.vars <- vector("list", length = n.row.vars)
             i <- 1 : n.row.vars
             names(row.vars) <- s[i]
-            col.vars[[1]] <- s[-i]
+            col.vars[[1L]] <- s[-i]
             z <- z[-(1 : 2)]
         }
         else {
@@ -300,7 +317,7 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
                     paste("Factor", seq_along(col.vars), sep = ".")
             else {
                 nam <- names(col.vars)
-                ind <- which(nchar(nam) == 0)
+                ind <- which(!nzchar(nam))
                 names(col.vars)[ind] <-
                     paste("Factor", ind, sep = ".")
             }
@@ -309,7 +326,7 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
 
     p <- 1
     n <- integer(n.row.vars)
-    for(k in seq(from = 1, to = n.row.vars)) {
+    for(k in seq.int(from = 1, to = n.row.vars)) {
         n[k] <- sum(z >= max(z) - k + 1) / p
         p <- p * n[k]
     }
@@ -319,15 +336,15 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
     values <- as.numeric(s[!is.row.lab])
     tmp <- s[is.row.lab]
     len <- length(tmp)
-    for(k in seq(from = 1, to = n.row.vars)) {
-        i <- seq(from = 1, to = len, by = len / n[k])
+    for(k in seq.int(from = 1, to = n.row.vars)) {
+        i <- seq.int(from = 1, to = len, by = len / n[k])
         row.vars[[k]] <- unique(tmp[i])
-        tmp <- tmp[seq(from = 2, to = len / n[k])]
+        tmp <- tmp[seq.int(from = 2, to = len / n[k])]
         len <- length(tmp)
     }
     values <- matrix(values,
-                     nr = prod(sapply(row.vars, length)),
-                     nc = prod(sapply(col.vars, length)),
+                     nrow = prod(sapply(row.vars, length)),
+                     ncol = prod(sapply(col.vars, length)),
                      byrow = TRUE)
     structure(values,
               row.vars = row.vars,
@@ -335,5 +352,6 @@ read.ftable <- function(file, sep = "", quote = "\"", row.var.names,
               class = "ftable")
 }
 
-as.data.frame.ftable <- function(x, row.names = NULL, optional = FALSE, ...)
+as.data.frame.ftable <-
+function(x, row.names = NULL, optional = FALSE, ...)
     as.data.frame(as.table(x), row.names, optional)
