@@ -1,6 +1,8 @@
 #  File src/library/stats/R/glm.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,6 +16,8 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
+utils::globalVariables("n", add = TRUE)
+
 ### This function fits a generalized linear model via
 ### iteratively reweighted least squares for any family.
 ### Written by Simon Davies, Dec 1995
@@ -100,11 +104,15 @@ glm <- function(formula, family = gaussian, data, weights,
     ## re-calculated by setting an offset (provided there is an intercept).
     ## Prior to 2.4.0 this was only done for non-zero offsets.
     if(length(offset) && attr(mt, "intercept") > 0L) {
-        fit$null.deviance <-
+        fit2 <-
             eval(call(if(is.function(method)) "method" else method,
                       x = X[, "(Intercept)", drop=FALSE], y = Y,
                       weights = weights, offset = offset, family = family,
-                      control = control, intercept = TRUE))$deviance
+                      control = control, intercept = TRUE))
+        ## That fit might not have converged ....
+        if(!fit2$converged)
+            warning("fitting to calculate the null deviance did not converge -- increase maxit?")
+        fit$null.deviance <- fit2$deviance
     }
     if(model) fit$model <- mf
     fit$na.action <- attr(mf, "na.action")
@@ -229,18 +237,9 @@ glm.fit <-
             z <- (eta - offset)[good] + (y - mu)[good]/mu.eta.val[good]
             w <- sqrt((weights[good] * mu.eta.val[good]^2)/variance(mu)[good])
             ngoodobs <- as.integer(nobs - sum(!good))
-            ## call Fortran code
-            fit <- .Fortran("dqrls",
-                            qr = x[good, ] * w, n = ngoodobs,
-                            p = nvars, y = w * z, ny = 1L,
-                            tol = min(1e-7, control$epsilon/1000),
-                            coefficients = double(nvars),
-                            residuals = double(ngoodobs),
-                            effects = double(ngoodobs),
-                            rank = integer(1L),
-                            pivot = 1L:nvars, qraux = double(nvars),
-                            work = double(2 * nvars),
-                            PACKAGE = "base")
+            ## call Fortran code via C wrapper
+            fit <- .Call(C_Cdqrls, x[good, , drop = FALSE] * w, z * w,
+                         min(1e-7, control$epsilon/1000))
             if (any(!is.finite(fit$coefficients))) {
                 conv <- FALSE
                 warning(gettextf("non-finite coefficients at iteration %d", iter), domain = NA)
@@ -418,6 +417,8 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
 	return(anova.glmlist(c(list(object), dotargs),
 			     dispersion = dispersion, test=test))
 
+    ## score tests require a bit of extra computing
+    doscore <- !is.null(test) && test=="Rao"
     ## extract variables from model
 
     varlist <- attr(object$terms, "variables")
@@ -430,10 +431,28 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
     nvars <- max(0, varseq)
     resdev <- resdf <- NULL
 
+    if (doscore){
+      score <- numeric(nvars)
+      # fit a null model
+      method <- object$method
+      y <- object$y
+      fit <- eval(call(if(is.function(method)) "method" else method,
+                       x=x[, varseq == 0, drop = FALSE],
+                       y=y,
+                       weights=object$prior.weights,
+                       start  =object$start,
+                       offset =object$offset,
+                       family =object$family,
+                       control=object$control))
+      r <- fit$residuals
+      w <- fit$weights
+    }
+
     ## if there is more than one explanatory variable then
     ## recall glm.fit to fit variables sequentially
 
-    if(nvars > 1) {
+    ## for score tests, we need to do so in any case
+    if(nvars > 1 || doscore) {
 	method <- object$method
         ## allow for 'y = FALSE' in the call (PR#13098)
         y <- object$y
@@ -454,9 +473,25 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
                              offset =object$offset,
                              family =object$family,
                              control=object$control))
+            if (doscore) {
+              zz <- eval(call(if(is.function(method)) "method" else method,
+                             x=x[, varseq <= i, drop = FALSE],
+                             y=r,
+                             weights=w))
+              score[i] <-  zz$null.deviance - zz$deviance
+              r <- fit$residuals
+              w <- fit$weights
+            }
 	    resdev <- c(resdev, fit$deviance)
 	    resdf <- c(resdf, fit$df.residual)
 	}
+        if (doscore) {
+          zz <- eval(call(if(is.function(method)) "method" else method,
+                          x=x,
+                          y=r,
+                          weights=w))
+          score[nvars] <-  zz$null.deviance - zz$deviance
+        }
     }
 
     ## add values from null and full model
@@ -472,6 +507,8 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
     if (length(tl) == 0L) table <- table[1,,drop=FALSE] # kludge for null model
     dimnames(table) <- list(c("NULL", tl),
 			    c("Df", "Deviance", "Resid. Df", "Resid. Dev"))
+    if (doscore)
+      table <- cbind(table, Rao=c(NA,score))
     title <- paste("Analysis of Deviance Table", "\n\nModel: ",
 		   object$family$family, ", link: ", object$family$link,
 		   "\n\nResponse: ", as.character(varlist[-1L])[1L],
@@ -504,6 +541,8 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
 anova.glmlist <- function(object, ..., dispersion=NULL, test=NULL)
 {
 
+    doscore <- !is.null(test) && test=="Rao"
+
     ## find responses for all models and remove
     ## any models with a different response
 
@@ -531,6 +570,26 @@ anova.glmlist <- function(object, ..., dispersion=NULL, test=NULL)
     resdf  <- as.numeric(lapply(object, function(x) x$df.residual))
     resdev <- as.numeric(lapply(object, function(x) x$deviance))
 
+    if (doscore){
+      score <- numeric(nmodels)
+      score[1] <- NA
+      df <- -diff(resdf)
+
+      for (i in seq_len(nmodels-1)) {
+        m1 <- if (df[i]>0) object[[i]] else object[[i+1]]
+        m2 <- if (df[i]>0) object[[i+1]] else object[[i]]
+        r <- m1$residuals
+        w <- m1$weights
+        method <- m2$method
+        zz <- eval(call(if(is.function(method)) "method" else method,
+                        x=model.matrix(m2),
+                        y=r,
+                        weights=w))
+        score[i+1] <-  zz$null.deviance - zz$deviance
+        if (df < 0) score[i+1] <- - score[i+1]
+      }
+    }
+
     ## construct table and title
 
     table <- data.frame(resdf, resdev, c(NA, -diff(resdf)),
@@ -539,6 +598,9 @@ anova.glmlist <- function(object, ..., dispersion=NULL, test=NULL)
 			paste(deparse(formula(x)), collapse="\n") )
     dimnames(table) <- list(1L:nmodels, c("Resid. Df", "Resid. Dev", "Df",
 					 "Deviance"))
+    if (doscore)
+      table <- cbind(table, Rao=score)
+
     title <- "Analysis of Deviance Table\n"
     topnote <- paste("Model ", format(1L:nmodels),": ",
 		     variables, sep="", collapse="\n")
