@@ -1,6 +1,8 @@
 #  File src/library/stats/R/symnum.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -29,7 +31,7 @@ symnum <- function(x, cutpoints = c(  .3,  .6,	 .8,  .9, .95),
 
     ##--------------- Argument checking -----------------------------
     if(length(x) == 0L)
-	return(noquote(if(is.null(d <- dim(x)))character(0L) else array("", dim=d)))
+	return(noquote(if(is.null(d <- dim(x)))character() else array("", dim=d)))
     has.na <- any(nax <- is.na(x))
     if(numeric.x) {
 	force(corr) # missingness..
