@@ -1,6 +1,8 @@
 #  File src/library/stats/R/ts.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -764,6 +766,7 @@ arima.sim <- function(model, n, rand.gen = rnorm,
                       start.innov = rand.gen(n.start, ...), ...)
 {
     if(!is.list(model)) stop("'model' must be list")
+    if(n <= 0L) stop("'n' must be strictly positive")
     p <- length(model$ar)
     if(p) {
         minroots <- min(Mod(polyroot(c(1, -model$ar))))
@@ -785,13 +788,13 @@ arima.sim <- function(model, n, rand.gen = rnorm,
     if(!missing(start.innov) && length(start.innov) < n.start)
         stop(gettextf("'start.innov' is too short: need %d points", n.start),
              domain = NA)
-    x <- ts(c(start.innov[1L:n.start], innov[1L:n]), start = 1 - n.start)
+    x <- ts(c(start.innov[seq_len(n.start)], innov[1L:n]), start = 1 - n.start)
     if(length(model$ma)) {
         x <- filter(x, c(1, model$ma), sides = 1L)
         x[seq_along(model$ma)] <- 0 # rather than NA
     }
     if(length(model$ar)) x <- filter(x, model$ar, method = "recursive")
-    if(n.start > 0) x <- x[-(1L:n.start)]
+    if(n.start > 0) x <- x[-(seq_len(n.start))]
     if(d > 0) x <- diffinv(x, differences = d)
     as.ts(x)
 }
