@@ -1,3 +1,23 @@
+/*
+ *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1999-2005  The R Core Team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
+ *
+ */
+
 /* ansari.c
    Compute the exact distribution of the Ansari-Bradley test statistic.
    */
@@ -116,7 +136,7 @@ void
 qansari(Sint *len, double *x, Sint *m, Sint *n)
 {
     Sint i, l, u;
-    double c, p, q, xi;
+    double c, p, xi;
     double ***w;
 
     w = w_init(*m, *n);
@@ -132,8 +152,8 @@ qansari(Sint *len, double *x, Sint *m, Sint *n)
 	else if(xi == 1)
 	    x[i] = u;
 	else {
-	    p = 0;
-	    q = 0;
+	    p = 0.;
+	    int q = 0;
 	    for(;;) {
 		p += cansari(q, (Sint)*m, (Sint)*n, w) / c;
 		if (p >= xi)
