@@ -1,3 +1,22 @@
+/*
+ *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1999-2009   The R Core Team.
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
+ */
+
 /* ks.c
    Compute the asymptotic distribution of the one- and two-sample
    two-sided Kolmogorov-Smirnov statistics, and the exact distributions
@@ -13,6 +32,7 @@ static double K(int n, double d);
 static void m_multiply(double *A, double *B, double *C, int m);
 static void m_power(double *A, int eA, double *V, int *eV, int m, int n);
 
+/* Two-sample two-sided asymptotic distribution */
 void
 pkstwo(Sint *n, double *x, double *tol)
 {
@@ -68,6 +88,7 @@ pkstwo(Sint *n, double *x, double *tol)
     }
 }
 
+/* Two-sided two-sample */
 void
 psmirnov2x(double *x, Sint *m, Sint *n)
 {
@@ -106,6 +127,7 @@ psmirnov2x(double *x, Sint *m, Sint *n)
     *x = u[*n];
 }
 
+/* The two-sided one-sample 'exact' distribution */
 void
 pkolmogorov2x(double *x, Sint *n)
 {
@@ -128,6 +150,12 @@ K(int n, double d)
    int k, m, i, j, g, eH, eQ;
    double h, s, *H, *Q;
 
+   /* 
+      The faster right-tail approximation is omitted here.
+      s = d*d*n; 
+      if(s > 7.24 || (s > 3.76 && n > 99)) 
+          return 1-2*exp(-(2.000071+.331/sqrt(n)+1.409/n)*s);
+   */
    k = (int) (n * d) + 1;
    m = 2 * k - 1;
    h = k - n * d;
