@@ -1,6 +1,6 @@
 /*  R : A Computer Language for Statistical Data Analysis
  *
- *  Copyright (C) 1999-2002	The R Development Core Team
+ *  Copyright (C) 1999-2002	The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,8 +13,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -78,7 +78,7 @@ static SEXP Starma_tag;
 #define GET_STARMA \
     Starma G; \
     if (TYPEOF(pG) != EXTPTRSXP || R_ExternalPtrTag(pG) != Starma_tag) \
-        error(_("bad Starma struct"));\
+	error(_("bad Starma struct"));\
     G = (Starma) R_ExternalPtrAddr(pG)
 
 SEXP setup_starma(SEXP na, SEXP x, SEXP pn, SEXP xreg, SEXP pm,
@@ -166,9 +166,8 @@ SEXP arma0fa(SEXP pG, SEXP inparams)
 {
     int i, j, ifault = 0, it, streg;
     double sumlog, ssq, tmp, ans;
-    SEXP res;
-    GET_STARMA;
 
+    GET_STARMA;
     dotrans(G, REAL(inparams), G->params, G->trans);
 
     if(G->ns > 0) {
@@ -233,18 +232,13 @@ SEXP arma0fa(SEXP pG, SEXP inparams)
 	G->s2 = ssq/(double)G->nused;
 	ans = 0.5*(log(ssq/(double)G->nused) + sumlog/(double)G->nused);
     }
-    res = allocVector(REALSXP, 1);
-    REAL(res)[0] = ans;
-    return res;
+    return ScalarReal(ans);
 }
 
 SEXP get_s2(SEXP pG)
 {
-    SEXP res = allocVector(REALSXP, 1);
     GET_STARMA;
-
-    REAL(res)[0] = G->s2;
-    return res;
+    return ScalarReal(G->s2);
 }
 
 SEXP get_resid(SEXP pG)
@@ -300,10 +294,9 @@ void artoma(int *pp, double *phi, double *psi, int *npsi)
     int i, j, p = *pp;
 
     for(i = 0; i < p; i++) psi[i] = phi[i];
-    for(i = p + 1; i < *npsi; i++) psi[i] = 0.0;
-    for(i = 0; i < *npsi - p - 1; i++) {
+    for(i = p; i < *npsi; i++) psi[i] = 0.0;
+    for(i = 0; i < *npsi - p - 1; i++)
 	for(j = 0; j < p; j++) psi[i + j + 1] += phi[j]*psi[i];
-    }
 }
 
 static void partrans(int p, double *raw, double *new)
@@ -373,7 +366,7 @@ SEXP Invtrans(SEXP pG, SEXP x)
     GET_STARMA;
 
     n = G->mp + G->mq + G->msp + G->msq;
-    
+
     v = 0;
     invpartrans(G->mp, raw + v, new + v);
     v += G->mp;
@@ -426,7 +419,7 @@ SEXP Gradtrans(SEXP pG, SEXP x)
 	for(i = 0; i < G->msp; i++) {
 	    w1[i] += eps;
 	    partrans(G->msp, w1, w3);
-	    for(j = 0; j < G->msp; j++) 
+	    for(j = 0; j < G->msp; j++)
 		A[i + v + (j+v)*n] = (w3[j] - w2[j])/eps;
 	    w1[i] -= eps;
 	}
@@ -438,7 +431,7 @@ SEXP Gradtrans(SEXP pG, SEXP x)
 	for(i = 0; i < G->msq; i++) {
 	    w1[i] += eps;
 	    partrans(G->msq, w1, w3);
-	    for(j = 0; j < G->msq; j++) 
+	    for(j = 0; j < G->msq; j++)
 		A[i + v + (j+v)*n] = (w3[j] - w2[j])/eps;
 	    w1[i] -= eps;
 	}
@@ -452,7 +445,7 @@ ARMAtoMA(SEXP ar, SEXP ma, SEXP lag_max)
     int i, j, p = LENGTH(ar), q = LENGTH(ma), m = asInteger(lag_max);
     double *phi = REAL(ar), *theta = REAL(ma), *psi, tmp;
     SEXP res;
-    
+
     if(m <= 0 || m == NA_INTEGER)
 	error(_("invalid value of lag.max"));
     PROTECT(res = allocVector(REALSXP, m));
@@ -466,4 +459,3 @@ ARMAtoMA(SEXP ar, SEXP ma, SEXP lag_max)
     UNPROTECT(1);
     return res;
 }
-
