@@ -1,6 +1,8 @@
 #  File src/library/tcltk/R/utils.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -184,9 +186,9 @@ tk_choose.files <-
         if(!is.character(filters) || length(dim(filters)) != 2 || ncol(filters) != 2)
             stop("'filters' must be a 2-column character matrix")
         f <- filters
-        f[] <- paste("{", filters, "}", sep="")
+        f[] <- paste0("{", filters, "}")
         ff <- apply(f, 1, paste, collapse = " ")
-        fff <- paste("{", ff, "}", sep="")
+        fff <- paste0("{", ff, "}")
         args <- c(args, filetypes = paste(fff, collapse = " "))
     }
     res <- tclvalue(do.call(tcl, args))
