@@ -1,6 +1,8 @@
 #  File src/library/tools/R/citation.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -31,6 +33,15 @@ function(cfile, encoding = NULL)
     ## we could simply convert to UTF-8.
     if(encoding %in% c("latin1", "UTF-8") && !l10n_info()$MBCS) {
         parse(file = cfile, encoding = encoding)
+    } else if(encoding %in% c("C", "ASCII")) {
+        ## We do want to make sure this is ASCII: in single-byte
+        ## locales 8-bit chars are likely to be parsed as bytes.
+        ## Based on showNonASCII()
+        x <- readLines(cfile, warn = FALSE)
+        asc <- iconv(x, "latin1", "ASCII")
+        if (any(is.na(asc) | asc != x))
+            stop("non-ASCII input in a CITATION file without a declared encoding")
+        parse(file = cfile)
     } else {
         con <- file(cfile, encoding = encoding)
         on.exit(close(con))
