@@ -1,6 +1,8 @@
 #   File src/library/utils/R/Sweave.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -90,11 +92,17 @@ Sweave <- function(file, driver = RweaveLatex(),
 
     namedchunks <- list()
     prevfilenum <- 0L
-    prevlinediff <- 0L    
+    prevlinediff <- 0L
     for (linenum in seq_along(text)) {
     	line <- text[linenum]
     	filenum <- srcFilenum[linenum]
     	linediff <- srcLinenum[linenum] - linenum
+	if(nzchar(Sys.getenv("R_DEBUG_Sweave"))) {
+	    ## Extensive logging for debugging, needs 'ls' (unix-like or Rtools):
+	    cat(sprintf("l.%3d: %30s -'%4s'- ", linenum, substr(line,1,30), mode))
+	    cat(sprintf("%16s\n", system(paste("ls -s",
+				   summary(drobj$output)$description), intern=TRUE)))
+	}
         if (length(grep(syntax$doc, line))) { # start new documentation chunk
             if (mode == "doc") {
                 if (!is.null(chunk)) drobj <- driver$writedoc(drobj, chunk)
@@ -122,7 +130,7 @@ Sweave <- function(file, driver = RweaveLatex(),
                                             driver$checkopts)
             ## these #line directives are used for error messages when parsing
             file <- srcFilenames[filenum]
-            chunk <- paste("#line ", linenum+linediff+1L, ' "', basename(file), '"', sep="")
+            chunk <- paste0("#line ", linenum+linediff+1L, ' "', basename(file), '"')
             attr(chunk, "srclines") <- linenum + linediff
             attr(chunk, "srcFilenum") <- filenum
             chunknr <- chunknr + 1L  # this is really 'code chunk number'
@@ -141,17 +149,17 @@ Sweave <- function(file, driver = RweaveLatex(),
                     ## when parsing
                     file <- srcFilenames[filenum]
                     line <- c(namedchunks[[chunkref]],
-                              paste("#line ", linenum+linediff+1L, ' "',
-                                    basename(file), '"', sep=""))
+			      paste0("#line ", linenum+linediff+1L,
+				     ' "', basename(file), '"'))
                 }
             }
-            if (mode == "code" && 
+            if (mode == "code" &&
                 (prevfilenum != filenum ||
                  prevlinediff != linediff)) {
                 file <- srcFilenames[filenum]
-                line <- c(paste("#line ", linenum+linediff, ' "', basename(file), '"', sep=""),
+                line <- c(paste0("#line ", linenum+linediff, ' "', basename(file), '"'),
                           line)
-            }             
+            }
             srclines <- c(attr(chunk, "srclines"), rep(linenum+linediff, length(line)))
             srcfilenum <- c(attr(chunk, "srcFilenum"), rep(filenum, length(line)))
 	    chunk <- c(chunk, line)
@@ -184,7 +192,7 @@ SweaveReadFile <- function(file, syntax, encoding = "")
     df <- dirname(f)
     if (!file.exists(f)) {
         f <- list.files(df, full.names = TRUE,
-                        pattern = paste(bf, syntax$extension, sep = ""))
+                        pattern = paste0(bf, syntax$extension))
 
         if (length(f) == 0L)
             stop(gettextf("no Sweave file with name %s found",
@@ -207,10 +215,9 @@ SweaveReadFile <- function(file, syntax, encoding = "")
             enc <- if (nzchar(encoding)) {
                 encoding
             } else {
-                warning(sQuote(basename(file)),
-                        " has unknown encoding: assuming Latin-1",
+                stop(sQuote(basename(file)),
+                        " is not ASCII and does not declare an encoding",
                         domain = NA, call. = FALSE)
-                "latin1"
             }
         } else if (enc == "unknown") {
             stop(sQuote(basename(file)),
@@ -228,11 +235,11 @@ SweaveReadFile <- function(file, syntax, encoding = "")
     if (length(pos) > 0L) {
         sname <- sub(syntax$syntaxname, "\\1", text[pos[1L]])
         syntax <- get(sname, mode = "list")
-        if (class(syntax) != "SweaveSyntax")
+        if (!identical(class(syntax), "SweaveSyntax"))
             stop(gettextf("object %s does not have class \"SweaveSyntax\"",
                           sQuote(sname)), domain = NA)
         text <- text[-pos]
-        srcLinenum <- srcLinenum[-pos]       
+        srcLinenum <- srcLinenum[-pos]
     }
     srcFilenum <- rep(1, length(srcLinenum))
 
@@ -252,7 +259,7 @@ SweaveReadFile <- function(file, syntax, encoding = "")
 	    pre <- seq_len(pos-1L)
 	    post <- seq_len(length(text) - pos) + pos
 	    text <- c(text[pre], itext, text[post])
-	    
+
 	    srcLinenum <- c(srcLinenum[pre], attr(itext, "srcLinenum"),
 	    		    srcLinenum[post])
 	    srcFilenum <- c(srcFilenum[pre], attr(itext, "srcFilenum")+length(f),
@@ -265,7 +272,7 @@ SweaveReadFile <- function(file, syntax, encoding = "")
     attr(text, "files") <- f
     attr(text, "encoding") <- enc
     attr(text, "srcLinenum") <- srcLinenum
-    attr(text, "srcFilenum") <- srcFilenum    
+    attr(text, "srcFilenum") <- srcFilenum
     text
 }
 
@@ -312,7 +319,7 @@ SweaveGetSyntax <- function(file)
     synt <- apropos("SweaveSyntax", mode = "list")
     for (sname in synt) {
         s <- get(sname, mode = "list")
-        if (class(s) != "SweaveSyntax") next
+        if (!identical(class(s), "SweaveSyntax")) next
         if (length(grep(s$extension, file))) return(s)
     }
     SweaveSyntaxNoweb
@@ -323,9 +330,8 @@ SweaveSyntConv <- function(file, syntax, output=NULL)
 {
     if (is.character(syntax)) syntax <- get(syntax)
 
-    if (class(syntax) != "SweaveSyntax")
+    if (!identical(class(syntax), "SweaveSyntax"))
         stop("target syntax not of class \"SweaveSyntax\"")
-
     if (is.null(syntax$trans))
         stop("target syntax contains no translation table")
 
@@ -374,7 +380,7 @@ SweaveParseOptions <- function(text, defaults = list(), check = NULL)
     ## This is undocumented
     if (!is.null(options[["label"]]) && !is.null(options[["engine"]]))
         options[["label"]] <-
-            sub(paste("\\.", options[["engine"]], "$", sep=""),
+            sub(paste0("\\.", options[["engine"]], "$"),
                 "", options[["label"]])
 
     if (!is.null(check)) check(options) else options
@@ -418,7 +424,7 @@ SweaveHooks <- function(options, run = FALSE, envir = .GlobalEnv)
             "  --options=      comma-separated list of Sweave options",
             "  --pdf           convert to PDF document",
             "",
-            "Report bugs to <r-bugs@r-project.org>.",
+            "Report bugs at bugs.r-project.org .",
             sep = "\n")
     }
     do_exit <- function(status = 0L)
@@ -442,7 +448,7 @@ SweaveHooks <- function(options, run = FALSE, envir = .GlobalEnv)
                 R.version[["major"]], ".",  R.version[["minor"]],
                 " (r", R.version[["svn rev"]], ")\n", sep = "")
             cat("",
-                "Copyright (C) 2006-2011 The R Core Development Team.",
+                "Copyright (C) 2006-2011 The R Core Team.",
                 "This is free software; see the GNU General Public License version 2",
                 "or later for copying conditions.  There is NO warranty.",
                 sep = "\n")
@@ -501,7 +507,7 @@ SweaveHooks <- function(options, run = FALSE, envir = .GlobalEnv)
             "  --encoding=enc  assume encoding 'enc' for file",
             "  --options=      comma-separated list of Stangle options",
             "",
-            "Report bugs to <r-bugs@r-project.org>.",
+            "Report bugs at bugs@r-project.org .",
             sep = "\n")
     }
     do_exit <- function(status = 0L)
@@ -525,7 +531,7 @@ SweaveHooks <- function(options, run = FALSE, envir = .GlobalEnv)
                 R.version[["major"]], ".",  R.version[["minor"]],
                 " (r", R.version[["svn rev"]], ")\n", sep = "")
             cat("",
-                "Copyright (C) 2006-2011 The R Core Development Team.",
+                "Copyright (C) 2006-2011 The R Core Team.",
                 "This is free software; see the GNU General Public License version 2",
                 "or later for copying conditions.  There is NO warranty.",
                 sep = "\n")
