@@ -1,6 +1,8 @@
 #  File src/library/utils/R/package.skeleton.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +18,7 @@
 
 package.skeleton <-
     function(name = "anRpackage", list = character(), environment = .GlobalEnv,
-	     path = ".", force = FALSE, namespace = FALSE,
+	     path = ".", force = FALSE, namespace = TRUE,
              code_files = character())
 {
     safe.dir.create <- function(path)
@@ -26,8 +28,6 @@ package.skeleton <-
 	    stop(gettextf("cannot create directory '%s'", path), domain = NA)
     }
 
-    if(!is.logical(namespace) || length(namespace) != 1L)
-        stop("'namespace' must be a single logical")
     if(!is.character(code_files))
         stop("'code_files' must be a character vector")
     use_code_files <- length(code_files) > 0L
@@ -63,7 +63,8 @@ package.skeleton <-
     curLocale <- Sys.getlocale("LC_CTYPE")
     on.exit(Sys.setlocale("LC_CTYPE", curLocale), add = TRUE)
     if(Sys.setlocale("LC_CTYPE", "C") != "C")
-        warning("cannot turn off locale-specific chars via LC_CTYPE")
+        warning("cannot turn off locale-specific chars via LC_CTYPE",
+                domain = NA)
 
     have <- unlist(lapply(list, exists, envir = environment))
     if(any(!have))
@@ -76,7 +77,7 @@ package.skeleton <-
     if(!length(list))
 	stop("no R objects specified or available")
 
-    message("Creating directories ...")
+    message("Creating directories ...", domain = NA)
     ## Make the directories
     dir <- file.path(path, name)
     if(file.exists(dir) && !force)
@@ -88,7 +89,7 @@ package.skeleton <-
     safe.dir.create(data_dir <- file.path(dir, "data"))
 
     ## DESCRIPTION
-    message("Creating DESCRIPTION ...")
+    message("Creating DESCRIPTION ...", domain = NA)
     description <- file(file.path(dir, "DESCRIPTION"), "wt")
     cat("Package: ", name, "\n",
 	"Type: Package\n",
@@ -99,44 +100,40 @@ package.skeleton <-
 	"Maintainer: Who to complain to <yourfault@somewhere.net>\n",
 	"Description: More about what it does (maybe more than one line)\n",
 	"License: What license is it under?\n",
-	"LazyLoad: yes\n",
 	if(usingS4) "Depends: methods\n",
 	file = description, sep = "")
     close(description)
 
+    if(!namespace)
+	warning("From R 2.14.0 on, every package gets a NAMESPACE.",
+		" Argument 'namespace' is deprecated.", domain = NA)
     ## NAMESPACE
     ## <NOTE>
     ## For the time being, we export all non-internal objects using the pattern
     ## of names beginning with alpha.  All S4 methods and classes are exported.
     ## S3 methods will be exported if the function's name would be exported.
     ## </NOTE>
-    if(namespace) {
-        message("Creating NAMESPACE ...")
-        out <- file(file.path(dir, "NAMESPACE"), "wt")
-        writeLines("exportPattern(\"^[[:alpha:]]+\")", out)
-        if(length(methodsList)) {
-            cat("exportMethods(\n    ", file = out)
-            cat(paste('"', methodsList, '"', sep="", collapse = ",\n    "), "\n)\n", file = out)
-        }
-        if(length(classesList)) {
-            cat("exportClasses(\n    ", file = out)
-            cat(paste('"', classesList, '"', sep="", collapse = ",\n     "), "\n)\n", file = out)
-        }
-        close(out)
+    message("Creating NAMESPACE ...", domain = NA)
+    out <- file(file.path(dir, "NAMESPACE"), "wt")
+    writeLines("exportPattern(\"^[[:alpha:]]+\")", out)
+    if(length(methodsList)) {
+	cat("exportMethods(\n    ", file = out)
+	cat(paste0('"', methodsList, '"', collapse = ",\n    "), "\n)\n", file = out)
     }
+    if(length(classesList)) {
+	cat("exportClasses(\n    ", file = out)
+	cat(paste0('"', classesList, '"', collapse = ",\n     "), "\n)\n", file = out)
+    }
+    close(out)
 
     ## Read-and-delete-me
-    message("Creating Read-and-delete-me ...")
+    message("Creating Read-and-delete-me ...", domain = NA)
     out <- file(file.path(dir, "Read-and-delete-me"), "wt")
     msg <-
         c("* Edit the help file skeletons in 'man', possibly combining help files for multiple functions.",
-          if(namespace)
           "* Edit the exports in 'NAMESPACE', and add necessary imports.",
           "* Put any C/C++/Fortran code in 'src'.",
-          if(namespace)
-          "* If you have compiled code, add a useDynLib() directive to 'NAMESPACE'."
-          else
-          "* If you have compiled code, add a .First.lib() function in 'R' to load the shared object.",
+          "* If you have compiled code, add a useDynLib() directive to 'NAMESPACE'.",
           "* Run R CMD build to build the package tarball.",
           "* Run R CMD check to check the package tarball.",
           "",
@@ -154,21 +151,25 @@ package.skeleton <-
 
     ## Dump the items in 'data' or 'R'
     if(!use_code_files) {
-        message("Saving functions and data ...")
+        message("Saving functions and data ...", domain = NA)
         if(length(internalObjInds))
             dump(internalObjs,
                  file = file.path(code_dir,
                  sprintf("%s-internal.R", name)))
         for(item in list){
-            if(is.function(get(item, envir = environment)))
+            objItem <- get(item, envir = environment)
+            if(is.function(objItem))  {
+                if(isS4(objItem))
+                    stop(gettextf("Generic functions and other S4 objects (e.g., '%s') cannot be dumped; use the code_files= argument", item), domain = NA)
                 dump(item,
                      file = file.path(code_dir, sprintf("%s.R", list0[item])))
+            }
             else       # we cannot guarantee this is a valid file name
                 try(save(list = item, envir = environment,
                          file = file.path(data_dir, sprintf("%s.rda", item))))
         }
     } else {
-        message("Copying code files ...")
+        message("Copying code files ...", domain = NA)
         file.copy(code_files, code_dir)
         ## Only "abc.R"-like files are really ok:
 	R_files <- tools::list_files_with_type(code_dir, "code",
@@ -179,36 +180,15 @@ package.skeleton <-
 	if(length(wrong)) {
 	    warning("Invalid file name(s) for R code in ", code_dir,":\n",
 		    strwrap(paste(sQuote(wrong), collapse = ", "), indent=2),
-		    "\n are now renamed to 'z<name>.R'")
+		    "\n are now renamed to 'z<name>.R'", domain = NA)
 	    file.rename(from = file.path(code_dir, wrong),
 			to = file.path(code_dir,
-			paste("z", sub("(\\.[^.]*)?$", ".R", wrong), sep="")))
+			paste0("z", sub("(\\.[^.]*)?$", ".R", wrong))))
         }
     }
 
     ## Make help file skeletons in 'man'
-    message("Making help files ...")
-    if(!namespace && length(internalObjInds)) {
-        notNeeded <- grep(methods:::.methodsPackageMetaNamePattern, internalObjs)
-        notNeeded <- c(notNeeded, match(".packageName", internalObjs, 0L))
-        if(length(notNeeded) < length(internalObjs)) {
-            internalObjs <- internalObjs[-notNeeded]
-            Rdfile <- file(file.path(docs_dir,
-                                     sprintf("%s-internal.Rd", name)),
-                           "wt")
-            cat("\\name{", name, "-internal}\n",
-                "\\title{Internal ", name, " objects}\n",
-                file = Rdfile, sep = "")
-            for(item in internalObjs) {
-                cat("\\alias{", item, "}\n", file = Rdfile, sep = "")
-            }
-            cat("\\description{Internal ", name, " objects.}\n",
-                "\\details{These are not to be called by the user.}\n",
-                "\\keyword{internal}",
-                file = Rdfile, sep = "")
-            close(Rdfile)
-        }
-    }
+    message("Making help files ...", domain = NA)
     ## Suppress partially inappropriate messages from prompt().
     yy <- try(suppressMessages({
 	promptPackage(name,
@@ -258,8 +238,8 @@ package.skeleton <-
     if(length(list.files(data_dir)) == 0L)
         unlink(data_dir, recursive = TRUE)
 
-    message("Done.")
-    message(gettextf("Further steps are described in '%s'.",
+    message("Done.", domain = NA)
+    message(sprintf("Further steps are described in '%s'.",
                      file.path(dir, "Read-and-delete-me")),
             domain = NA)
 }
@@ -272,16 +252,17 @@ package.skeleton <-
         ## If we use given code files, we could still check whether
         ## these file are valid across platforms ...
         ## </NOTE>
+        list <- as.character(list) # remove S4 class if any, to add names() later
         if(length(list) == 0L) return(list)
         list0 <- gsub("[[:cntrl:]\"*/:<>?\\|]", "_", list)
         wrong <- grep("^(con|prn|aux|clock\\$|nul|lpt[1-3]|com[1-4])(\\..*|)$",
                       list0)
         if(length(wrong))
-            list0[wrong] <- paste("zz", list0[wrong], sep="")
+            list0[wrong] <- paste0("zz", list0[wrong])
         ## using grep was wrong, as could give -integer(0)
         ok <- grepl("^[[:alnum:]]", list0)
         if(any(!ok))
-            list0[!ok] <- paste("z", list0[!ok], sep="")
+            list0[!ok] <- paste0("z", list0[!ok])
         ## now on Mac/Windows lower/uppercase will collide too
         list1 <- tolower(list0)
         list2 <- make.unique(list1, sep="_")
