@@ -32,10 +32,16 @@
 #include <limits.h>
 #include <string.h>*/
 
+/* This is C90 */
+#ifndef HAVE_LOCALE_H
+# define HAVE_LOCALE_H 1
+#endif
 #ifdef HAVE_STRINGS_H
 #include <strings.h>  /* for strncasecmp */
 #endif
 
+#include <ctype.h> /* for isspace */
+
 #define match_char(ch1, ch2) if (ch1 != ch2) return NULL
 
 /* we guarantee to have strncasecmp in R */
@@ -208,9 +214,8 @@ w_strptime_internal (wchar_t *rp, const wchar_t *fmt, struct tm *tm,
 		     enum locale_status *decided, double *psecs, 
 		     int *poffset)
 {
-    const wchar_t *rp_backup;
     int cnt;
-    size_t val;
+    int val;
     int have_I, is_pm;
     int century, want_century;
     int have_wday, want_xday;
@@ -250,9 +255,6 @@ w_strptime_internal (wchar_t *rp, const wchar_t *fmt, struct tm *tm,
 	/* We need this for handling the `E' modifier.  */
     start_over:
 
-	/* Make back up of current processing pointer.  */
-	rp_backup = rp;
-
 	switch (*fmt++)
 	{
 	case L'%':
@@ -332,7 +334,7 @@ w_strptime_internal (wchar_t *rp, const wchar_t *fmt, struct tm *tm,
 	case L'k':
 	case L'H':
 	  /* Match hour in 24-hour clock.  */
-	  get_number (0, 23, 2);
+	  get_number (0, 24, 2); /* allow 24:00:00 */
 	  tm->tm_hour = val;
 	  have_I = 0;
 	  break;
@@ -462,7 +464,8 @@ w_strptime_internal (wchar_t *rp, const wchar_t *fmt, struct tm *tm,
 	    get_number (0, 99, 2);
 	    /* The "Year 2000: The Millennium Rollover" paper suggests that
 	       values in the range 69-99 refer to the twentieth century.  */
-	    tm->tm_year = val >= 69 ? val : val + 100;
+	    int ival = val;
+	    tm->tm_year = ival >= 69 ? ival : ival + 100;
 	    /* Indicate that we want to use the century, if specified.  */
 	    want_century = 1;
 	    want_xday = 1;
@@ -492,7 +495,7 @@ w_strptime_internal (wchar_t *rp, const wchar_t *fmt, struct tm *tm,
 		    val = (val / 100) * 100 + ((val % 100) * 50) / 30;
 		}
 		if (val > 1200) return NULL;
-		off = (val * 3600) / 100;
+		off = ((val * 3600) / 100);
 		if (neg) off = -off;
 		*poffset = off;
 	    }
@@ -555,7 +558,7 @@ w_strptime_internal (wchar_t *rp, const wchar_t *fmt, struct tm *tm,
 		    wchar_t *end;
 		    sval = wcstod(rp, &end);
 		    if( sval >= 0.0 && sval <= 61.0) {
-			tm->tm_sec = sval;
+			tm->tm_sec = (int) sval;
 			*psecs = sval;
 		    }
 		    rp = end;
@@ -585,7 +588,8 @@ w_strptime_internal (wchar_t *rp, const wchar_t *fmt, struct tm *tm,
 	    case L'y':
 		/* Match year within century using alternate numeric symbols.  */
 		get_alt_number (0, 99, 2);
-		tm->tm_year = val >= 69 ? val : val + 100;
+	        int ival = val;
+	        tm->tm_year = ival >= 69 ? ival : ival + 100;
 		want_xday = 1;
 		break;
 	    default:
@@ -671,9 +675,8 @@ strptime_internal (const char *rp, const char *fmt, struct tm *tm,
 		   enum locale_status *decided, double *psecs,
 		   int *poffset)
 {
-    const char *rp_backup;
     int cnt;
-    size_t val;
+    int val;
     int have_I, is_pm;
     int century, want_century;
     int have_wday, want_xday;
@@ -713,9 +716,6 @@ strptime_internal (const char *rp, const char *fmt, struct tm *tm,
 	/* We need this for handling the `E' modifier.  */
     start_over:
 
-	/* Make back up of current processing pointer.  */
-	rp_backup = rp;
-
 	switch (*fmt++)
 	{
 	case '%':
@@ -795,7 +795,7 @@ strptime_internal (const char *rp, const char *fmt, struct tm *tm,
 	case 'k':
 	case 'H':
 	  /* Match hour in 24-hour clock.  */
-	  get_number (0, 23, 2);
+	  get_number (0, 24, 2);  /* allow 24:00:00 */
 	  tm->tm_hour = val;
 	  have_I = 0;
 	  break;
@@ -928,7 +928,8 @@ strptime_internal (const char *rp, const char *fmt, struct tm *tm,
 	       And this is mandated by the POSIX 2001 standard, with a
 	       caveat that it might change in future.
 	    */
-	    tm->tm_year = val >= 69 ? val : val + 100;
+	    int ival = val;
+	    tm->tm_year = ival >= 69 ? ival : ival + 100;
 	    /* Indicate that we want to use the century, if specified.  */
 	    want_century = 1;
 	    want_xday = 1;
@@ -1022,7 +1023,7 @@ strptime_internal (const char *rp, const char *fmt, struct tm *tm,
 		       char *end;
 		       sval = strtod(rp, &end);
 		       if( sval >= 0.0 && sval <= 61.0) {
-			   tm->tm_sec = sval;
+			   tm->tm_sec = (int) sval;
 			   *psecs = sval;
 		       }
 		       rp = end;
@@ -1052,7 +1053,8 @@ strptime_internal (const char *rp, const char *fmt, struct tm *tm,
 	    case 'y':
 		/* Match year within century using alternate numeric symbols.  */
 		get_alt_number (0, 99, 2);
-		tm->tm_year = val >= 69 ? val : val + 100;
+		int ival = val;
+		tm->tm_year = ival >= 69 ? ival : ival + 100;
 		want_xday = 1;
 		break;
 	    default:
