@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2008  Robert Gentleman, Ross Ihaka and the
- *                            R Development Core Team
+ *  Copyright (C) 1997--2012  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,8 +18,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is handled as a whole string */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -158,6 +155,8 @@ SEXP attribute_hidden getAttrib0(SEXP vec, SEXP name)
 
 SEXP getAttrib(SEXP vec, SEXP name)
 {
+    if(TYPEOF(vec) == CHARSXP)
+	error("cannot have attributes on a CHARSXP");
     /* pre-test to avoid expensive operations if clearly not needed -- LT */
     if (ATTRIB(vec) == R_NilValue &&
 	! (TYPEOF(vec) == LISTSXP || TYPEOF(vec) == LANGSXP))
@@ -212,17 +211,20 @@ SEXP R_copyDFattr(SEXP in, SEXP out)
 /* 'name' should be 1-element STRSXP or SYMSXP */
 SEXP setAttrib(SEXP vec, SEXP name, SEXP val)
 {
+    PROTECT(vec);
+    PROTECT(name);
+
     if (isString(name))
 	name = install(translateChar(STRING_ELT(name, 0)));
-    if (val == R_NilValue)
+    if (val == R_NilValue) {
+	UNPROTECT(2);
 	return removeAttrib(vec, name);
+    }
 
     /* We allow attempting to remove names from NULL */
     if (vec == R_NilValue)
 	error(_("attempt to set an attribute on NULL"));
 
-    PROTECT(vec);
-    PROTECT(name);
     if (NAMED(val)) val = duplicate(val);
     SET_NAMED(val, NAMED(val) | NAMED(vec));
     UNPROTECT(2);
@@ -316,29 +318,27 @@ void attribute_hidden copyMostAttribNoTs(SEXP inp, SEXP ans)
     UNPROTECT(2);
 }
 
+/* Tweaks here based in part on PR#14934 */
 static SEXP installAttrib(SEXP vec, SEXP name, SEXP val)
 {
-    SEXP s, t;
+    SEXP t = R_NilValue; /* -Wall */
 
-    PROTECT(vec);
-    PROTECT(name);
-    PROTECT(val);
-    for (s = ATTRIB(vec); s != R_NilValue; s = CDR(s)) {
+    if(TYPEOF(vec) == CHARSXP)
+	error("cannot set attribute on a CHARSXP");
+    /* this does no allocation */
+    for (SEXP s = ATTRIB(vec); s != R_NilValue; s = CDR(s)) {
 	if (TAG(s) == name) {
 	    SETCAR(s, val);
-	    UNPROTECT(3);
 	    return val;
 	}
+	t = s; // record last attribute, if any
     }
-    s = allocList(1);
-    SETCAR(s, val);
+    /* The usual convention is that the caller protects, 
+       so this is historical over-cautiousness */
+    PROTECT(vec); PROTECT(name); PROTECT(val);
+    SEXP s = CONS(val, R_NilValue);
     SET_TAG(s, name);
-    if (ATTRIB(vec) == R_NilValue)
-	SET_ATTRIB(vec, s);
-    else {
-	t = nthcdr(ATTRIB(vec), length(ATTRIB(vec)) - 1);
-	SETCDR(t, s);
-    }
+    if (ATTRIB(vec) == R_NilValue) SET_ATTRIB(vec, s); else SETCDR(t, s);
     UNPROTECT(3);
     return val;
 }
@@ -346,6 +346,8 @@ static SEXP installAttrib(SEXP vec, SEXP name, SEXP val)
 static SEXP removeAttrib(SEXP vec, SEXP name)
 {
     SEXP t;
+    if(TYPEOF(vec) == CHARSXP)
+	error("cannot set attribute on a CHARSXP");
     if (name == R_NamesSymbol && isList(vec)) {
 	for (t = vec; t != R_NilValue; t = CDR(t))
 	    SET_TAG(t, R_NilValue);
@@ -370,6 +372,9 @@ static void checkNames(SEXP x, SEXP s)
 	if (length(x) != length(s))
 	    error(_("'names' attribute [%d] must be the same length as the vector [%d]"), length(s), length(x));
     }
+    else if(IS_S4_OBJECT(x)) {
+      /* leave validity checks to S4 code */
+    }
     else error(_("names() applied to a non-vector"));
 }
 
@@ -389,6 +394,13 @@ SEXP tspgets(SEXP vec, SEXP val)
     if (vec == R_NilValue)
 	error(_("attempt to set an attribute on NULL"));
 
+    if(IS_S4_OBJECT(vec)) { /* leave validity checking to validObject */
+        if (!isNumeric(val)) /* but should have been checked */
+	    error(_("'tsp' attribute must be numeric"));
+	installAttrib(vec, R_TspSymbol, val);
+	return vec;
+    }
+
     if (!isNumeric(val) || length(val) != 3)
 	error(_("'tsp' attribute must be numeric of length three"));
 
@@ -448,6 +460,7 @@ SEXP attribute_hidden do_commentgets(SEXP call, SEXP op, SEXP args, SEXP env)
     if (NAMED(CAR(args)) == 2) SETCAR(args, duplicate(CAR(args)));
     if (length(CADR(args)) == 0) SETCADR(args, R_NilValue);
     setAttrib(CAR(args), R_CommentSymbol, CADR(args));
+    SET_NAMED(CAR(args), 0);
     return CAR(args);
 }
 
@@ -497,20 +510,33 @@ SEXP classgets(SEXP vec, SEXP klass)
     return R_NilValue;/*- just for -Wall */
 }
 
-/* oldClass() : */
+/* oldClass<-(), primitive */
 SEXP attribute_hidden do_classgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
+    check1arg(args, call, "x");
+
     if (NAMED(CAR(args)) == 2) SETCAR(args, duplicate(CAR(args)));
     if (length(CADR(args)) == 0) SETCADR(args, R_NilValue);
+    if(IS_S4_OBJECT(CAR(args)))
+      UNSET_S4_OBJECT(CAR(args));
     setAttrib(CAR(args), R_ClassSymbol, CADR(args));
+    SET_NAMED(CAR(args), 0);
     return CAR(args);
 }
 
+/* oldClass, primitive */
 SEXP attribute_hidden do_class(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
-    return getAttrib(CAR(args), R_ClassSymbol);
+    check1arg(args, call, "x");
+    SEXP x = CAR(args), s3class;
+    if(IS_S4_OBJECT(x)) {
+      if((s3class = S3Class(x)) != R_NilValue) {
+	return s3class;
+      }
+    } /* else */
+    return getAttrib(x, R_ClassSymbol);
 }
 
 /* character elements corresponding to the syntactic types in the
@@ -588,13 +614,62 @@ SEXP R_data_class(SEXP obj, Rboolean singleString)
     return value;
 }
 
+static SEXP s_dot_S3Class = 0;
+
+static SEXP R_S4_extends_table = 0;
+
+ 
+static SEXP cache_class(const char *class, SEXP klass) {
+    if(!R_S4_extends_table) {
+	R_S4_extends_table = R_NewHashedEnv(R_NilValue, ScalarInteger(0));
+	R_PreserveObject(R_S4_extends_table);
+    }
+    if(isNull(klass)) { /* retrieve cached value */
+	SEXP val;
+	val = findVarInFrame(R_S4_extends_table, install(class));
+	return (val == R_UnboundValue) ? klass : val;
+    }
+    defineVar(install(class), klass, R_S4_extends_table);
+    return klass;
+}
+
+static SEXP S4_extends(SEXP klass) {
+  static SEXP s_extends = 0, s_extendsForS3;
+    SEXP e, val; const char *class;
+    if(!s_extends) {
+	s_extends = install("extends");
+	s_extendsForS3 = install(".extendsForS3");
+	R_S4_extends_table = R_NewHashedEnv(R_NilValue, ScalarInteger(0));
+	R_PreserveObject(R_S4_extends_table);
+    }
+    /* sanity check for methods package available */
+    if(findVar(s_extends, R_GlobalEnv) == R_UnboundValue)
+        return klass;
+    class = translateChar(STRING_ELT(klass, 0)); /* TODO: include package attr. */
+    val = findVarInFrame(R_S4_extends_table, install(class));
+    if(val != R_UnboundValue)
+       return val;
+    PROTECT(e = allocVector(LANGSXP, 2));
+    SETCAR(e, s_extendsForS3);
+    val = CDR(e);
+    SETCAR(val, klass);
+    val = eval(e, R_MethodsNamespace);
+    cache_class(class, val);
+    UNPROTECT(1);
+    return(val);
+}
+
 /* Version for S3-dispatch */
 SEXP attribute_hidden R_data_class2 (SEXP obj)
 {
     SEXP klass = getAttrib(obj, R_ClassSymbol);
-    if(length(klass) > 0)
-	return(klass);
-    else {
+      if(length(klass) > 0) {
+	if(IS_S4_OBJECT(obj))
+	    return S4_extends(klass);
+	else
+	    return klass;
+      }
+      else { /* length(klass) == 0 */
 	SEXPTYPE t;
 	SEXP value, class0 = R_NilValue, dim = getAttrib(obj, R_DimSymbol);
 	int n = length(dim);
@@ -652,6 +727,16 @@ SEXP attribute_hidden R_data_class2 (SEXP obj)
 SEXP attribute_hidden R_do_data_class(SEXP call, SEXP op, SEXP args, SEXP env)
 {
   checkArity(op, args);
+  if(PRIMVAL(op) == 1) {
+      const char *class; SEXP klass;
+      check1arg(args, call, "class");
+      klass = CAR(args);
+      if(TYPEOF(klass) != STRSXP || LENGTH(klass) < 1)
+	  error("invalid class argument to internal .class_cache");
+      class = translateChar(STRING_ELT(klass, 0));
+      return cache_class(class, CADR(args));
+  }
+  check1arg(args, call, "x");
   return R_data_class(CAR(args), FALSE);
 }
 
@@ -660,11 +745,30 @@ SEXP attribute_hidden do_namesgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
     checkArity(op, args);
+    check1arg(args, call, "x");
+
     if (DispatchOrEval(call, op, "names<-", args, env, &ans, 0, 1))
 	return(ans);
+    /* Special case: removing non-existent names, to avoid a copy */
+    if (CADR(args) == R_NilValue &&
+	getAttrib(CAR(args), R_NamesSymbol) == R_NilValue)
+	return CAR(args);
     PROTECT(args = ans);
     if (NAMED(CAR(args)) == 2)
 	SETCAR(args, duplicate(CAR(args)));
+    if(IS_S4_OBJECT(CAR(args))) {
+	const char *klass = CHAR(STRING_ELT(R_data_class(CAR(args), FALSE), 0));
+	if(getAttrib(CAR(args), R_NamesSymbol) == R_NilValue) {
+	    /* S4 class w/o a names slot or attribute */
+	    if(TYPEOF(CAR(args)) == S4SXP)
+		error(_("Class '%s' has no 'names' slot"), klass);
+	    else
+		warning(_("Class '%s' has no 'names' slot; assigning a names attribute will create an invalid object"), klass);
+	}
+	else if(TYPEOF(CAR(args)) == S4SXP)
+	    error(_("Illegal to use names()<- to set the 'names' slot in a non-vector class ('%s')"), klass);
+	/* else, go ahead, but can't check validity of replacement*/
+    }
     if (CADR(args) != R_NilValue) {
 	PROTECT(call = allocList(2));
 	SET_TYPEOF(call, LANGSXP);
@@ -675,6 +779,7 @@ SEXP attribute_hidden do_namesgets(SEXP call, SEXP op, SEXP args, SEXP env)
     }
     setAttrib(CAR(args), R_NamesSymbol, CADR(args));
     UNPROTECT(1);
+    SET_NAMED(CAR(args), 0);
     return CAR(args);
 }
 
@@ -742,7 +847,7 @@ SEXP namesgets(SEXP vec, SEXP val)
 	    else
 		SET_TAG(s, R_NilValue);
     }
-    else if (isVector(vec))
+    else if (isVector(vec) || IS_S4_OBJECT(vec))
 	/* Normal case */
 	installAttrib(vec, R_NamesSymbol, val);
     else
@@ -756,11 +861,13 @@ SEXP attribute_hidden do_names(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
     checkArity(op, args);
+    check1arg(args, call, "x");
     if (DispatchOrEval(call, op, "names", args, env, &ans, 0, 1))
 	return(ans);
     PROTECT(args = ans);
     ans = CAR(args);
-    if (isVector(ans) || isList(ans) || isLanguage(ans))
+    if (isVector(ans) || isList(ans) || isLanguage(ans) ||
+	IS_S4_OBJECT(ans))
 	ans = getAttrib(ans, R_NamesSymbol);
     else ans =  R_NilValue;
     UNPROTECT(1);
@@ -770,13 +877,17 @@ SEXP attribute_hidden do_names(SEXP call, SEXP op, SEXP args, SEXP env)
 SEXP attribute_hidden do_dimnamesgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
+
     checkArity(op, args);
+    check1arg(args, call, "x");
+
     if (DispatchOrEval(call, op, "dimnames<-", args, env, &ans, 0, 1))
 	return(ans);
     PROTECT(args = ans);
     if (NAMED(CAR(args)) > 1) SETCAR(args, duplicate(CAR(args)));
     setAttrib(CAR(args), R_DimNamesSymbol, CADR(args));
     UNPROTECT(1);
+    SET_NAMED(CAR(args), 0);
     return CAR(args);
 }
 
@@ -787,19 +898,10 @@ static SEXP dimnamesgets1(SEXP val1)
     if (LENGTH(val1) == 0) return R_NilValue;
     /* if (isObject(val1)) dispatch on as.character.foo, but we don't
        have the context at this point to do so */
-    if (inherits(val1, "factor")) { /* mimic as.character.factor */
-	int i, n = LENGTH(val1);
-	SEXP labels = getAttrib(val1, install("levels"));
-	PROTECT(this2 = allocVector(STRSXP, n));
-	for(i = 0; i < n; i++) {
-	    int ii = INTEGER(val1)[i];
-	    SET_STRING_ELT(this2, i,
-			   (ii == NA_INTEGER) ? NA_STRING
-			   : STRING_ELT(labels, ii - 1));
-	}
-	UNPROTECT(1);
-	return this2;
-    }
+
+    if (inherits(val1, "factor"))  /* mimic as.character.factor */
+        return asCharacterFactor(val1);
+
     if (!isString(val1)) { /* mimic as.character.default */
 	PROTECT(this2 = coerceVector(val1, STRSXP));
 	SET_ATTRIB(this2, R_NilValue);
@@ -813,7 +915,7 @@ static SEXP dimnamesgets1(SEXP val1)
 
 SEXP dimnamesgets(SEXP vec, SEXP val)
 {
-    SEXP dims, top;
+    SEXP dims, top, newval;
     int i, k;
 
     PROTECT(vec);
@@ -823,15 +925,20 @@ SEXP dimnamesgets(SEXP vec, SEXP val)
 	error(_("'dimnames' applied to non-array"));
     /* This is probably overkill, but you never know; */
     /* there may be old pair-lists out there */
+    /* There are, when this gets used as names<- for 1-d arrays */
     if (!isPairList(val) && !isNewList(val))
 	error(_("'dimnames' must be a list"));
     dims = getAttrib(vec, R_DimSymbol);
-    if ((k = LENGTH(dims)) != length(val))
+    if ((k = LENGTH(dims)) < length(val))
 	error(_("length of 'dimnames' [%d] must match that of 'dims' [%d]"),
 	      length(val), k);
+    if (length(val) == 0) {
+	removeAttrib(vec, R_DimNamesSymbol);
+	UNPROTECT(2);
+	return vec;
+    }
     /* Old list to new list */
     if (isList(val)) {
-	SEXP newval;
 	newval = allocVector(VECSXP, k);
 	for (i = 0; i < k; i++) {
 	    SET_VECTOR_ELT(newval, i, CAR(val));
@@ -840,6 +947,14 @@ SEXP dimnamesgets(SEXP vec, SEXP val)
 	UNPROTECT(1);
 	PROTECT(val = newval);
     }
+    if (length(val) > 0 && length(val) < k) {
+	newval = lengthgets(val, k);
+	UNPROTECT(1);
+	PROTECT(val = newval);
+    }
+    if (k != length(val))
+	error(_("length of 'dimnames' [%d] must match that of 'dims' [%d]"),
+	      length(val), k);
     for (i = 0; i < k; i++) {
 	SEXP _this = VECTOR_ELT(val, i);
 	if (_this != R_NilValue) {
@@ -860,13 +975,14 @@ SEXP dimnamesgets(SEXP vec, SEXP val)
 	    SET_TAG(val, install(translateChar(STRING_ELT(top, i++))));
     }
     UNPROTECT(2);
-    return (vec);
+    return vec;
 }
 
 SEXP attribute_hidden do_dimnames(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
     checkArity(op, args);
+    check1arg(args, call, "x");
     if (DispatchOrEval(call, op, "dimnames", args, env, &ans, 0, 1))
 	return(ans);
     PROTECT(args = ans);
@@ -879,6 +995,7 @@ SEXP attribute_hidden do_dim(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
     checkArity(op, args);
+    check1arg(args, call, "x");
     if (DispatchOrEval(call, op, "dim", args, env, &ans, 0, 1))
 	return(ans);
     PROTECT(args = ans);
@@ -889,16 +1006,25 @@ SEXP attribute_hidden do_dim(SEXP call, SEXP op, SEXP args, SEXP env)
 
 SEXP attribute_hidden do_dimgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans;
+    SEXP ans, x;
     checkArity(op, args);
     if (DispatchOrEval(call, op, "dim<-", args, env, &ans, 0, 1))
 	return(ans);
+    x = CAR(args);
+    /* Duplication might be expensive */
+    if (CADR(args) == R_NilValue) {
+	SEXP s;
+	for (s = ATTRIB(x); s != R_NilValue; s = CDR(s))
+	    if (TAG(s) == R_DimSymbol || TAG(s) == R_NamesSymbol) break;
+	if (s == R_NilValue) return x;
+    }
     PROTECT(args = ans);
-    if (NAMED(CAR(args)) > 1) SETCAR(args, duplicate(CAR(args)));
-    setAttrib(CAR(args), R_DimSymbol, CADR(args));
-    setAttrib(CAR(args), R_NamesSymbol, R_NilValue);
+    if (NAMED(x) > 1) SETCAR(args, x = duplicate(x));
+    setAttrib(x, R_DimSymbol, CADR(args));
+    setAttrib(x, R_NamesSymbol, R_NilValue);
     UNPROTECT(1);
-    return CAR(args);
+    SET_NAMED(x, 0);
+    return x;
 }
 
 SEXP dimgets(SEXP vec, SEXP val)
@@ -920,8 +1046,14 @@ SEXP dimgets(SEXP vec, SEXP val)
     if (ndim == 0)
 	error(_("length-0 dimension vector is invalid"));
     total = 1;
-    for (i = 0; i < ndim; i++)
+    for (i = 0; i < ndim; i++) {
+	/* need this test first as NA_INTEGER is < 0 */
+	if (INTEGER(val)[i] == NA_INTEGER)
+	    error(_("the dims contain missing values"));
+	if (INTEGER(val)[i] < 0)
+	    error(_("the dims contain negative values"));
 	total *= INTEGER(val)[i];
+    }
     if (total != len)
 	error(_("dims [product %d] do not match the length of object [%d]"), total, len);
     removeAttrib(vec, R_DimNamesSymbol);
@@ -934,6 +1066,9 @@ SEXP attribute_hidden do_attributes(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP attrs, names, namesattr, value;
     int nvalues;
+
+    checkArity(op, args);
+    check1arg(args, call, "x");
     namesattr = R_NilValue;
     attrs = ATTRIB(CAR(args));
     nvalues = length(attrs);
@@ -978,9 +1113,16 @@ SEXP attribute_hidden do_attributes(SEXP call, SEXP op, SEXP args, SEXP env)
 SEXP attribute_hidden do_levelsgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans;
+
     checkArity(op, args);
+    check1arg(args, call, "x");
+
     if (DispatchOrEval(call, op, "levels<-", args, env, &ans, 0, 1))
+	/* calls, e.g., levels<-.factor() */
 	return(ans);
+    if(!isNull(CADR(args)) && any_duplicated(CADR(args), FALSE))
+	warningcall(call, _("duplicated levels will not be allowed in factors anymore"));
+/* TODO errorcall(call, _("duplicated levels are not allowed in factors anymore")); */
     PROTECT(args = ans);
     if (NAMED(CAR(args)) > 1) SETCAR(args, duplicate(CAR(args)));
     setAttrib(CAR(args), R_LevelsSymbol, CADR(args));
@@ -1001,6 +1143,9 @@ SEXP attribute_hidden do_attributesgets(SEXP call, SEXP op, SEXP args, SEXP env)
 
     /* Extract the arguments from the argument list */
 
+    checkArity(op, args);
+    check1arg(args, call, "x");
+
     object = CAR(args);
     attrs = CADR(args);
 
@@ -1095,29 +1240,41 @@ fairly minor.  LT */
 
 SEXP attribute_hidden do_attr(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP s, t, tag = R_NilValue, alist;
+    SEXP ap, argList, s, t, tag = R_NilValue, alist, ans;
     const char *str;
-    int n, nargs = length(args), exact = 0;
+    int nargs = length(args), exact = 0;
     enum { NONE, PARTIAL, PARTIAL2, FULL } match = NONE;
 
     if (nargs < 2 || nargs > 3)
 	errorcall(call, "either 2 or 3 arguments are required");
 
-    s = CAR(args);
-    t = CADR(args);
-    if(nargs == 3) {
-	exact = asLogical(CADDR(args));
-	if(exact == NA_LOGICAL) exact = 0;
-    }
-
+    /* argument matching */
+    PROTECT(ap = list3(R_NilValue, R_NilValue, R_NilValue));
+    SET_TAG(ap,  install("x"));
+    SET_TAG(CDR(ap), install("which"));
+    SET_TAG(CDDR(ap), install("exact"));
+    argList = matchArgs(ap, args, call);
+    UNPROTECT(1); /* ap */
+    PROTECT(argList);
+    s = CAR(argList);
+    t = CADR(argList);
     if (!isString(t))
 	errorcall(call, _("'which' must be of mode character"));
     if (length(t) != 1)
 	errorcall(call, _("exactly one attribute 'which' must be given"));
 
-    if(STRING_ELT(t, 0) == NA_STRING) return R_NilValue;
+    if(nargs == 3) {
+	exact = asLogical(CADDR(args));
+	if(exact == NA_LOGICAL) exact = 0;
+    }
+
+
+    if(STRING_ELT(t, 0) == NA_STRING) {
+	UNPROTECT(1);
+	return R_NilValue;
+    }
     str = translateChar(STRING_ELT(t, 0));
-    n = strlen(str);
+    size_t n = strlen(str);
 
     /* try to find a match among the attributes list */
     for (alist = ATTRIB(s); alist != R_NilValue; alist = CDR(alist)) {
@@ -1129,7 +1286,7 @@ SEXP attribute_hidden do_attr(SEXP call, SEXP op, SEXP args, SEXP env)
 		match = FULL;
 		break;
 	    }
-	    else if (match == PARTIAL || match == PARTIAL2) {
+    else if (match == PARTIAL || match == PARTIAL2) {
 		/* this match is partial and we already have a partial match,
 		   so the query is ambiguous and we will return R_NilValue
 		   unless a full match comes up.
@@ -1141,7 +1298,10 @@ SEXP attribute_hidden do_attr(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 	}
     }
-    if (match == PARTIAL2) return R_NilValue;
+    if (match == PARTIAL2) {
+	UNPROTECT(1);
+	return R_NilValue;
+    }
 
     /* Unless a full match has been found, check for a "names" attribute.
        This is stored via TAGs on pairlists, and via rownames on 1D arrays.
@@ -1161,6 +1321,7 @@ SEXP attribute_hidden do_attr(SEXP call, SEXP op, SEXP args, SEXP env)
 	    if(t != R_NilValue && R_warn_partial_match_attr)
 		warningcall(call, _("partial match of '%s' to '%s'"), str,
 			    CHAR(PRINTNAME(tag)));
+	    UNPROTECT(1);
 	    return t;
 	}
 	else if (match == PARTIAL && strcmp(CHAR(PRINTNAME(tag)), "names")) {
@@ -1169,24 +1330,32 @@ SEXP attribute_hidden do_attr(SEXP call, SEXP op, SEXP args, SEXP env)
 	       query is ambiguous and we return R_NilValue.  If there is no
 	       "names" attribute, then the partially matched one, which is
 	       the current value of tag, can be used. */
-	    if (getAttrib(s, R_NamesSymbol) != R_NilValue)
+	    if (getAttrib(s, R_NamesSymbol) != R_NilValue) {
+		UNPROTECT(1);
 		return R_NilValue;
+	    }
 	}
     }
 
-    if (match == NONE  || (exact && match != FULL))
+    if (match == NONE  || (exact && match != FULL)) {
+	UNPROTECT(1);
 	return R_NilValue;
+    }
     if (match == PARTIAL && R_warn_partial_match_attr)
 	warningcall(call, _("partial match of '%s' to '%s'"), str,
 		    CHAR(PRINTNAME(tag)));
 
-    return getAttrib(s, tag);
+    ans =  getAttrib(s, tag);
+    UNPROTECT(1);
+    return ans;
 }
 
 SEXP attribute_hidden do_attrgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    /*  attr(obj, "<name>")  <-  value  */
-    SEXP obj, name;
+    /*  attr(x, which = "<name>")  <-  value  */
+    SEXP obj, name, ap, argList;
+
+    checkArity(op, args);
 
     obj = CAR(args);
     if (NAMED(obj) == 2)
@@ -1194,11 +1363,25 @@ SEXP attribute_hidden do_attrgets(SEXP call, SEXP op, SEXP args, SEXP env)
     else
 	PROTECT(obj);
 
-    name = CADR(args);
+    /* argument matching */
+    PROTECT(ap = list3(R_NilValue, R_NilValue, R_NilValue));
+    SET_TAG(ap,  install("x"));
+    SET_TAG(CDR(ap), install("which"));
+    SET_TAG(CDDR(ap), install("value"));
+    argList = matchArgs(ap, args, call);
+    UNPROTECT(1); /* ap */
+    PROTECT(argList);
+
+    name = CADR(argList);
     if (!isValidString(name) || STRING_ELT(name, 0) == NA_STRING)
 	error(_("'name' must be non-null character string"));
+    /* TODO?  if (isFactor(obj) && !strcmp(asChar(name), "levels"))
+     * ---         if(any_duplicated(CADDR(args)))
+     *                  error(.....)
+     */
     setAttrib(obj, name, CADDR(args));
-    UNPROTECT(1);
+    UNPROTECT(2);
+    SET_NAMED(obj, 0);
     return obj;
 }
 
@@ -1253,6 +1436,7 @@ static SEXP s_setDataPart;
 
 static void init_slot_handling(void) {
     s_dot_Data = install(".Data");
+    s_dot_S3Class = install(".S3Class");
     s_getDataPart = install("getDataPart");
     s_setDataPart = install("setDataPart");
     /* create and preserve an object that is NOT R_NilValue, and is used
@@ -1295,6 +1479,12 @@ static SEXP set_data_part(SEXP obj,  SEXP rhs) {
     return(val);
 }
 
+SEXP S3Class(SEXP obj)
+{
+    if(!s_dot_S3Class) init_slot_handling();
+    return getAttrib(obj, s_dot_S3Class);
+}
+
 /* Slots are stored as attributes to
    provide some back-compatibility
 */
@@ -1313,12 +1503,16 @@ int R_has_slot(SEXP obj, SEXP name) {
     if(isString(name)) name = install(CHAR(STRING_ELT(name, 0)))
 
     R_SLOT_INIT;
-    if(name == s_dot_Data)
+    if(name == s_dot_Data && TYPEOF(obj) != S4SXP)
 	return(1);
     /* else */
     return(getAttrib(obj, name) != R_NilValue);
 }
 
+/* the @ operator, and its assignment form.  Processed much like $
+   (see do_subset3) but without S3-style methods.
+*/
+/* currently, R_get_slot() ["methods"] is a trivial wrapper for this: */
 SEXP R_do_slot(SEXP obj, SEXP name) {
     R_SLOT_INIT;
     if(name == s_dot_Data)
@@ -1327,6 +1521,11 @@ SEXP R_do_slot(SEXP obj, SEXP name) {
 	SEXP value = getAttrib(obj, name);
 	if(value == R_NilValue) {
 	    SEXP input = name, classString;
+	    if(name == s_dot_S3Class) /* defaults to class(obj) */
+	        return R_data_class(obj, FALSE);
+	    else if(name == R_NamesSymbol &&
+		    TYPEOF(obj) == VECSXP) /* needed for namedList class */
+	        return value;
 	    if(isSymbol(name) ) {
 		input = PROTECT(ScalarString(PRINTNAME(name)));
 		classString = getAttrib(obj, R_ClassSymbol);
@@ -1354,7 +1553,14 @@ SEXP R_do_slot(SEXP obj, SEXP name) {
 }
 #undef R_SLOT_INIT
 
+/* currently, R_set_slot() ["methods"] is a trivial wrapper for this: */
 SEXP R_do_slot_assign(SEXP obj, SEXP name, SEXP value) {
+#ifndef _R_ver_le_2_11_x_
+    if (isNull(obj))/* cannot use !IS_S4_OBJECT(obj), because
+		     *  slot(obj, name, check=FALSE) <- value  must work on
+		     * "pre-objects", currently only in makePrototypeFromClassDef() */
+	error(_("attempt to set slot on NULL object"));
+#endif
     PROTECT(obj); PROTECT(value);
 				/* Ensure that name is a symbol */
     if(isString(name) && LENGTH(name) == 1)
@@ -1369,29 +1575,26 @@ SEXP R_do_slot_assign(SEXP obj, SEXP name, SEXP value) {
 
     if(name == s_dot_Data) {	/* special handling */
 	obj = set_data_part(obj, value);
-	UNPROTECT(2);
-	return obj;
-    }
-    if(isNull(value))		/* Slots, but not attributes, can be NULL.*/
-	value = pseudo_NULL;	/* Store a special symbol instead. */
+    } else {
+	if(isNull(value))		/* Slots, but not attributes, can be NULL.*/
+	    value = pseudo_NULL;	/* Store a special symbol instead. */
 
-    setAttrib(obj, name, value);
+#ifdef _R_ver_le_2_11_x_
+	setAttrib(obj, name, value);
+#else
+	/* simplified version of setAttrib(obj, name, value);
+	   here we do *not* treat "names", "dimnames", "dim", .. specially : */
+	PROTECT(name);
+	if (NAMED(value)) value = duplicate(value);
+	SET_NAMED(value, NAMED(value) | NAMED(obj));
+	UNPROTECT(1);
+	installAttrib(obj, name, value);
+#endif
+    }
     UNPROTECT(2);
     return obj;
 }
 
-#ifdef UNUSED
-SEXP R_pseudo_null() {
-    if(pseudo_NULL == 0)
-	init_slot_handling();
-    return pseudo_NULL;
-}
-#endif
-
-
-/* the @ operator, and its assignment form.  Processed much like $
-   (see do_subset3) but without S3-style methods.
-*/
 SEXP attribute_hidden do_AT(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP  nlist, object, ans, klass;
@@ -1409,11 +1612,11 @@ SEXP attribute_hidden do_AT(SEXP call, SEXP op, SEXP args, SEXP env)
     if(nlist != s_dot_Data && !IS_S4_OBJECT(object)) {
 	klass = getAttrib(object, R_ClassSymbol);
 	if(length(klass) == 0)
-	    warning(_("trying to get slot \"%s\" from an object of a basic class (\"%s\") with no slots"),
+	    error(_("trying to get slot \"%s\" from an object of a basic class (\"%s\") with no slots"),
 		  CHAR(PRINTNAME(nlist)),
 		  CHAR(STRING_ELT(R_data_class(object, FALSE), 0)));
 	else
-	    warning(_("trying to get slot \"%s\" from an object (class \"%s\") that is not an S4 object "),
+	    error(_("trying to get slot \"%s\" from an object (class \"%s\") that is not an S4 object "),
 		  CHAR(PRINTNAME(nlist)),
 		  translateChar(STRING_ELT(klass, 0)));
     }
@@ -1422,3 +1625,59 @@ SEXP attribute_hidden do_AT(SEXP call, SEXP op, SEXP args, SEXP env)
     UNPROTECT(1);
     return ans;
 }
+
+/* Return a suitable S3 object (OK, the name of the routine comes from
+   an earlier version and isn't quite accurate.) If there is a .S3Class
+   slot convert to that S3 class.
+   Otherwise, unless type == S4SXP, look for a .Data or .xData slot.  The
+   value of type controls what's wanted.  If it is S4SXP, then ONLY
+   .S3class is used.  If it is ANYSXP, don't check except that automatic
+   conversion from the current type only applies for classes that extend
+   one of the basic types (i.e., not S4SXP).  For all other types, the
+   recovered data must match the type.
+   Because S3 objects can't have type S4SXP, .S3Class slot is not searched
+   for in that type object, unless ONLY that class is wanted.
+   (Obviously, this is another routine that has accumulated barnacles and
+   should at some time be broken into separate parts.)
+*/
+SEXP attribute_hidden
+R_getS4DataSlot(SEXP obj, SEXPTYPE type)
+{
+  static SEXP s_xData, s_dotData; SEXP value = R_NilValue;
+  if(!s_xData) {
+    s_xData = install(".xData");
+    s_dotData = install(".Data");
+  }
+  if(TYPEOF(obj) != S4SXP || type == S4SXP) {
+    SEXP s3class = S3Class(obj);
+    if(s3class == R_NilValue && type == S4SXP)
+      return R_NilValue;
+    PROTECT(s3class);
+    if(NAMED(obj)) obj = duplicate(obj);
+    UNPROTECT(1);
+    if(s3class != R_NilValue) {/* replace class with S3 class */
+      setAttrib(obj, R_ClassSymbol, s3class);
+      setAttrib(obj, s_dot_S3Class, R_NilValue); /* not in the S3 class */
+    }
+    else { /* to avoid inf. recursion, must unset class attribute */
+      setAttrib(obj, R_ClassSymbol, R_NilValue);
+    }
+    UNSET_S4_OBJECT(obj);
+    if(type == S4SXP)
+      return obj;
+    value = obj;
+  }
+  else
+      value = getAttrib(obj, s_dotData);
+  if(value == R_NilValue)
+      value = getAttrib(obj, s_xData);
+/* the mechanism for extending abnormal types.  In the future, would b
+   good to consolidate under the ".Data" slot, but this has
+   been used to mean S4 objects with non-S4 type, so for now
+   a secondary slot name, ".xData" is used to avoid confusion
+*/  if(value != R_NilValue &&
+     (type == ANYSXP || type == TYPEOF(value)))
+     return value;
+  else
+     return R_NilValue;
+}
