@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2000   The R Development Core Team.
+ *  Copyright (C) 1998-2006   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  *
  *  Contexts:
@@ -32,22 +32,25 @@
  *  recursively.  The memory is reclaimed naturally on return through
  *  the recursions (the R_GlobalContext pointer needs adjustment).
  *
- *  A context contains the following information:
+ *  A context contains the following information (and more):
  *
  *	nextcontext	the next level context
  *	cjmpbuf		longjump information for non-local return
  *	cstacktop	the current level of the pointer protection stack
  *	callflag	the context "type"
- *	call		the call (name of function) that effected this
- *			context
+ *	call		the call (name of function, or expression to
+ *			get the function) that effected this
+ *			context if a closure, otherwise often NULL.
+ *	callfun		the function, if this was a closure.
  *	cloenv		for closures, the environment of the closure.
  *	sysparent	the environment the closure was called from
  *	conexit		code for on.exit calls, to be executed in cloenv
  *			at exit from the closure (normal or abnormal).
  *	cend		a pointer to function which executes if there is
  *			non-local return (i.e. an error)
- *	cenddata	a void pointer do data for cend to use
+ *	cenddata	a void pointer to data for cend to use
  *	vmax		the current setting of the R_alloc stack
+ *	srcref		the srcref at the time of the call
  *
  *  Context types can be one of:
  *
@@ -71,7 +74,7 @@
  *
  *	void begincontext(RCNTXT *cptr, int flags,
  *			  SEXP syscall, SEXP env, SEXP
- *			  sysp, SEXP promargs)
+ *			  sysp, SEXP promargs, SEXP callfun)
  *
  *  which sets up the context pointed to by cptr in the appropriate way.
  *  When the context goes "out-of-scope" a call to
@@ -104,8 +107,8 @@
 #include <config.h>
 #endif
 
-#include "Defn.h"
-
+#define R_USE_SIGNALS 1
+#include <Defn.h>
 
 /* R_run_onexits - runs the conexit/cend code for all contexts from
    R_GlobalContext down to but not including the argument context.
@@ -113,23 +116,35 @@
    determines the argument is responsible for making sure
    CTXT_TOPLEVEL's are not crossed unless appropriate. */
 
-void R_run_onexits(RCNTXT *cptr)
+void attribute_hidden R_run_onexits(RCNTXT *cptr)
 {
     RCNTXT *c;
 
     for (c = R_GlobalContext; c != cptr; c = c->nextcontext) {
 	if (c == NULL)
-	    error("bad target context--should NEVER happen;\n"
-		  "please bug.report() [R_run_onexits]");
+	    error(_("bad target context--should NEVER happen;\n\
+please bug.report() [R_run_onexits]"));
 	if (c->cend != NULL) {
 	    void (*cend)(void *) = c->cend;
 	    c->cend = NULL; /* prevent recursion */
+	    R_HandlerStack = c->handlerstack;
+	    R_RestartStack = c->restartstack;
 	    cend(c->cenddata);
 	}
 	if (c->cloenv != R_NilValue && c->conexit != R_NilValue) {
 	    SEXP s = c->conexit;
 	    c->conexit = R_NilValue; /* prevent recursion */
+	    R_HandlerStack = c->handlerstack;
+	    R_RestartStack = c->restartstack;
 	    PROTECT(s);
+	    /* Since these are run before any jumps rather than after
+	       jumping to the context where the exit handler was set
+	       we need to make sure there is enough room on the
+	       evaluation stack in case the jump is from handling a
+	       stack overflow. To be safe it is good to also call
+	       R_CheckStack. LT */
+	    R_Expressions = R_Expressions_keep + 500;
+	    R_CheckStack();
 	    eval(s, c->cloenv);
 	    UNPROTECT(1);
 	}
@@ -140,15 +155,33 @@ void R_run_onexits(RCNTXT *cptr)
 /* R_restore_globals - restore global variables from a target context
    before a LONGJMP.  The target context itself is not restored here
    since this is done slightly differently in jumpfun below, in
-   errors.c:jump_now, and in main.c:ParseBrwoser.  Eventually these
+   errors.c:jump_now, and in main.c:ParseBrowser.  Eventually these
    three should be unified so there is only one place where a LONGJMP
    occurs. */
 
-void R_restore_globals(RCNTXT *cptr)
+void attribute_hidden R_restore_globals(RCNTXT *cptr)
 {
     R_PPStackTop = cptr->cstacktop;
     R_EvalDepth = cptr->evaldepth;
     vmaxset(cptr->vmax);
+    R_interrupts_suspended = cptr->intsusp;
+    R_HandlerStack = cptr->handlerstack;
+    R_RestartStack = cptr->restartstack;
+    while (R_PendingPromises != cptr->prstack) {
+	/* The value 2 installed in PRSEEN 2 allows forcePromise in
+	   eval.c to signal a warning when asked to evaluate a promise
+	   whose evaluation has been interrupted by a jump. */
+	SET_PRSEEN(R_PendingPromises->promise, 2);
+	R_PendingPromises = R_PendingPromises->next;
+    }
+    /* Need to reset R_Expressions in case we are jumping after
+       handling a stack overflow. */
+    R_Expressions = R_Expressions_keep;
+    R_BCNodeStackTop = cptr->nodestack;
+#ifdef BC_INT_STACK
+    R_BCIntStackTop = cptr->intstack;
+#endif
+    R_Srcref = cptr->srcref;
 }
 
 
@@ -156,7 +189,7 @@ void R_restore_globals(RCNTXT *cptr)
 
 static void jumpfun(RCNTXT * cptr, int mask, SEXP val)
 {
-    int savevis = R_Visible;
+    Rboolean savevis = R_Visible;
 
     /* run onexit/cend code for all contexts down to but not including
        the jump target */
@@ -167,13 +200,13 @@ static void jumpfun(RCNTXT * cptr, int mask, SEXP val)
 
     R_ReturnedValue = val;
     R_GlobalContext = cptr; /* this used to be set to
-                               cptr->nextcontext for non-toplevel
-                               jumps (with the context set back at the
-                               SETJMP for restarts).  Changing this to
-                               always using cptr as the new global
-                               context should simplify some code and
-                               perhaps allow loops to be handled with
-                               fewer SETJMP's.  LT */
+			       cptr->nextcontext for non-toplevel
+			       jumps (with the context set back at the
+			       SETJMP for restarts).  Changing this to
+			       always using cptr as the new global
+			       context should simplify some code and
+			       perhaps allow loops to be handled with
+			       fewer SETJMP's.  LT */
     R_restore_globals(R_GlobalContext);
 
     LONGJMP(cptr->cjmpbuf, mask);
@@ -182,8 +215,10 @@ static void jumpfun(RCNTXT * cptr, int mask, SEXP val)
 
 /* begincontext - begin an execution context */
 
+/* begincontext and endcontext are used in dataentry.c and modules */
 void begincontext(RCNTXT * cptr, int flags,
-		  SEXP syscall, SEXP env, SEXP sysp, SEXP promargs)
+		  SEXP syscall, SEXP env, SEXP sysp,
+		  SEXP promargs, SEXP callfun)
 {
     cptr->nextcontext = R_GlobalContext;
     cptr->cstacktop = R_PPStackTop;
@@ -195,7 +230,17 @@ void begincontext(RCNTXT * cptr, int flags,
     cptr->conexit = R_NilValue;
     cptr->cend = NULL;
     cptr->promargs = promargs;
+    cptr->callfun = callfun;
     cptr->vmax = vmaxget();
+    cptr->intsusp = R_interrupts_suspended;
+    cptr->handlerstack = R_HandlerStack;
+    cptr->restartstack = R_RestartStack;
+    cptr->prstack = R_PendingPromises;
+    cptr->nodestack = R_BCNodeStackTop;
+#ifdef BC_INT_STACK
+    cptr->intstack = R_BCIntStackTop;
+#endif
+    cptr->srcref = R_Srcref;
     R_GlobalContext = cptr;
 }
 
@@ -204,9 +249,11 @@ void begincontext(RCNTXT * cptr, int flags,
 
 void endcontext(RCNTXT * cptr)
 {
+    R_HandlerStack = cptr->handlerstack;
+    R_RestartStack = cptr->restartstack;
     if (cptr->cloenv != R_NilValue && cptr->conexit != R_NilValue ) {
 	SEXP s = cptr->conexit;
-	int savevis = R_Visible;
+	Rboolean savevis = R_Visible;
 	cptr->conexit = R_NilValue; /* prevent recursion */
 	PROTECT(s);
 	eval(s, cptr->cloenv);
@@ -219,7 +266,7 @@ void endcontext(RCNTXT * cptr)
 
 /* findcontext - find the correct context */
 
-void findcontext(int mask, SEXP env, SEXP val)
+void attribute_hidden findcontext(int mask, SEXP env, SEXP val)
 {
     RCNTXT *cptr;
     cptr = R_GlobalContext;
@@ -228,8 +275,8 @@ void findcontext(int mask, SEXP env, SEXP val)
 	     cptr != NULL && cptr->callflag != CTXT_TOPLEVEL;
 	     cptr = cptr->nextcontext)
 	    if (cptr->callflag & CTXT_LOOP && cptr->cloenv == env )
-	        jumpfun(cptr, mask, val);
-        error("No loop to break from, jumping to top level");
+		jumpfun(cptr, mask, val);
+	error(_("no loop for break/next, jumping to top level"));
     }
     else {				/* return; or browser */
 	for (cptr = R_GlobalContext;
@@ -237,10 +284,21 @@ void findcontext(int mask, SEXP env, SEXP val)
 	     cptr = cptr->nextcontext)
 	    if ((cptr->callflag & mask) && cptr->cloenv == env)
 		jumpfun(cptr, mask, val);
-	error("No function to return from, jumping to top level");
+	error(_("no function to return from, jumping to top level"));
     }
 }
 
+void attribute_hidden R_JumpToContext(RCNTXT *target, int mask, SEXP val)
+{
+    RCNTXT *cptr;
+    for (cptr = R_GlobalContext;
+	 cptr != NULL && cptr->callflag != CTXT_TOPLEVEL;
+	 cptr = cptr->nextcontext)
+	if (cptr == target)
+	    jumpfun(cptr, mask, val);
+    error(_("target context is not on the stack"));
+}
+
 
 /* R_sysframe - look back up the context stack until the */
 /* nth closure context and return that cloenv. */
@@ -248,7 +306,7 @@ void findcontext(int mask, SEXP env, SEXP val)
 /* negative n counts back from the current frame */
 /* positive n counts up from the globalEnv */
 
-SEXP R_sysframe(int n, RCNTXT *cptr)
+SEXP attribute_hidden R_sysframe(int n, RCNTXT *cptr)
 {
     if (n == 0)
 	return(R_GlobalEnv);
@@ -259,7 +317,8 @@ SEXP R_sysframe(int n, RCNTXT *cptr)
 	n = -n;
 
     if(n < 0)
-	errorcall(R_GlobalContext->call,"not that many enclosing environments");
+	errorcall(R_GlobalContext->call,
+		  _("not that many frames on the stack"));
 
     while (cptr->nextcontext != NULL) {
 	if (cptr->callflag & CTXT_FUNCTION ) {
@@ -274,7 +333,8 @@ SEXP R_sysframe(int n, RCNTXT *cptr)
     if(n == 0 && cptr->nextcontext == NULL)
 	return R_GlobalEnv;
     else
-	error("sys.frame: not that many enclosing functions");
+	errorcall(R_GlobalContext->call,
+		  _("not that many frames on the stack"));
     return R_NilValue;	   /* just for -Wall */
 }
 
@@ -285,12 +345,13 @@ SEXP R_sysframe(int n, RCNTXT *cptr)
 /* It would be much simpler if sysparent just returned cptr->sysparent */
 /* but then we wouldn't be compatible with S. */
 
-int R_sysparent(int n, RCNTXT *cptr)
+int attribute_hidden R_sysparent(int n, RCNTXT *cptr)
 {
     int j;
     SEXP s;
     if(n <= 0)
-	errorcall(R_ToplevelContext->call,"only positive arguments are allowed");
+	errorcall(R_ToplevelContext->call,
+		  _("only positive values of 'n' are allowed"));
     while (cptr->nextcontext != NULL && n > 1) {
 	if (cptr->callflag & CTXT_FUNCTION )
 	    n--;
@@ -317,7 +378,7 @@ int R_sysparent(int n, RCNTXT *cptr)
     return n;
 }
 
-int framedepth(RCNTXT *cptr)
+int attribute_hidden framedepth(RCNTXT *cptr)
 {
     int nframe = 0;
     while (cptr->nextcontext != NULL) {
@@ -328,68 +389,64 @@ int framedepth(RCNTXT *cptr)
     return nframe;
 }
 
-SEXP R_syscall(int n, RCNTXT *cptr)
+SEXP attribute_hidden R_syscall(int n, RCNTXT *cptr)
 {
     /* negative n counts back from the current frame */
     /* positive n counts up from the globalEnv */
+    SEXP result;
+    
     if (n > 0)
-	n = framedepth(cptr)-n;
+	n = framedepth(cptr) - n;
     else
 	n = - n;
-    if(n < 0 )
-	errorcall(R_GlobalContext->call, "illegal frame number");
+    if(n < 0)
+	errorcall(R_GlobalContext->call,
+		  _("not that many frames on the stack"));
+    while (cptr->nextcontext != NULL) {
+	if (cptr->callflag & CTXT_FUNCTION ) {
+	    if (n == 0) {
+	    	PROTECT(result = duplicate(cptr->call));
+	    	if (cptr->srcref && !isNull(cptr->srcref))
+	    	    setAttrib(result, R_SrcrefSymbol, duplicate(cptr->srcref));
+	    	UNPROTECT(1);
+	    	return result;
+	    } else
+		n--;
+	}
+	cptr = cptr->nextcontext;
+    }
+    if (n == 0 && cptr->nextcontext == NULL) {
+	PROTECT(result = duplicate(cptr->call));
+	if (cptr->srcref && !isNull(cptr->srcref))
+	    setAttrib(result, R_SrcrefSymbol, duplicate(cptr->srcref));
+	UNPROTECT(1);
+	return result;
+    }
+    errorcall(R_GlobalContext->call, _("not that many frames on the stack"));
+    return R_NilValue;	/* just for -Wall */
+}
+
+SEXP attribute_hidden R_sysfunction(int n, RCNTXT *cptr)
+{
+    if (n > 0)
+	n = framedepth(cptr) - n;
+    else
+	n = - n;
+    if (n < 0)
+	errorcall(R_GlobalContext->call,
+		  _("not that many frames on the stack"));
     while (cptr->nextcontext != NULL) {
 	if (cptr->callflag & CTXT_FUNCTION ) {
 	    if (n == 0)
-		return (duplicate(cptr->call));
+		return duplicate(cptr->callfun);  /***** do we need to DUP? */
 	    else
 		n--;
 	}
 	cptr = cptr->nextcontext;
     }
     if (n == 0 && cptr->nextcontext == NULL)
-	return (duplicate(cptr->call));
-    errorcall(R_GlobalContext->call, "not that many enclosing functions");
-    return R_NilValue;	/* just for -Wall */
-}
-
-SEXP R_sysfunction(int n, RCNTXT *cptr)
-{
-    SEXP s, t;
-    if (n > 0)
-	n = framedepth(cptr) - n;
-    else
-	n = - n;
-    if (n < 0 )
-	errorcall(R_GlobalContext->call, "illegal frame number");
-    while (cptr->nextcontext != NULL) {
-	if (cptr->callflag & CTXT_FUNCTION ) {
-	    if (n == 0) {
-		s = CAR(cptr->call);
-		if (isSymbol(s))
-		    t = findVar1(s, cptr->sysparent, FUNSXP, 1);
-		else if( isLanguage(s) )
-		    t = eval(s, cptr->sysparent);
-		else if( isFunction(s) )
-		    t = s;
-		else
-		    t = R_NilValue;
-		while (TYPEOF(t) == PROMSXP) 
-		    t = eval(s, cptr->sysparent); 
-		return t;
-	    }
-	    else
-		n--;
-	}
-	cptr = cptr->nextcontext;
-    }
-    if (n == 0 && cptr->nextcontext == NULL){
-	s = findVar(CAR(cptr->call), cptr->sysparent);
-	while (TYPEOF(s) == PROMSXP) 
-	    s = eval(s, cptr->sysparent); 
-	return s;
-    }
-    errorcall(R_GlobalContext->call, "not that many enclosing functions");
+	return duplicate(cptr->callfun);  /***** do we need to DUP? */
+    errorcall(R_GlobalContext->call, _("not that many frames on the stack"));
     return R_NilValue;	/* just for -Wall */
 }
 
@@ -399,7 +456,7 @@ SEXP R_sysfunction(int n, RCNTXT *cptr)
    then get the context of the call that owns the environment.  As it
    is, it will restart the wrong function if used in a promise.
    L.T. */
-SEXP do_restart(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_restart(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     RCNTXT *cptr;
 
@@ -409,27 +466,115 @@ SEXP do_restart(SEXP call, SEXP op, SEXP args, SEXP rho)
 	return(R_NilValue);
     for(cptr = R_GlobalContext->nextcontext; cptr!= R_ToplevelContext;
 	    cptr = cptr->nextcontext) {
-        if (cptr->callflag & CTXT_FUNCTION) {
-		SET_RESTART_BIT_ON(cptr->callflag);
-		break;
+	if (cptr->callflag & CTXT_FUNCTION) {
+	    SET_RESTART_BIT_ON(cptr->callflag);
+	    break;
 	}
     }
     if( cptr == R_ToplevelContext )
-	errorcall(call, "no function to restart");
+	error(_("no function to restart"));
     return(R_NilValue);
 }
 
+/* count how many contexts of the specified type are present on the stack */
+/* browser contexts are a bit special because they are transient and for  */
+/* any closure context with the debug bit set one will be created; so we  */
+/* need to count those as well                                            */
+int countContexts(int ctxttype, int browser) {
+    int n=0;
+    RCNTXT *cptr;
+
+    cptr = R_GlobalContext;
+    while( cptr != R_ToplevelContext) {
+        if( cptr->callflag == ctxttype ) 
+            n++;
+        else if( browser ) {
+           if(cptr->callflag & CTXT_FUNCTION && RDEBUG(cptr->cloenv) )
+              n++;
+        }
+        cptr = cptr->nextcontext;
+    }
+    return n;
+}
+  
+   
+/* functions to support looking up information about the browser */
+/* contexts that are in the evaluation stack */
+
+SEXP attribute_hidden do_sysbrowser(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    SEXP rval=R_NilValue;
+    RCNTXT *cptr;
+    int n;
+
+    checkArity(op, args);
+    n = asInteger(CAR(args));
+    if(n < 1 ) error(_("number of contexts must be positive"));
+
+    /* first find the closest  browser context */
+    cptr = R_GlobalContext;
+    while (cptr != R_ToplevelContext) {
+        if (cptr->callflag == CTXT_BROWSER) {
+                break;
+        }
+        cptr = cptr->nextcontext;
+    }
+    /* error if not a browser context */
+
+    if( !(cptr->callflag == CTXT_BROWSER) )
+        error(_("no browser context to query"));
+
+    switch (PRIMVAL(op)) {
+    case 1: /* text */
+    case 2: /* condition */
+        /* first rewind to the right place if needed */
+        /* note we want n>1, as we have already      */
+        /* rewound to the first context              */
+        if( n > 1 ) {
+           while (cptr != R_ToplevelContext && n > 0 ) {
+               if (cptr->callflag == CTXT_BROWSER) {
+                   n--;
+                   break;
+               }
+               cptr = cptr->nextcontext;
+           }
+        }
+        if( !(cptr->callflag == CTXT_BROWSER) )
+           error(_("not that many calls to browser are active"));
+
+        if( PRIMVAL(op) == 1 )
+            rval = CAR(cptr->promargs);
+        else
+            rval = CADR(cptr->promargs);
+        break;
+    case 3: /* turn on debugging n levels up */
+        while ( (cptr != R_ToplevelContext) && n > 0 ) {
+            if (cptr->callflag & CTXT_FUNCTION) 
+                  n--;
+            cptr = cptr->nextcontext;
+        } 
+        if( !(cptr->callflag & CTXT_FUNCTION) )
+           error(_("not that many functions on the call stack"));
+        else
+           SET_RDEBUG(cptr->cloenv, 1);
+        break;
+    }
+    return(rval);
+}
+
 /* An implementation of S's frame access functions. They usually count */
 /* up from the globalEnv while we like to count down from the currentEnv. */
 /* So if the argument is negative count down if positive count up. */
-/* We don't want to count the closure that do_sys is contained in so the */
+/* We don't want to count the closure that do_sys is contained in, so the */
 /* indexing is adjusted to handle this. */
 
-SEXP do_sys(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_sys(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    int i, n, nframe;
-    SEXP rval,t;
+    int i, n  = -1, nframe;
+    SEXP rval, t;
     RCNTXT *cptr;
+
+    checkArity(op, args);
     /* first find the context that sys.xxx needs to be evaluated in */
     cptr = R_GlobalContext;
     t = cptr->sysparent;
@@ -440,81 +585,77 @@ SEXP do_sys(SEXP call, SEXP op, SEXP args, SEXP rho)
 	cptr = cptr->nextcontext;
     }
 
-    if (length(args) == 1) {
-	t = eval(CAR(args), rho);
-	n = asInteger(t);
-    }
-    else
-	n = - 1;
+    if (length(args) == 1) n = asInteger(CAR(args));
 
-    if(n == NA_INTEGER)
-	errorcall(call, "invalid number of environment levels");
     switch (PRIMVAL(op)) {
     case 1: /* parent */
-	nframe = framedepth(cptr);
-	rval = allocVector(INTSXP,1);
-	i = nframe;
+	if(n == NA_INTEGER)
+	    error(_("invalid '%s' argument"), "n");
+	i = nframe = framedepth(cptr);
 	/* This is a pretty awful kludge, but the alternative would be
 	   a major redesign of everything... -pd */
 	while (n-- > 0)
 	    i = R_sysparent(nframe - i + 1, cptr);
-	INTEGER(rval)[0] = i;
-	return rval;
+	return ScalarInteger(i);
     case 2: /* call */
+	if(n == NA_INTEGER)
+	    error(_("invalid '%s' argument"), "which");
 	return R_syscall(n, cptr);
     case 3: /* frame */
+	if(n == NA_INTEGER)
+	    error(_("invalid '%s' argument"), "which");
 	return R_sysframe(n, cptr);
     case 4: /* sys.nframe */
-	rval=allocVector(INTSXP,1);
-	INTEGER(rval)[0]=framedepth(cptr);
-	return rval;
+	return ScalarInteger(framedepth(cptr));
     case 5: /* sys.calls */
-	nframe=framedepth(cptr);
-	PROTECT(rval=allocList(nframe));
+	nframe = framedepth(cptr);
+	PROTECT(rval = allocList(nframe));
 	t=rval;
-	for(i=1 ; i<=nframe; i++, t=CDR(t))
-	    SETCAR(t, R_syscall(i,cptr));
+	for(i = 1; i <= nframe; i++, t = CDR(t))
+	    SETCAR(t, R_syscall(i, cptr));
 	UNPROTECT(1);
 	return rval;
     case 6: /* sys.frames */
-	nframe=framedepth(cptr);
-	PROTECT(rval=allocList(nframe));
-	t=rval;
-	for(i=1 ; i<=nframe ; i++, t=CDR(t))
-	    SETCAR(t, R_sysframe(i,cptr));
+	nframe = framedepth(cptr);
+	PROTECT(rval = allocList(nframe));
+	t = rval;
+	for(i = 1; i <= nframe; i++, t = CDR(t))
+	    SETCAR(t, R_sysframe(i, cptr));
 	UNPROTECT(1);
 	return rval;
     case 7: /* sys.on.exit */
-	if( R_GlobalContext->nextcontext != NULL )
+	if( R_GlobalContext->nextcontext != NULL)
 	    return R_GlobalContext->nextcontext->conexit;
 	else
 	    return R_NilValue;
     case 8: /* sys.parents */
-	nframe=framedepth(cptr);
-	rval=allocVector(INTSXP,nframe);
-	for(i=0; i<nframe ; i++ )
-	    INTEGER(rval)[i]= R_sysparent(nframe-i,cptr);
+	nframe = framedepth(cptr);
+	rval = allocVector(INTSXP, nframe);
+	for(i = 0; i < nframe; i++)
+	    INTEGER(rval)[i] = R_sysparent(nframe - i, cptr);
 	return rval;
     case 9: /* sys.function */
+	if(n == NA_INTEGER)
+	    error(_("invalid '%s' value"), "which");
 	return(R_sysfunction(n, cptr));
     default:
-	error("internal error in do_sys");
+	error(_("internal error in 'do_sys'"));
 	return R_NilValue;/* just for -Wall */
     }
 }
 
-SEXP do_parentframe(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_parentframe(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     int n;
     SEXP t;
     RCNTXT *cptr;
 
- 
-    t = eval(CAR(args), rho);
+    checkArity(op, args);
+    t = CAR(args);
     n = asInteger(t);
 
     if(n == NA_INTEGER || n < 1 )
-	errorcall(call, "invalid number of environment levels");
+	error(_("invalid '%s' value"), "n");
 
     cptr = R_GlobalContext;
     t = cptr->sysparent;
@@ -522,7 +663,7 @@ SEXP do_parentframe(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if (cptr->callflag & CTXT_FUNCTION ) {
 	    if (cptr->cloenv == t)
 	    {
-		if (n == 1) 
+		if (n == 1)
 		    return cptr->sysparent;
 		n--;
 		t = cptr->sysparent;
@@ -543,15 +684,17 @@ Rboolean R_ToplevelExec(void (*fun)(void *), void *data)
 {
     RCNTXT thiscontext;
     RCNTXT * volatile saveToplevelContext;
-    volatile SEXP topExp;
+    volatile SEXP topExp, oldHStack;
     Rboolean result;
 
 
     PROTECT(topExp = R_CurrentExpr);
+    PROTECT(oldHStack = R_HandlerStack);
+    R_HandlerStack = R_NilValue;
     saveToplevelContext = R_ToplevelContext;
 
     begincontext(&thiscontext, CTXT_TOPLEVEL, R_NilValue, R_GlobalEnv,
-		 R_NilValue, R_NilValue);
+		 R_BaseEnv, R_NilValue, R_NilValue);
     if (SETJMP(thiscontext.cjmpbuf))
 	result = FALSE;
     else {
@@ -563,7 +706,8 @@ Rboolean R_ToplevelExec(void (*fun)(void *), void *data)
 
     R_ToplevelContext = saveToplevelContext;
     R_CurrentExpr = topExp;
-    UNPROTECT(1);
+    R_HandlerStack = oldHStack;
+    UNPROTECT(2);
 
     return result;
 }
@@ -572,16 +716,19 @@ Rboolean R_ToplevelExec(void (*fun)(void *), void *data)
 
 /*
   This is a simple interface for evaluating R expressions
-  from C with a guarantee that one will return to the 
-  point in the code from which the call was made.
+  from C with a guarantee that one will return to the
+  point in the code from which the call was made (if it does
+  return at all).
   This uses R_TopleveExec to do this.  It is important
-  in applications that embed R or wish to make general 
+  in applications that embed R or wish to make general
   callbacks to R with error handling.
 
   It is currently hidden with a data structure definition
   and C routine visible only here. The R_tryEval() is the
   only visible aspect. This can be lifted into the header
   files if necessary. (DTL)
+
+  R_tryEval is in Rinternals.h (so public), but not in the API.
  */
 typedef struct {
     SEXP expression;
@@ -597,28 +744,44 @@ protectedEval(void *d)
     if(data->env) {
 	env = data->env;
     }
-    data->val = eval(data->expression, env); 
+    data->val = eval(data->expression, env);
     PROTECT(data->val);
 }
 
 SEXP
 R_tryEval(SEXP e, SEXP env, int *ErrorOccurred)
 {
- Rboolean ok;
- ProtectedEvalData data;
+    Rboolean ok;
+    ProtectedEvalData data;
 
- data.expression = e;
- data.val = NULL;
- data.env = env;
+    data.expression = e;
+    data.val = NULL;
+    data.env = env;
 
- ok = R_ToplevelExec(protectedEval, &data);
- if(ErrorOccurred) {
-     *ErrorOccurred = (ok == FALSE);
- }
- if(ok == FALSE)
-     data.val = NULL;
- else
-     UNPROTECT(1);
+    ok = R_ToplevelExec(protectedEval, &data);
+    if (ErrorOccurred) {
+	*ErrorOccurred = (ok == FALSE);
+    }
+    if (ok == FALSE)
+	data.val = NULL;
+    else
+	UNPROTECT(1);
 
- return(data.val);
+    return(data.val);
+}
+
+/* Temporary hack to suppress error message printing around a
+   R_tryEval call for use in methods_list_dispatch.c; should be
+   replaced once we have a way of establishing error handlers from C
+   code (probably would want a calling handler if we want to allow
+   user-defined calling handlers to enter a debugger, for
+   example). LT */
+SEXP R_tryEvalSilent(SEXP e, SEXP env, int *ErrorOccurred)
+{
+    SEXP val;
+    Rboolean oldshow = R_ShowErrorMessages;
+    R_ShowErrorMessages = FALSE;
+    val = R_tryEval(e, env, ErrorOccurred);
+    R_ShowErrorMessages = oldshow;
+    return val;
 }
