@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1995-2010	The R Development Core Team
+ *  Copyright (C) 1995-2011	The R Core Team
  *  Copyright (C) 2003		The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,13 +33,19 @@
 			ysd += ym * ym;		\
 		    }
 
+#define ANS(I,J)  ans[I + J * ncx]
+
 /* Note that "if (kendall)" and	 "if (cor)" are used inside a double for() loop;
    which makes the code better readable -- and is hopefully dealt with
    by a smartly optimizing compiler
 */
 
+/** Compute   Cov(xx[], yy[])  or  Cor(.,.)  with n = length(xx)
+ */
 #define COV_PAIRWISE_BODY						\
-	    xx = &x[i * n];						\
+	long double sum, xmean =0., ymean =0., xsd, ysd, xm, ym;		\
+        int k, nobs, n1 = -1;	/* -Wall initializing */		\
+									\
 	    nobs = 0;							\
 	    if(!kendall) {						\
 		xmean = ymean = 0.;					\
@@ -99,27 +105,24 @@
 		else if(!kendall)					\
 		    sum /= n1;						\
 									\
-		ans[i + j * ncx] = sum;					\
+		ANS(i,j) = sum;						\
 	    }								\
 	    else							\
-		ans[i + j * ncx] = NA_REAL;
+		ANS(i,j) = NA_REAL
 
 
 static void cov_pairwise1(int n, int ncx, double *x,
 			  double *ans, Rboolean *sd_0, Rboolean cor,
 			  Rboolean kendall)
 {
-    LDOUBLE sum, xmean =0., ymean =0., xsd, ysd, xm, ym;
-    double *xx, *yy;
-    int i, j, k, nobs, n1 = -1;	/* -Wall initializing */
+    for (int i = 0 ; i < ncx ; i++) {
+	double *xx = &x[i * n];
+	for (int j = 0 ; j <= i ; j++) {
+	    double *yy = &x[j * n];
 
-    for (i = 0 ; i < ncx ; i++) {
-	for (j = 0 ; j <= i ; j++) {
-	    yy = &x[j * n];
+	    COV_PAIRWISE_BODY;
 
-	    COV_PAIRWISE_BODY
-
-	    ans[j + i * ncx] = ans[i + j * ncx];
+	    ANS(j,i) = ANS(i,j);
 	}
     }
 }
@@ -128,15 +131,12 @@ static void cov_pairwise2(int n, int ncx, int ncy, double *x, double *y,
 			  double *ans, Rboolean *sd_0, Rboolean cor,
 			  Rboolean kendall)
 {
-    LDOUBLE sum, xmean =0., ymean =0., xsd, ysd, xm, ym;
-    double *xx, *yy;
-    int i, j, k, nobs, n1 = -1;	/* -Wall initializing */
+    for (int i = 0 ; i < ncx ; i++) {
+	double *xx = &x[i * n];
+	for (int j = 0 ; j < ncy ; j++) {
+	    double *yy = &y[j * n];
 
-    for (i = 0 ; i < ncx ; i++) {
-	for (j = 0 ; j < ncy ; j++) {
-	    yy = &y[j * n];
-
-	    COV_PAIRWISE_BODY
+	    COV_PAIRWISE_BODY;
 	}
     }
 }
@@ -146,10 +146,8 @@ static void cov_pairwise2(int n, int ncx, int ncy, double *x, double *y,
 /* method = "complete" or "all.obs" (only difference: na_fail):
  *           --------      -------
 */
-#define ANS(I,J)  ans[I + J * ncx]
-
 #define COV_ini_0				\
-    LDOUBLE sum, tmp, xxm, yym;			\
+    long double sum, tmp, xxm, yym;			\
     double *xx, *yy;				\
     int i, j, k, n1=-1/* -Wall */
 
@@ -668,6 +666,8 @@ SEXP attribute_hidden do_cov(SEXP call, SEXP op, SEXP args, SEXP env)
 	na_fail = TRUE;
 	break;
     case 2:		/* complete */
+	/* did na.omit in R */
+	if (!LENGTH(x)) error(_("no complete element pairs"));
 	break;
     case 3:		/* pairwise.complete */
 	pair = TRUE;
