@@ -1,9 +1,7 @@
-
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2008  Robert Gentleman, Ross Ihaka and the
- *			      R Development Core Team
+ *  Copyright (C) 1997--2011  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +20,7 @@
  *
  *  IMPLEMENTATION NOTES:
  *
- *  Deparsing, has 3 layers.  The user interface, do_deparse, should
+ *  Deparsing has 3 layers.  The user interface, do_deparse, should
  *  not be called from an internal function, the actual deparsing needs
  *  to be done twice, once to count things up and a second time to put
  *  them into the string vector for return.  Printing this to a file
@@ -91,7 +89,9 @@
 #include <config.h>
 #endif
 
+#define R_USE_SIGNALS 1
 #include <Defn.h>
+#include <float.h> /* for DBL_DIG */
 #include <Print.h>
 #include <Fileio.h>
 
@@ -135,6 +135,7 @@ static void print2buff(const char *, LocalParseData *);
 static void printtab2buff(int, LocalParseData *);
 static void writeline(LocalParseData *);
 static void vector2buff(SEXP, LocalParseData *);
+static void src2buff1(SEXP, LocalParseData *);
 static Rboolean src2buff(SEXP, int, LocalParseData *);
 static void vec2buff(SEXP, LocalParseData *);
 static void linebreak(Rboolean *lbreak, LocalParseData *);
@@ -201,7 +202,7 @@ static SEXP deparse1WithCutoff(SEXP call, Rboolean abbrev, int cutoff,
     localData.opts = opts;
     localData.strvec = R_NilValue;
 
-    PrintDefaults(R_NilValue);/* from global options() */
+    PrintDefaults(); /* from global options() */
     savedigits = R_print.digits;
     R_print.digits = DBL_DIG;/* MAX precision */
 
@@ -229,7 +230,8 @@ static SEXP deparse1WithCutoff(SEXP call, Rboolean abbrev, int cutoff,
     }
     if(nlines > 0 && localData.linenumber < nlines)
 	svec = lengthgets(svec, localData.linenumber);
-    UNPROTECT(1); /* new version does not need to be protected */
+    UNPROTECT(1);
+    PROTECT(svec); /* protect from warning() allocating, PR#14356 */
     R_print.digits = savedigits;
     if ((opts & WARNINCOMPLETE) && localData.isS4)
 	warning(_("deparse of an S4 object will not be source()able"));
@@ -239,21 +241,47 @@ static SEXP deparse1WithCutoff(SEXP call, Rboolean abbrev, int cutoff,
 	warning(_("deparse may be not be source()able in R < 2.7.0"));
     /* somewhere lower down might have allocated ... */
     R_FreeStringBuffer(&(localData.buffer));
+    UNPROTECT(1);
     return svec;
 }
 
-/* deparse1line uses the maximum cutoff rather than the default */
+/* deparse1line concatenates all lines into one long one */
 /* This is needed in terms.formula, where we must be able */
 /* to deparse a term label into a single line of text so */
 /* that it can be reparsed correctly */
 SEXP deparse1line(SEXP call, Rboolean abbrev)
 {
-   SEXP temp;
-   Rboolean backtick=TRUE;
+    SEXP temp;
+    Rboolean backtick=TRUE;
+    int lines;
 
-   temp = deparse1WithCutoff(call, abbrev, MAX_Cutoff, backtick,
-			     SIMPLEDEPARSE, 1);
-   return(temp);
+    PROTECT(temp = deparse1WithCutoff(call, abbrev, MAX_Cutoff, backtick,
+			     SIMPLEDEPARSE, -1));
+    if ((lines = length(temp)) > 1) {
+	char *buf;
+	int i, len;
+	const void *vmax;
+	cetype_t enc = CE_NATIVE;
+	for (len=0, i = 0; i < length(temp); i++) {
+	    SEXP s = STRING_ELT(temp, i);
+	    cetype_t thisenc = getCharCE(s);
+	    len += strlen(CHAR(s));
+	    if (thisenc != CE_NATIVE) 
+	    	enc = thisenc; /* assume only one non-native encoding */ 
+	}    
+	vmax = vmaxget();
+	buf = R_alloc((size_t) len+lines, sizeof(char));
+	*buf = '\0';
+	for (i = 0; i < length(temp); i++) {
+	    strcat(buf, CHAR(STRING_ELT(temp, i)));
+	    if (i < lines - 1)
+	    	strcat(buf, "\n");
+	}
+	temp = ScalarString(mkCharCE(buf, enc));
+	vmaxset(vmax);
+    }		
+    UNPROTECT(1);	
+    return(temp);
 }
 
 SEXP attribute_hidden deparse1s(SEXP call)
@@ -268,12 +296,19 @@ SEXP attribute_hidden deparse1s(SEXP call)
 
 #include "Rconnections.h"
 
+static void con_cleanup(void *data)
+{
+    Rconnection con = data;
+    if(con->isopen) con->close(con);
+}
+
 SEXP attribute_hidden do_dput(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP saveenv, tval;
     int i, ifile, res;
     Rboolean wasopen, havewarned = FALSE, opts;
     Rconnection con = (Rconnection) 1; /* stdout */
+    RCNTXT cntxt;
 
     checkArity(op, args);
 
@@ -305,12 +340,13 @@ SEXP attribute_hidden do_dput(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    strcpy(con->mode, "w");
 	    if(!con->open(con)) error(_("cannot open the connection"));
 	    strcpy(con->mode, mode);
-	    if(!con->canwrite) {
-		con->close(con);
-		error(_("cannot write to this connection"));
-	    }
-	} else if(!con->canwrite)
-	    error(_("cannot write to this connection"));
+	    /* Set up a context which will close the connection on error */
+	    begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
+			 R_NilValue, R_NilValue);
+	    cntxt.cend = &con_cleanup;
+	    cntxt.cenddata = con;
+	}
+	if(!con->canwrite) error(_("cannot write to this connection"));
     }/* else: "Stdout" */
     for (i = 0; i < LENGTH(tval); i++)
 	if (ifile == 1)
@@ -322,7 +358,7 @@ SEXP attribute_hidden do_dput(SEXP call, SEXP op, SEXP args, SEXP rho)
 		warning(_("wrote too few characters"));
 	}
     UNPROTECT(1); /* tval */
-    if (!wasopen) con->close(con);
+    if(!wasopen) {endcontext(&cntxt); con->close(con);}
     return (CAR(args));
 }
 
@@ -334,6 +370,7 @@ SEXP attribute_hidden do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
     Rconnection con;
     int opts;
     const char *obj_name;
+    RCNTXT cntxt;
 
     checkArity(op, args);
 
@@ -389,15 +426,16 @@ SEXP attribute_hidden do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
 		strcpy(con->mode, "w");
 		if(!con->open(con)) error(_("cannot open the connection"));
 		strcpy(con->mode, mode);
-		if(!con->canwrite) {
-		    con->close(con);
-		    error(_("cannot write to this connection"));
-		}
-	    } else if(!con->canwrite)
-		error(_("cannot write to this connection"));
+		/* Set up a context which will close the connection on error */
+		begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
+			     R_NilValue, R_NilValue);
+		cntxt.cend = &con_cleanup;
+		cntxt.cenddata = con;
+	    } 
+	    if(!con->canwrite) error(_("cannot write to this connection"));
 	    for (i = 0, nout = 0; i < nobjs; i++) {
 		const char *s;
-		int extra = 6;
+		unsigned int extra = 6;
 		if (CAR(o) == R_UnboundValue) continue;
 		SET_STRING_ELT(outnames, nout++, STRING_ELT(names, i));
 		s = translateChar(STRING_ELT(names, i));
@@ -419,7 +457,7 @@ SEXP attribute_hidden do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
 		}
 		o = CDR(o);
 	    }
-	    if (!wasopen) con->close(con);
+	    if(!wasopen) {endcontext(&cntxt); con->close(con);}
 	}
     }
 
@@ -609,41 +647,16 @@ static void printcomment(SEXP s, LocalParseData *d)
 }
 
 
-static const char * backquotify(const char *s)
+static const char * quotify(SEXP name, int quote)
 {
-    /* backquotifying can at most double the length of the symbol in bytes,
-       plus surrounding quotes and terminator. */
-    static char buf[2*MAXIDSIZE+10];
-    char *t = buf;
+    const char *s = CHAR(name);
 
-    /* If a symbol is not a valid name, put it in backquotes, escaping
-     * any backquotes in the string itself */
-
-    /* NOTE: This could be fragile if sufficiently weird names are
-     * used. Ideally, we should insert backslash escapes, etc. */
+    /* If a symbol is not a valid name, put it in quotes, escaping
+     * any quotes in the string itself */
 
     if (isValidName(s) || *s == '\0') return s;
 
-    /* Don't translate 'impossible' error condition. */
-    if(strlen(s) > MAXIDSIZE)
-	error("symbol '%s' is too long to be a valid symbol", s);
-
-    *t++ = '`';
-    if(mbcslocale && !utf8locale) {
-	mbstate_t mb_st; int j, used;
-	mbs_init(&mb_st);
-	while( (used = Mbrtowc(NULL, s, MB_CUR_MAX, &mb_st)) ) {
-	    if ( *s  == '`' || *s == '\\' ) *t++ = '\\';
-	    for(j = 0; j < used; j++) *t++ = *s++;
-	}
-    } else
-	while ( *s ) {
-	    if ( *s  == '`' || *s == '\\' ) *t++ = '\\';
-	    *t++ = *s++;
-	}
-    *t++ = '`';
-    *t = '\0';
-    return buf;
+    return EncodeString(name, 0, quote, Rprt_adj_none);
 }
 
 /* This is the recursive part of deparsing. */
@@ -673,15 +686,9 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 	    print2buff("quote(", d);
 	}
 	if (localOpts & S_COMPAT) {
-	    const char *ss = CHAR(PRINTNAME(s));
-	    if (isValidName(ss)) print2buff(ss, d);
-	    else {
-		print2buff("\"", d);
-		print2buff(ss, d);
-		print2buff("\"", d);
-	    }
+	    print2buff(quotify(PRINTNAME(s), '"'), d);
 	} else if (d->backtick)
-	    print2buff(backquotify(CHAR(PRINTNAME(s))), d);
+	    print2buff(quotify(PRINTNAME(s), '`'), d);
 	else
 	    print2buff(CHAR(PRINTNAME(s)), d);
 	if (localOpts & QUOTEEXPRESSIONS) {
@@ -719,14 +726,10 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 	break;
     case CLOSXP:
 	if (localOpts & SHOWATTRIBUTES) attr1(s, d);
-	if ((d->opts & USESOURCE) &&
-	    (n = length(t = getAttrib(s, R_SourceSymbol))) > 0
-	    && isString(t)) {
-	    for(i = 0 ; i < n ; i++) {
-		print2buff(translateChar(STRING_ELT(t, i)), d);
-		writeline(d);
-	    }
-	} else {
+	if ((d->opts & USESOURCE)
+	    && !isNull(t = getAttrib(s, R_SrcrefSymbol))) 
+	    	src2buff1(t, d);
+	else {
 	    /* We have established that we don't want to use the
 	       source for this function */
 	    d->opts &= SIMPLE_OPTS & ~USESOURCE;
@@ -904,17 +907,10 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    break;
 		case PP_FUNCALL:
 		case PP_RETURN:
-		    if (isValidName(CHAR(PRINTNAME(op)))) /* ASCII */
-			print2buff(CHAR(PRINTNAME(op)), d);
-		    else if (d->backtick) {
-			print2buff("`", d);
-			print2buff(CHAR(PRINTNAME(op)), d);
-			print2buff("`", d);
-		    } else {
-			print2buff("\"", d);
-			print2buff(CHAR(PRINTNAME(op)), d);
-			print2buff("\"", d);
-		    }
+		    if (d->backtick) 
+		        print2buff(quotify(PRINTNAME(op), '`'), d);
+		    else 
+		    	print2buff(quotify(PRINTNAME(op), '"'), d);
 		    print2buff("(", d);
 		    d->inlist++;
 		    args2buff(s, 0, 0, d);
@@ -1085,18 +1081,10 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    }
 		    else {
 			if ( isSymbol(CAR(s)) ){
-			    const char *ss = CHAR(PRINTNAME(CAR(s)));
-			    if (isValidName(ss))
-				print2buff(ss, d);
-			    else if(d->opts & S_COMPAT) {
-				print2buff("\'", d);
-				print2buff(ss, d);
-				print2buff("\'", d);
-			    } else {
-				print2buff("`", d);
-				print2buff(ss, d);
-				print2buff("`", d);
-			    }
+			    if(d->opts & S_COMPAT) 
+			        print2buff(quotify(PRINTNAME(CAR(s)), '\''), d);
+			    else 
+				print2buff(quotify(PRINTNAME(CAR(s)), '`'), d);
 			}
 			else
 			    deparse2buff(CAR(s), d);
@@ -1144,12 +1132,10 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 	print2buff(tpb, d);
     }
 	break;
-#ifdef BYTECODE
     case BCODESXP:
 	d->sourceable = FALSE;
 	print2buff("<bytecode>", d);
 	break;
-#endif
     case WEAKREFSXP:
 	d->sourceable = FALSE;
 	print2buff("<weak reference>", d);
@@ -1185,7 +1171,7 @@ static void writeline(LocalParseData *d)
 
 static void print2buff(const char *strng, LocalParseData *d)
 {
-    int tlen, bufflen;
+    size_t tlen, bufflen;
 
     if (d->startline) {
 	d->startline = FALSE;
@@ -1340,24 +1326,31 @@ static void vector2buff(SEXP vector, LocalParseData *d)
     }
 }
 
+/* src2buff1: Deparse one source ref to buffer */
+
+static void src2buff1(SEXP srcref, LocalParseData *d)
+{
+    int i,n;
+    PROTECT(srcref);
+
+    PROTECT(srcref = lang2(install("as.character"), srcref));
+    PROTECT(srcref = eval(srcref, R_BaseEnv));
+    n = length(srcref);
+    for(i = 0 ; i < n ; i++) {
+	print2buff(translateChar(STRING_ELT(srcref, i)), d);
+	if(i < n-1) writeline(d);
+    }
+    UNPROTECT(3);
+}
+
 /* src2buff : Deparse source element k to buffer, if possible; return FALSE on failure */
 
 static Rboolean src2buff(SEXP sv, int k, LocalParseData *d)
 {
     SEXP t;
-    int i, n;
 
     if (TYPEOF(sv) == VECSXP && length(sv) > k && !isNull(t = VECTOR_ELT(sv, k))) {
-	PROTECT(t);
-
-	PROTECT(t = lang2(install("as.character"), t));
-	PROTECT(t = eval(t, R_BaseEnv));
-	n = length(t);
-	for(i = 0 ; i < n ; i++) {
-	    print2buff(translateChar(STRING_ELT(t, i)), d);
-	    if(i < n-1) writeline(d);
-	}
-	UNPROTECT(3);
+	src2buff1(t, d);
 	return TRUE;
     }
     else return FALSE;
@@ -1422,18 +1415,13 @@ static void args2buff(SEXP arglist, int lineb, int formals, LocalParseData *d)
 	if (TAG(arglist) != R_NilValue) {
 	    SEXP s = TAG(arglist);
 
-	    const char *ss = CHAR(PRINTNAME(s));
-	    if( s == R_DotsSymbol || isValidName(ss) )
-		print2buff(ss, d);
-	    else if(d->backtick) {
-		print2buff("`", d);
-		print2buff(ss, d);
-		print2buff("`", d);
-	    } else {
-		print2buff("\"", d);
-		print2buff(ss, d);
-		print2buff("\"", d);
-	    }
+	    if( s == R_DotsSymbol )
+		print2buff(CHAR(PRINTNAME(s)), d);
+	    else if(d->backtick) 
+		print2buff(quotify(PRINTNAME(s), '`'), d);
+	    else 
+	        print2buff(quotify(PRINTNAME(s), '"'), d);
+	    
 	    if(formals) {
 		if (CAR(arglist) != R_MissingArg) {
 		    print2buff(" = ", d);
