@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2007   The R Development Core Team.
+ *  Copyright (C) 1998-2011   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -42,7 +40,7 @@
  *
  *   2) Those used (and sometimes set) from C code;
  *	Either accessing and/or setting a global C variable,
- *	or just accessed by e.g.  GetOption(install("pager"), ..)
+ *	or just accessed by e.g.  GetOption1(install("pager"))
  *
  * A (complete?!) list of these (2):
  *
@@ -55,6 +53,7 @@
  *	"verbose"
  *	"keep.source"
  *	"keep.source.pkgs"
+ *	"browserNLdisabled"
 
  *	"de.cellwidth"		../unix/X11/ & ../gnuwin32/dataentry.c
  *	"device"
@@ -70,6 +69,7 @@
  *	"warn"
  *	"warning.length"
  *	"warning.expression"
+ *	"nwarnings"
 
  *
  * S additionally/instead has (and one might think about some)
@@ -105,18 +105,23 @@ static SEXP makeErrorCall(SEXP fun)
 }
 
 SEXP GetOption(SEXP tag, SEXP rho)
+{
+    return GetOption1(tag);
+}
+
+
+SEXP GetOption1(SEXP tag)
 {
     SEXP opt = findVar(Options(), R_BaseEnv);
-    if (!isList(opt))
-	error(_("corrupted options list"));
+    if (!isList(opt)) error(_("corrupted options list"));
     opt = FindTaggedItem(opt, tag);
     return CAR(opt);
 }
 
-int GetOptionWidth(SEXP rho)
+int GetOptionWidth(void)
 {
     int w;
-    w = asInteger(GetOption(install("width"), rho));
+    w = asInteger(GetOption1(install("width")));
     if (w < R_MIN_WIDTH_OPT || w > R_MAX_WIDTH_OPT) {
 	warning(_("invalid printing width, used 80"));
 	return 80;
@@ -124,10 +129,10 @@ int GetOptionWidth(SEXP rho)
     return w;
 }
 
-int GetOptionDigits(SEXP rho)
+int GetOptionDigits(void)
 {
     int d;
-    d = asInteger(GetOption(install("digits"), rho));
+    d = asInteger(GetOption1(install("digits")));
     if (d < R_MIN_DIGITS_OPT || d > R_MAX_DIGITS_OPT) {
 	warning(_("invalid printing digits, used 7"));
 	return 7;
@@ -136,15 +141,15 @@ int GetOptionDigits(SEXP rho)
 }
 
 
-int attribute_hidden Rf_GetOptionParAsk()
+Rboolean Rf_GetOptionDeviceAsk(void)
 {
     int ask;
-    ask = asLogical(GetOption(install("par.ask.default"), R_BaseEnv));
+    ask = asLogical(GetOption1(install("device.ask.default")));
     if(ask == NA_LOGICAL) {
-	warning(_("invalid par(\"par.ask.default\"), using FALSE"));
+	warning(_("invalid value for \"device.ask.default\", using FALSE"));
 	return FALSE;
     }
-   return ask != 0;
+    return ask != 0;
 }
 
 
@@ -220,9 +225,9 @@ void attribute_hidden InitOptions(void)
     char *p;
 
 #ifdef HAVE_RL_COMPLETION_MATCHES
-    PROTECT(v = val = allocList(13));
+    PROTECT(v = val = allocList(15));
 #else
-    PROTECT(v = val = allocList(12));
+    PROTECT(v = val = allocList(14));
 #endif
 
     SET_TAG(v, install("prompt"));
@@ -254,7 +259,7 @@ void attribute_hidden InitOptions(void)
     v = CDR(v);
 
     SET_TAG(v, install("check.bounds"));
-    SETCAR(v, ScalarLogical(0)); 	/* no checking */
+    SETCAR(v, ScalarLogical(0));	/* no checking */
     v = CDR(v);
 
     p = getenv("R_KEEP_PKG_SOURCE");
@@ -268,14 +273,22 @@ void attribute_hidden InitOptions(void)
     SETCAR(v, ScalarLogical(R_KeepSource));
     v = CDR(v);
 
-    SET_TAG(v, install("warnings.length"));
+    SET_TAG(v, install("warning.length"));
     SETCAR(v, ScalarInteger(1000));
     v = CDR(v);
 
+    SET_TAG(v, install("nwarnings"));
+    SETCAR(v, ScalarInteger(50));
+    v = CDR(v);
+
     SET_TAG(v, install("OutDec"));
     SETCAR(v, mkString("."));
     v = CDR(v);
 
+    SET_TAG(v, install("browserNLdisabled"));
+    SETCAR(v, ScalarLogical(FALSE));
+    v = CDR(v);
+
 #ifdef HAVE_RL_COMPLETION_MATCHES
     /* value from Rf_initialize_R */
     SET_TAG(v, install("rl_word_breaks"));
@@ -318,7 +331,7 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	PROTECT(sind = allocVector(INTSXP, n));  indx = INTEGER(sind);
 	for (i = 0; i < n; i++) indx[i] = i;
-	orderVector1(indx, n, names, TRUE, FALSE);
+	orderVector1(indx, n, names, TRUE, FALSE, R_NilValue);
 	PROTECT(value2 = allocVector(VECSXP, n));
 	PROTECT(names2 = allocVector(STRSXP, n));
 	for(i = 0; i < n; i++) {
@@ -339,7 +352,7 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     n = length(args);
     if (n == 1 && (isPairList(CAR(args)) || isVectorList(CAR(args)))
-        && TAG(args) == R_NilValue ) {
+	&& TAG(args) == R_NilValue ) {
 	args = CAR(args);
 	n = length(args);
     }
@@ -366,12 +379,12 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 	switch (TYPEOF(args)) {
 	case LISTSXP:
 	    argi = CAR(args);
-	    namei = EnsureString(TAG(args));
+	    namei = EnsureString(TAG(args)); /* gives "" for no tag */
 	    args = CDR(args);
 	    break;
 	case VECSXP:
 	    argi = VECTOR_ELT(args, i);
-	    namei = EnsureString(STRING_ELT(argnames, i));
+	    namei = STRING_ELT(argnames, i);
 	    break;
 	default: /* already checked, but be safe here */
 	    UNIMPLEMENTED_TYPE("options", args);
@@ -419,7 +432,7 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 		if (s == NA_STRING || length(s) == 0)
 		    error(_("invalid value for '%s'"), CHAR(namei));
 		/* We want to make sure these are in the native encoding */
-		SET_VECTOR_ELT(value, i, 
+		SET_VECTOR_ELT(value, i,
 			       SetOption(tag, mkString(translateChar(s))));
 	    }
 	    else if (streql(CHAR(namei), "prompt")) {
@@ -427,7 +440,7 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 		if (s == NA_STRING || length(s) == 0)
 		    error(_("invalid value for '%s'"), CHAR(namei));
 		/* We want to make sure these are in the native encoding */
-		SET_VECTOR_ELT(value, i, 
+		SET_VECTOR_ELT(value, i,
 			       SetOption(tag, mkString(translateChar(s))));
 	    }
 	    else if (streql(CHAR(namei), "contrasts")) {
@@ -445,24 +458,36 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    else if (streql(CHAR(namei), "warn")) {
 		if (!isNumeric(argi) || length(argi) != 1)
 		    error(_("invalid value for '%s'"), CHAR(namei));
-                SET_VECTOR_ELT(value, i, SetOption(tag, argi));
+		SET_VECTOR_ELT(value, i, SetOption(tag, argi));
 	    }
 	    else if (streql(CHAR(namei), "warning.length")) {
 		k = asInteger(argi);
 		if (k < 100 || k > 8170)
 		    error(_("invalid value for '%s'"), CHAR(namei));
 		R_WarnLength = k;
-                SET_VECTOR_ELT(value, i, SetOption(tag, argi));
+		SET_VECTOR_ELT(value, i, SetOption(tag, argi));
 	    }
 	    else if ( streql(CHAR(namei), "warning.expression") )  {
 		if( !isLanguage(argi) &&  ! isExpression(argi) )
 		    error(_("invalid value for '%s'"), CHAR(namei));
 		SET_VECTOR_ELT(value, i, SetOption(tag, argi));
 	    }
+	    else if (streql(CHAR(namei), "max.print")) {
+		k = asInteger(argi);
+		if (k < 1) error(_("invalid value for '%s'"), CHAR(namei));
+		SET_VECTOR_ELT(value, i, SetOption(tag, ScalarInteger(k)));
+	    }
+	    else if (streql(CHAR(namei), "nwarnings")) {
+		k = asInteger(argi);
+		if (k < 1) error(_("invalid value for '%s'"), CHAR(namei));
+		R_nwarnings = k;
+		R_CollectWarnings = 0; /* force a reset */
+		SET_VECTOR_ELT(value, i, SetOption(tag, ScalarInteger(k)));
+	    }
 	    else if ( streql(CHAR(namei), "error") ) {
-	        if(isFunction(argi))
+		if(isFunction(argi))
 		  argi = makeErrorCall(argi);
-	        else if( !isLanguage(argi) &&  !isExpression(argi) )
+		else if( !isLanguage(argi) &&  !isExpression(argi) )
 		    error(_("invalid value for '%s'"), CHAR(namei));
 		SET_VECTOR_ELT(value, i, SetOption(tag, argi));
 	    }
@@ -492,18 +517,11 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    }
 	    else if (streql(CHAR(namei), "max.contour.segments")) {
 		k = asInteger(argi);
-		if (k < 0 || k  == NA_INTEGER)
+		if (k < 0) // also many times above: rely on  NA_INTEGER  <  <finite_int>
 		    error(_("invalid value for '%s'"), CHAR(namei));
 		max_contour_segments = k;
 		SET_VECTOR_ELT(value, i, SetOption(tag, ScalarInteger(k)));
 	    }
-	    else if (streql(CHAR(namei), "warnEscapes")) {
-		if (TYPEOF(argi) != LGLSXP || LENGTH(argi) != 1)
-		    error(_("invalid value for '%s'"), CHAR(namei));
-		k = asLogical(argi);
-		R_WarnEscapes = k;
-		SET_VECTOR_ELT(value, i, SetOption(tag, ScalarLogical(k)));
-	    }
 	    else if (streql(CHAR(namei), "rl_word_breaks")) {
 		if (TYPEOF(argi) != STRSXP || LENGTH(argi) != 1)
 		    error(_("invalid value for '%s'"), CHAR(namei));
@@ -554,16 +572,33 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 		R_NShowCalls = k;
 		SET_VECTOR_ELT(value, i, SetOption(tag, ScalarInteger(k)));
 	    }
+	    else if (streql(CHAR(namei), "par.ask.default")) {
+		error(_("\"par.ask.default\" has been replaced by \"device.ask.default\""));
+	    }
+	    else if (streql(CHAR(namei), "browserNLdisabled")) {
+		if (TYPEOF(argi) != LGLSXP || LENGTH(argi) != 1)
+		    error(_("invalid value for '%s'"), CHAR(namei));
+		k = asLogical(argi);
+		if (k == NA_LOGICAL)
+		    error(_("invalid value for '%s'"), CHAR(namei));
+		R_DisableNLinBrowser = k;
+		SET_VECTOR_ELT(value, i, SetOption(tag, ScalarLogical(k)));
+	    }
 	    else {
 		SET_VECTOR_ELT(value, i, SetOption(tag, duplicate(argi)));
 	    }
 	    SET_STRING_ELT(names, i, namei);
 	}
 	else { /* querying arg */
+	    const char *tag;
 	    if (!isString(argi) || LENGTH(argi) <= 0)
-		error(R_MSG_IA);
-	    SET_VECTOR_ELT(value, i, duplicate(CAR(FindTaggedItem(options,
-				     install(CHAR(STRING_ELT(argi, 0)))))));
+		error(_("invalid argument"));
+	    tag = CHAR(STRING_ELT(argi, 0));
+	    if (streql(tag, "par.ask.default")) {
+		error(_("\"par.ask.default\" has been replaced by \"device.ask.default\""));
+	    }
+
+	    SET_VECTOR_ELT(value, i, duplicate(CAR(FindTaggedItem(options, install(tag)))));
 	    SET_STRING_ELT(names, i, STRING_ELT(argi, 0));
 	    R_Visible = TRUE;
 	}
