@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-1998	Robert Gentleman and Ross Ihaka.
- *  Copyright (C) 2000-2009	The R Development Core Team.
+ *  Copyright (C) 2000-2012	The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -81,21 +81,23 @@ static char tagbuf[TAGBUFLEN + 5];
 
 
 /* Used in X11 module for dataentry */
-void PrintDefaults(SEXP rho)
+/* 'rho' is unused */
+void PrintDefaults(void)
 {
     R_print.na_string = NA_STRING;
     R_print.na_string_noquote = mkChar("<NA>");
-    R_print.na_width = strlen(CHAR(R_print.na_string));
-    R_print.na_width_noquote = strlen(CHAR(R_print.na_string_noquote));
+    R_print.na_width = (int) strlen(CHAR(R_print.na_string));
+    R_print.na_width_noquote = (int) strlen(CHAR(R_print.na_string_noquote));
     R_print.quote = 1;
     R_print.right = Rprt_adj_left;
-    R_print.digits = GetOptionDigits(rho);
-    R_print.scipen = asInteger(GetOption(install("scipen"), rho));
+    R_print.digits = GetOptionDigits();
+    R_print.scipen = asInteger(GetOption1(install("scipen")));
     if (R_print.scipen == NA_INTEGER) R_print.scipen = 0;
-    R_print.max = asInteger(GetOption(install("max.print"), rho));
-    if (R_print.max == NA_INTEGER) R_print.max = 99999;
+    R_print.max = asInteger(GetOption1(install("max.print")));
+    if (R_print.max == NA_INTEGER || R_print.max < 0) R_print.max = 99999;
+    else if(R_print.max == INT_MAX) R_print.max--; // so we can add
     R_print.gap = 1;
-    R_print.width = GetOptionWidth(rho);
+    R_print.width = GetOptionWidth();
     R_print.useSource = USESOURCE;
 }
 
@@ -128,7 +130,7 @@ SEXP attribute_hidden do_prmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
     char *rowname = NULL, *colname = NULL;
 
     checkArity(op,args);
-    PrintDefaults(rho);
+    PrintDefaults();
     a = args;
     x = CAR(a); a = CDR(a);
     rowlab = CAR(a); a = CDR(a);
@@ -142,7 +144,7 @@ SEXP attribute_hidden do_prmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("invalid 'na.print' specification"));
 	R_print.na_string = R_print.na_string_noquote = STRING_ELT(naprint, 0);
 	R_print.na_width = R_print.na_width_noquote =
-	    strlen(CHAR(R_print.na_string));
+	    (int) strlen(CHAR(R_print.na_string));
     }
 
     if (length(rowlab) == 0) rowlab = R_NilValue;
@@ -154,7 +156,7 @@ SEXP attribute_hidden do_prmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     printMatrix(x, 0, getAttrib(x, R_DimSymbol), quote, R_print.right,
 		rowlab, collab, rowname, colname);
-    PrintDefaults(rho); /* reset, as na.print.etc may have been set */
+    PrintDefaults(); /* reset, as na.print.etc may have been set */
     return x;
 }/* do_prmatrix */
 
@@ -183,16 +185,20 @@ SEXP attribute_hidden do_printfunction(SEXP call, SEXP op, SEXP args, SEXP rho)
 static void PrintLanguageEtc(SEXP s, Rboolean useSource, Rboolean isClosure)
 {
     int i;
-    SEXP t = getAttrib(s, R_SourceSymbol);
-    if (!isString(t) || !useSource)
+    SEXP t = getAttrib(s, R_SrcrefSymbol);
+    if (!isInteger(t) || !useSource)
 	t = deparse1(s, 0, useSource | DEFAULTDEPARSE);
+    else {
+        PROTECT(t = lang2(install("as.character"), t));
+        t = eval(t, R_BaseEnv);
+        UNPROTECT(1);
+    }
+    PROTECT(t);
     for (i = 0; i < LENGTH(t); i++)
 	Rprintf("%s\n", CHAR(STRING_ELT(t, i))); /* translated */
+    UNPROTECT(1);
     if (isClosure) {
-#ifdef BYTECODE
-	if (isByteCode(BODY(s)))
-	    Rprintf("<bytecode: %p>\n", BODY(s));
-#endif
+	if (isByteCode(BODY(s))) Rprintf("<bytecode: %p>\n", BODY(s));
 	t = CLOENV(s);
 	if (t != R_GlobalEnv)
 	    Rprintf("%s\n", EncodeEnvironment(t));
@@ -218,7 +224,7 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
     Rboolean callShow = FALSE;
 
     checkArity(op, args);
-    PrintDefaults(rho);
+    PrintDefaults();
 
     x = CAR(args); args = CDR(args);
 
@@ -242,7 +248,7 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("invalid 'na.print' specification"));
 	R_print.na_string = R_print.na_string_noquote = STRING_ELT(naprint, 0);
 	R_print.na_width = R_print.na_width_noquote =
-	    strlen(CHAR(R_print.na_string));
+	    (int) strlen(CHAR(R_print.na_string));
     }
     args = CDR(args);
 
@@ -260,8 +266,9 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     if(!isNull(CAR(args))) {
 	R_print.max = asInteger(CAR(args));
-	if(R_print.max == NA_INTEGER)
+	if(R_print.max == NA_INTEGER || R_print.max < 0)
 	    error(_("invalid '%s' argument"), "max");
+	else if(R_print.max == INT_MAX) R_print.max--; // so we can add
     }
     args = CDR(args);
 
@@ -298,7 +305,7 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
 	CustomPrintValue(x, rho);
     }
 
-    PrintDefaults(rho); /* reset, as na.print etc may have been set */
+    PrintDefaults(); /* reset, as na.print etc may have been set */
     return x;
 }/* do_printdefault */
 
@@ -372,7 +379,7 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		if (LENGTH(tmp) == 1) {
 		    /* This can potentially overflow */
 		    const char *ctmp = translateChar(STRING_ELT(tmp, 0));
-		    int len = strlen(ctmp);
+		    int len = (int) strlen(ctmp);
 		    if(len < 100)
 			snprintf(pbuf, 115, "\"%s\"", ctmp);
 		    else {
@@ -417,7 +424,7 @@ static void PrintGenericVector(SEXP s, SEXP env)
     }
     else { /* .. no dim() .. */
 	names = getAttrib(s, R_NamesSymbol);
-	taglen = strlen(tagbuf);
+	taglen = (int) strlen(tagbuf);
 	ptag = tagbuf + taglen;
 	PROTECT(newcall = allocList(2));
 	SETCAR(newcall, install("print"));
@@ -466,7 +473,7 @@ static void PrintGenericVector(SEXP s, SEXP env)
 	    }
 	    Rprintf("\n");
 	    if(n_pr < ns)
-		Rprintf(" [ reached getOption(\"max.print\") -- omitted %d entries ]]\n",
+		Rprintf(" [ reached getOption(\"max.print\") -- omitted %d entries ]\n",
 			ns - n_pr);
 	}
 	else { /* ns = length(s) == 0 */
@@ -570,7 +577,7 @@ static void printList(SEXP s, SEXP env)
     }
     else {
 	i = 1;
-	taglen = strlen(tagbuf);
+	taglen = (int) strlen(tagbuf);
 	ptag = tagbuf + taglen;
 	PROTECT(newcall = allocList(2));
 	SETCAR(newcall, install("print"));
@@ -785,11 +792,9 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
     case EXTPTRSXP:
 	Rprintf("<pointer: %p>\n", R_ExternalPtrAddr(s));
 	break;
-#ifdef BYTECODE
     case BCODESXP:
 	Rprintf("<bytecode: %p>\n", s);
 	break;
-#endif
     case WEAKREFSXP:
 	Rprintf("<weak reference>\n");
 	break;
@@ -848,7 +853,8 @@ static void printAttributes(SEXP s, SEXP env, Rboolean useSlots)
 		if (TAG(a) == R_NamesSymbol)
 		    goto nextattr;
 	    }
-	    if(TAG(a) == R_CommentSymbol || TAG(a) == R_SourceSymbol || TAG(a) == R_SrcrefSymbol)
+	    if(TAG(a) == R_CommentSymbol || TAG(a) == R_SourceSymbol || TAG(a) == R_SrcrefSymbol
+	       || TAG(a) == R_WholeSrcrefSymbol || TAG(a) == R_SrcfileSymbol)
 		goto nextattr;
 	    if(useSlots)
 		sprintf(ptag, "Slot \"%s\":",
@@ -931,7 +937,7 @@ static void printAttributes(SEXP s, SEXP env, Rboolean useSlots)
 
 void attribute_hidden PrintValueEnv(SEXP s, SEXP env)
 {
-    PrintDefaults(env);
+    PrintDefaults();
     tagbuf[0] = '\0';
     PROTECT(s);
     if(isObject(s) || isFunction(s)) {
@@ -1002,7 +1008,7 @@ int F77_NAME(dblep0) (const char *label, int *nchar, double *data, int *ndata)
 {
     int k, nc = *nchar;
 
-    if(nc < 0) nc = strlen(label);
+    if(nc < 0) nc = (int) strlen(label);
     if(nc > 255) {
 	warning(_("invalid character length in dblepr"));
 	nc = 0;
@@ -1020,7 +1026,7 @@ int F77_NAME(intpr0) (const char *label, int *nchar, int *data, int *ndata)
 {
     int k, nc = *nchar;
 
-    if(nc < 0) nc = strlen(label);
+    if(nc < 0) nc = (int) strlen(label);
     if(nc > 255) {
 	warning(_("invalid character length in intpr"));
 	nc = 0;
@@ -1039,7 +1045,7 @@ int F77_NAME(realp0) (const char *label, int *nchar, float *data, int *ndata)
     int k, nc = *nchar, nd = *ndata;
     double *ddata;
 
-    if(nc < 0) nc = strlen(label);
+    if(nc < 0) nc = (int) strlen(label);
     if(nc > 255) {
 	warning(_("invalid character length in realpr"));
 	nc = 0;
