@@ -1747,7 +1747,7 @@ SEXP attribute_hidden do_ICUset(SEXP call, SEXP op, SEXP args, SEXP rho)
     UErrorCode  status = U_ZERO_ERROR;
 
     for (; args != R_NilValue; args = CDR(args)) {
-	if (isNull(TAG(args))) error(_("alll arguments must be named"));
+	if (isNull(TAG(args))) error(_("all arguments must be named"));
 	const char *this = CHAR(PRINTNAME(TAG(args)));
 	const char *s;
 
@@ -1867,3 +1867,103 @@ SEXP crc64ToString(SEXP in)
     snprintf(ans, 17, "%lx", (long unsigned int) crc);
     return mkString(ans);
 }
+
+
+/* Formerly a version in src/appl/binning.c */
+static void 
+C_bincount(double *x, int n, double *breaks, int nb, int *count,
+	   int right, int include_border)
+{
+    int i, lo, hi, nb1 = nb - 1, new, lft = !right;
+
+    for(i = 0; i < nb1; i++) count[i] = 0;
+    for(i = 0 ; i < n ; i++)
+	if(R_FINITE(x[i])) { // left in as a precaution
+	    lo = 0;
+	    hi = nb1;
+	    if(breaks[lo] <= x[i] &&
+	       (x[i] < breaks[hi] || (x[i] == breaks[hi] && include_border))) {
+		while(hi-lo >= 2) {
+		    new = (hi+lo)/2;
+		    if(x[i] > breaks[new] || (lft && x[i] == breaks[new]))
+			lo = new;
+		    else
+			hi = new;
+		}
+		count[lo] += 1;
+	    }
+	}
+}
+
+/* The R wrapper removed non-finite values and set the storage.mode */
+SEXP BinCount(SEXP x, SEXP breaks, SEXP right, SEXP lowest)
+{
+    if(TYPEOF(x) != REALSXP || TYPEOF(breaks) != REALSXP) 
+	error("invalid input");
+    int n = LENGTH(x), nB = LENGTH(breaks);
+    if (n == NA_INTEGER || nB == NA_INTEGER) error("invalid input");
+    int sr = asLogical(right), sl = asLogical(lowest);
+    if (sr == NA_INTEGER || sl == NA_INTEGER) error("invalid input");
+    SEXP counts;
+    PROTECT(counts = allocVector(INTSXP, nB - 1));
+    C_bincount(REAL(x), n, REAL(breaks), nB, INTEGER(counts), sr, sl);
+    UNPROTECT(1);
+    return counts;
+}
+
+#include <R_ext/Applic.h>
+
+/* The R wrapper set the storage.mode */
+SEXP BinCode(SEXP x, SEXP breaks, SEXP right, SEXP lowest)
+{
+    if(TYPEOF(x) != REALSXP || TYPEOF(breaks) != REALSXP) 
+	error("invalid input");
+    int n = LENGTH(x), nB = LENGTH(breaks);
+    if (n == NA_INTEGER || nB == NA_INTEGER) error("invalid input");
+    int sr = asLogical(right), sl = asLogical(lowest);
+    if (sr == NA_INTEGER || sl == NA_INTEGER) error("invalid input");
+    SEXP codes;
+    PROTECT(codes = allocVector(INTSXP, n));
+    int naok = 1;
+    bincode(REAL(x), &n, REAL(breaks), &nB, INTEGER(codes), &sr, &sl, &naok);
+    UNPROTECT(1);
+    return codes;
+}
+
+SEXP R_Tabulate(SEXP in, SEXP nbin)
+{
+    if(TYPEOF(in) != INTSXP)  error("invalid input");
+    int n = LENGTH(in);
+    if (n == NA_INTEGER) error("invalid input");
+    int nb = asInteger(nbin);
+    if (nb == NA_INTEGER || nb < 0) error("invalid input");
+    SEXP ans = allocVector(INTSXP, nb);
+    int *x = INTEGER(in), *y = INTEGER(ans);
+    memset(y, 0, nb * sizeof(int));
+    for(int i = 0 ; i < n ; i++)
+	if (x[i] != NA_INTEGER && x[i] > 0 && x[i] <= nb) y[x[i] - 1]++;
+    return ans;
+}
+
+SEXP FindIntervVec(SEXP xt, SEXP x, SEXP right, SEXP inside)
+{
+    if(TYPEOF(xt) != REALSXP || TYPEOF(x) != REALSXP) error("invalid input");
+    int n = LENGTH(xt);
+    if (n == NA_INTEGER) error("invalid input");
+    int nx = LENGTH(x);
+    int sr = asLogical(right), si = asLogical(inside);
+    if (sr == NA_INTEGER) error("invalid 'rightmost.closed' argument");
+    if (si == NA_INTEGER) error("invalid 'all.inside' argument");
+    SEXP ans = allocVector(INTSXP, nx);
+    double *rxt = REAL(xt), *rx = REAL(x);
+    int ii = 1;
+    for(int i = 0; i < nx; i++) {
+	if (ISNAN(REAL(x)[i])) ii = NA_INTEGER;
+	else {
+	    int mfl = si;
+	    ii = findInterval(rxt, n, rx[i], sr, si, ii, &mfl);
+	}
+	INTEGER(ans)[i] = ii;
+    }
+    return ans;
+}
