@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2011  The R Development Core Team
+ *  Copyright (C) 1997--2011  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -70,6 +70,7 @@
 typedef int (*X11IOhandler)(Display *);
 
 #include "devX11.h"
+#include "rlogo_icon.h" /* hard-coded ARGB icon */
 
 #include <Rmodules/RX11.h>
 
@@ -243,7 +244,7 @@ static void Cairo_update(pX11Desc xd)
 {
     if(inclose || !xd || !xd->buffered || xd->holdlevel > 0) return;
     cairo_paint(xd->xcc);
-    XDefineCursor(display, xd->window, arrow_cursor);
+    if (xd->type == WINDOW) XDefineCursor(display, xd->window, arrow_cursor);
     XSync(display, 0);
     xd->last = currentTime();
 }
@@ -342,7 +343,7 @@ static int Cairo_holdflush(pDevDesc dd, int level)
     if(xd->holdlevel == 0) {
 	if(xd->buffered) Cairo_update(xd);
 	else {
-	    XDefineCursor(display, xd->window, arrow_cursor);
+	  if (xd->type == WINDOW) XDefineCursor(display, xd->window, arrow_cursor);
 	    XSync(display, 0);
 	}
     } else if (old == 0) {
@@ -352,7 +353,7 @@ static int Cairo_holdflush(pDevDesc dd, int level)
 	    Cairo_update(xd);
 	    xd->holdlevel = level;
 	}
-	XDefineCursor(display, xd->window, watch_cursor);
+	if (xd->type == WINDOW) XDefineCursor(display, xd->window, watch_cursor);
 	XSync(display, 0);
     }
     return xd->holdlevel;
@@ -819,7 +820,7 @@ static void R_ProcessX11Events(void *data)
 {
     XEvent event;
 
-    while (displayOpen && XPending(display)) {
+    while (!R_isForkedChild && displayOpen && XPending(display)) {
 	XNextEvent(display, &event);
 	/* printf("%i\n",event.type); */
 	handleEvent(event);
@@ -1575,8 +1576,7 @@ X11_Open(pDevDesc dd, pX11Desc xd, const char *dsp,
 
 	    XStoreName(display, xd->window, xd->title);
 
-#ifndef USE_Xt
-	    /* For those too idle to make use of Xt (PR#14588) */
+	    /* See (PR#14588) */
 	    XClassHint *chint;
 	    chint = XAllocClassHint();
 	    if (chint) {
@@ -1585,7 +1585,13 @@ X11_Open(pDevDesc dd, pX11Desc xd, const char *dsp,
 		XSetClassHint(display, xd->window, chint);
 	    	XFree(chint);
 	    }
-#endif
+
+            /* set window icon */
+            XChangeProperty(display, xd->window,
+                            XInternAtom(display, "_NET_WM_ICON", False),
+                            XInternAtom(display, "CARDINAL", False), 32,
+                            PropModeReplace,
+                            (const unsigned char*) rlogo_icon, 2 + 48*48);
 
 	    /* set up protocols so that window manager sends */
 	    /* me an event when user "destroys" window */
@@ -1599,7 +1605,7 @@ X11_Open(pDevDesc dd, pX11Desc xd, const char *dsp,
 		cross_cursor = XCreateFontCursor(display, XC_crosshair);
 	    if(!watch_cursor)
 		watch_cursor = XCreateFontCursor(display, XC_watch) ;
-	    XDefineCursor(display, xd->window, arrow_cursor);
+	    if(xd->type==WINDOW) XDefineCursor(display, xd->window, arrow_cursor);
 
 #ifdef HAVE_WORKING_CAIRO
 	    if(xd->useCairo) {
@@ -2162,6 +2168,44 @@ static void X11_Path(double *x, double *y,
     warning(_("%s not available for this device"), "Path drawing");
 }
 
+static unsigned int makeX11Pixel(unsigned int * rasterImage, int pixel) {
+    return GetX11Pixel(R_RED(rasterImage[pixel]), 
+                       R_GREEN(rasterImage[pixel]), 
+                       R_BLUE(rasterImage[pixel]));
+}
+
+static void flipRaster(unsigned int *rasterImage,
+                       int imageWidth, int imageHeight,
+                       int invertX, int invertY,
+                       unsigned int *flippedRaster) {
+    int i, j;
+    int rowInc, rowOff, colInc, colOff;
+
+    if (invertX) {
+        colInc = -1;
+        colOff = imageWidth - 1;
+    } else {
+        colInc = 1;
+        colOff = 0;
+    }
+    if (invertY) {
+        rowInc = -1;
+        rowOff = imageHeight - 1;
+    } else {
+        rowInc = 1;
+        rowOff = 0;
+    }
+
+    for (i = 0; i < imageHeight ;i++) {
+        for (j = 0; j < imageWidth; j++) {
+            int row = (rowInc*i + rowOff);
+            int col = (colInc*j + colOff);
+            flippedRaster[i*imageWidth + j] = 
+                rasterImage[row*imageWidth + col];
+        }
+    }
+}
+
 static void X11_Raster(unsigned int *raster, int w, int h,
                       double x, double y, 
                       double width, double height,
@@ -2170,21 +2214,36 @@ static void X11_Raster(unsigned int *raster, int w, int h,
                       const pGEcontext gc, pDevDesc dd)
 {
     int i, j, pixel;
-    int imageWidth = (int) (width + .5);
-    int imageHeight = (int) (height + .5);
+    int imageWidth;
+    int imageHeight;
+    int invertX = 0;
+    int invertY = 0;
     double angle = rot*M_PI/180;
     pX11Desc xd = (pX11Desc) dd->deviceSpecific;
     XImage *image;
     unsigned int *rasterImage;
     const void *vmax = vmaxget();
    
-    if (imageHeight < 0) {
-        imageHeight = -imageHeight;
-        /* convert (x, y) from bottom-left to top-right */
+    if (height < 0) {
+        imageHeight = (int) -(height - .5);
+        /* convert (x, y) from bottom-left to top-left */
         y = y - imageHeight*cos(angle);
         if (angle != 0) {
             x = x - imageHeight*sin(angle);
         }
+    } else {
+        imageHeight = (int) (height + .5);
+        invertY = 1;
+    }
+
+    if (width < 0) {
+        imageWidth = (int) -(width - .5);
+        x = x - imageWidth*cos(angle);
+        if (angle != 0)
+            y = y - imageWidth*sin(angle);
+        invertX = 1;
+    } else {
+        imageWidth = (int) (width + .5);
     }
 
     rasterImage = (unsigned int *) R_alloc(imageWidth * imageHeight,
@@ -2215,8 +2274,8 @@ static void X11_Raster(unsigned int *raster, int w, int h,
         rotatedRaster = (unsigned int *) R_alloc(newW * newH, 
                                                  sizeof(unsigned int));
         R_GE_rasterRotate(resizedRaster, newW, newH, angle, rotatedRaster, gc,
-                          FALSE);
-
+                          FALSE);                          
+            
         /* 
          * Adjust (x, y) for resized and rotated image
          */
@@ -2228,7 +2287,17 @@ static void X11_Raster(unsigned int *raster, int w, int h,
         imageHeight = newH;
     }
 
-    image = XCreateImage(display, visual, depth, 
+    if (invertX || invertY) {
+        unsigned int *flippedRaster;
+
+        flippedRaster = (unsigned int *) R_alloc(imageWidth * imageHeight,
+                                                 sizeof(unsigned int));
+        flipRaster(rasterImage, imageWidth, imageHeight, 
+                   invertX, invertY, flippedRaster);
+        rasterImage = flippedRaster;
+    }
+
+    image = XCreateImage(display, visual, depth,
                          ZPixmap,
                          0, /* offset */
                          /* This is just provides (at least enough)
@@ -2245,11 +2314,8 @@ static void X11_Raster(unsigned int *raster, int w, int h,
 
     for (i = 0; i < imageHeight ;i++) {
         for (j = 0; j < imageWidth; j++) {
-            pixel = i * imageWidth + j;
-            XPutPixel(image, j, i, 
-                      GetX11Pixel(R_RED(rasterImage[pixel]), 
-                                  R_GREEN(rasterImage[pixel]), 
-                                  R_BLUE(rasterImage[pixel])));
+            pixel = i*imageWidth + j;
+            XPutPixel(image, j, i, makeX11Pixel(rasterImage, pixel));
         }
     }
 
@@ -2447,7 +2513,7 @@ static Rboolean X11_Locator(double *x, double *y, pDevDesc dd)
     if (xd->buffered) Cairo_update(xd);
 #endif
     R_ProcessX11Events((void*)NULL);	/* discard pending events */
-    XDefineCursor(display, xd->window, cross_cursor);
+    if(xd->type==WINDOW) XDefineCursor(display, xd->window, cross_cursor);
     XSync(display, 1);
     /* handle X events as normal until get a button */
     /* click in the desired device */
@@ -2476,8 +2542,10 @@ static Rboolean X11_Locator(double *x, double *y, pDevDesc dd)
 	else
 	    handleEvent(event);
     }
+    /* In case it got closed asynchronously, PR#14872 */
+    if (!displayOpen) return 0;
     /* if it was a Button1 succeed, otherwise fail */
-    XDefineCursor(display, xd->window, arrow_cursor);
+    if(xd->type==WINDOW) XDefineCursor(display, xd->window, arrow_cursor);
     XSync(display, 0);
     return (done == 1);
 }
@@ -2600,7 +2668,7 @@ static void X11_Mode(int mode, pDevDesc dd)
 	return;
     }
     if(mode == 1) {
-	XDefineCursor(display, xd->window, watch_cursor);
+	if(xd->type==WINDOW) XDefineCursor(display, xd->window, watch_cursor);
 	XSync(display, 0);
     }
     if(mode == 0) {
@@ -2613,7 +2681,7 @@ static void X11_Mode(int mode, pDevDesc dd)
 	}
 	if(xd->buffered) cairo_paint(xd->xcc);
 #endif
-	XDefineCursor(display, xd->window, arrow_cursor);
+	if(xd->type==WINDOW) XDefineCursor(display, xd->window, arrow_cursor);
 	XSync(display, 0);
     }
 }
@@ -3038,6 +3106,9 @@ static SEXP in_do_X11(SEXP call, SEXP op, SEXP args, SEXP env)
     checkArity(op, args);
     vmax = vmaxget();
 
+    if(R_isForkedChild)
+	error("a forked child should not open a graphics device");
+
     /* Decode the arguments */
     display = CHAR(STRING_ELT(CAR(args), 0)); args = CDR(args);
     width = asReal(CAR(args));	args = CDR(args);
