@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000, 2001 The R Development Core Team
+ *  Copyright (C) 2000-11 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  *  SYNOPSIS
  *
@@ -35,8 +35,20 @@
  *    conform to the IEEE 754 standard.
  */
 
+#include <config.h>
 #include "nmath.h"
 
+
+/*  nearbyint is C99, so all platforms should have it (and AFAIK, all do) */
+#ifdef HAVE_NEARBYINT
+# define R_rint nearbyint
+#elif defined(HAVE_RINT)
+# define R_rint rint
+#else
+# define R_rint private_rint
+extern double private_rint(double x);
+#endif
+
 /* Improvements by Martin Maechler, May 1997;
    further ones, Feb.2000:
    Replace  pow(x, (double)i) by  R_pow_di(x, i) {and use  int dig} */
@@ -59,17 +71,15 @@ double fprec(double x, double digits)
     double l10, pow10, sgn, p10, P10;
     int e10, e2, do_round, dig;
     /* Max.expon. of 10 (=308.2547) */
-    const double max10e = DBL_MAX_EXP * M_LOG10_2;
+    const static int max10e = DBL_MAX_EXP * M_LOG10_2;
 
-#ifdef IEEE_754
     if (ISNAN(x) || ISNAN(digits))
 	return x + digits;
     if (!R_FINITE(x)) return x;
     if (!R_FINITE(digits)) {
-	if(digits > 0) return x;
-	else return 0;
+	if(digits > 0.0) return x;
+	else digits = 1.0;
     }
-#endif
     if(x == 0) return x;
     dig = (int)floor(digits+0.5);
     if (dig > MAX_DIGITS) {
@@ -86,15 +96,18 @@ double fprec(double x, double digits)
     e10 = (int)(dig-1-floor(l10));
     if(fabs(l10) < max10e - 2) {
 	p10 = 1.0;
-	if(e10 > max10e) {
+	if(e10 > max10e) { /* numbers less than 10^(dig-1) * 1e-308 */
 	    p10 =  R_pow_di(10., e10-max10e);
 	    e10 = max10e;
-	} else if(e10 < - max10e) {
-	    p10 =  R_pow_di(10., e10+max10e);
-	    e10 = -max10e;	    
+	} 
+	if(e10 > 0) { /* Try always to have pow >= 1
+			 and so exactly representable */
+	    pow10 = R_pow_di(10., e10);
+	    return(sgn*(R_rint((x*pow10)*p10)/pow10)/p10);
+	} else {
+	    pow10 = R_pow_di(10., -e10);
+	    return(sgn*(R_rint((x/pow10))*pow10));
 	}
-	pow10 = R_pow_di(10., e10);
-	return(sgn*(floor((x*pow10)*p10+0.5)/pow10)/p10);
     } else { /* -- LARGE or small -- */
 	do_round = max10e - l10	 >= R_pow_di(10., -dig);
 	e2 = dig + ((e10>0)? 1 : -1) * MAX_DIGITS;
